package com.brilliance.swift.mt2mx;

import com.alibaba.fastjson.JSONArray;
import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.util.DateUtil;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.SwiftTransferUtil;
import com.brilliance.swift.vo.MxMtReasonCodeInfo;
import com.brilliance.swift.vo.SwiftTranslationErrorInfo;
import com.brilliance.swift.vo.SwiftTranslationReport;
import com.brilliance.swift.vo.VoSettlementMethodHelper;
import com.brilliance.swift.vo.common.*;
import com.prowidesoftware.swift.model.field.Field23E;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public abstract class AbstractMt2MxParseField implements Mt2MxParseField {

    protected Mt2MxContext context;

    public Mt2MxContext getContext() {
        return context;
    }

    public AbstractMt2MxParseField setContext(Mt2MxContext context) {
        this.context = context;
        return this;
    }

    protected Map<String, Object> jsonMaps = null;

    public void initJsonMaps() {
        jsonMaps = (Map<String, Object>) context.get(Mt2MxContextIdentifier.MT_TO_MX_JSON_MAP, true);
    }

    /**
     * 封装报文转换详细报告
     *
     * @param errorCode
     * @param location
     * @param originalValue
     * @return
     */
    @Deprecated
    protected void buildSTErrorInfo(String errorCode, String location, String originalValue) {
        SwiftTranslationReport str = context.get(SwiftTranslationReport.class);
        SwiftTranslationErrorInfo errorInfo = SwiftTransferUtil.getSwiftTranslationErrorInfoByErrorCode(errorCode);
        errorInfo.setLocation(location);
        errorInfo.setOriginalValue(originalValue);
        List<SwiftTranslationErrorInfo> errorInfos = str.getErrorInfos();
        boolean existFlag = false;
        for (int i = 0; i < errorInfos.size(); i++) {
            if (SwiftTransferUtil.compareStErrorInfo(errorInfo, errorInfos.get(i))) {
                existFlag = true;
                break;
            }
        }
        if (!existFlag) {
            str.addErrorInfo(errorInfo);
        }
    }

    /**
     * 封装报文转换详细报告
     *
     * @param id
     * @param location
     * @param originalValue
     * @return
     */
    protected void buildSTErrorInfo(long id, String location, String originalValue) {
        SwiftTranslationReport str = context.get(SwiftTranslationReport.class);
        SwiftTranslationErrorInfo errorInfo = SwiftTransferUtil.getSwiftTranslationErrorInfoById(id);
        if (errorInfo == null) return;
        errorInfo.setLocation(location);
        errorInfo.setOriginalValue(originalValue);
        List<SwiftTranslationErrorInfo> errorInfos = str.getErrorInfos();
        boolean existFlag = false;
        for (int i = 0; i < errorInfos.size(); i++) {
            if (SwiftTransferUtil.compareStErrorInfo(errorInfo, errorInfos.get(i))) {
                existFlag = true;
                break;
            }
        }
        if (!existFlag) {
            str.addErrorInfo(errorInfo);
        }
    }
    /**
     * MT 转 MX 转换函数开始
     */

    /**
     * 如果下一行是以continuePattern开始的
     * 此行不够指定长度需要后面加空格
     *
     * @param value
     * @param maxLength
     * @return
     */
    protected String refreshValue(String value, String continuePattern, int maxLength) {
        if (StringUtil.isEmpty(value)) {
            return "";
        }
        String regex = "(.*)\\\r\\\n" + continuePattern;
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(value);
        while (m.find()) {
            String findStr = m.group(1);
            if (findStr.length() < maxLength) {
                value = value.replace(findStr, findStr + " ");
            }
        }
        return value;
    }

    /**
     * 1.如果 mtAccount 以//CH开始,Id.Othr.SchmeNm.Cd = CUID,Id.Othr.Id=mtAccount.subString(5)
     * 2.如果 mtAccount 是IBAN账户，Id.IBAN = mtAccount
     * 3.否则 Id.Othr.Id=mtAccount
     *
     * @param mtAccount
     * @param maps(account对应的map对象)
     */
    protected void mt_to_mxPartyAccount(String mtAccount, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mtAccount)) {
            return;
        }
        if (mtAccount.startsWith("//CH") && mtAccount.length() > 4) {
            MapUtil.puts(maps, "id/othr/id", mtAccount.substring(4));
            MapUtil.puts(maps, "id/othr/schmeNm/cd", "CUID");
        } else if (mtAccount.length() > 1) {
            mtAccount = mtAccount.substring(1);
            if (SwiftTransferUtil.checkIbanAccount(mtAccount)) {
                MapUtil.puts(maps, "id/iban", mtAccount);
            } else {
                MapUtil.puts(maps, "id/othr/id", mtAccount);
            }
        }
    }

    /**
     * 1.如果 mtAccount 以//CH开始,Id.Othr.SchmeNm.Cd = CUID,Id.Othr.Id=mtAccount.subString(5)
     * 2.如果长度大于34，Id.Othr.Id=mtAccount截取33位，最后一位给+
     * 3.如果 mtAccount 是IBAN账户，Id.IBAN = mtAccount
     * 4.否则 Id.Othr.Id=mtAccount
     *
     * @param mtAccount
     * @param maps(account对应的map对象)
     */
    protected void mt_to_mxAccount35(String mtAccount, Map<String, Object> maps, String path) {
        if (StringUtil.isEmpty(mtAccount)) {
            return;
        }
        if (mtAccount.startsWith("//CH") && mtAccount.length() > 4) {
            MapUtil.puts(maps, "id/othr/id", mtAccount.substring(4));
            MapUtil.puts(maps, "id/othr/schmeNm/cd", "CUID");
        } else if (mtAccount.length() > 34) {
            buildSTErrorInfo(116, path, mtAccount);
            buildSTErrorInfo(12,path,mtAccount);
            MapUtil.puts(maps, "id/othr/id", mtAccount.substring(0, 33) + "+");
        } else {
            if (SwiftTransferUtil.checkIbanAccount(mtAccount)) {
                MapUtil.puts(maps, "id/iban", mtAccount);
            } else {
                MapUtil.puts(maps, "id/othr/id", mtAccount);
            }
        }
    }

    /**
     * Id.OrgId.AnyBIC = mtBicCode
     *
     * @param mtBicCode
     * @param maps
     */
    protected void mt_to_mxAnyBIC(String mtBicCode, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mtBicCode)) {
            return;
        }
        MapUtil.puts(maps, "id/orgId/anyBIC", mtBicCode);
    }

    /**
     * FinInstnId.BICFI = mtBicCode
     *
     * @param mtBicCode
     * @param maps
     */
    protected void mt_to_mxBICFI(String mtBicCode, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mtBicCode)) {
            return;
        }
        MapUtil.puts(maps, "finInstnId/bicfi", mtBicCode);
    }

    /**
     * nameAndAddress 通过回车换行切割成数组 nameAndAddress[]
     * Nm = nameAndAddress[0]
     * PstlAdr.AdrLine = 剩余的 nameAndAddress[]
     *
     * @param nameAndAddress
     * @param maps
     */
    protected void mt_to_mxPartyNameAndAddress(String nameAndAddress, Map<String, Object> maps) {
        if (StringUtil.isEmpty(nameAndAddress)) {
            return;
        }
        String[] nameAndAddresses = nameAndAddress.split(Mx2MtConstants.NEW_LINE);
        String name = "";
        List<String> addressList = new ArrayList<String>();
        for (int i = 0; i < nameAndAddresses.length; i++) {
            if (i == 0) {
                name = nameAndAddresses[i];
            } else {
                addressList.add(nameAndAddresses[i]);
            }
        }
        maps.put("nm", name);
        if (addressList.size() > 0) {
            for (int i = 0; i < addressList.size(); i++) {
                MapUtil.adds(maps, "pstlAdr/adrLine", addressList.get(i));
            }
        }
    }

    /**
     * 1/ 开头的给name
     * 2/ 开头的给adrLine
     * 3/[A-Z]{2}/LEIC/([A-Z0-9]{18,18}[0-9]{2,2}) 括号里面的内容给LEI
     * 否则 给 adrLine
     *
     * @param nameAndAddress
     * @param maps
     */
    protected void mt_to_mxPartyNameAndStructuredAddress(String nameAndAddress, Map<String, Object> maps) {
        if (StringUtil.isEmpty(nameAndAddress)) {
            return;
        }
        String[] nameAndAddresses = nameAndAddress.split(Mx2MtConstants.NEW_LINE);
        String name = "";
        String lei = "";
        List<String> addressList = new ArrayList<String>();
        for (int i = 0; i < nameAndAddresses.length; i++) {
            String tmpNameAndAddress = nameAndAddresses[i];
            if (tmpNameAndAddress.startsWith("1/")) {
                name += tmpNameAndAddress.substring(2);
            }
            if (tmpNameAndAddress.startsWith("2/")) {
                addressList.add(tmpNameAndAddress);
            }
            if (tmpNameAndAddress.startsWith("3/")) {
                String regex = "3/[A-Z]{2}/LEIC/([A-Z0-9]{18,18}[0-9]{2,2})";
                Pattern p = Pattern.compile(regex);
                Matcher m = p.matcher(tmpNameAndAddress);
                if (m.find()) {
                    /**
                     * group(0)就是指的整个串，group(1)指的是第一个括号里的东西，group(2)指的第二个括号里的东西
                     */
                    lei = m.group(1);
                } else {
                    addressList.add(tmpNameAndAddress);
                }
            }
        }
        if (name.length() > 0) {
            maps.put("nm", name);
        }
        if (lei.length() > 0) {
            MapUtil.puts(maps, "id/orgId/lei", lei);
        }
        if (addressList.size() > 0) {
            for (int i = 0; i < addressList.size(); i++) {
                MapUtil.adds(maps, "pstlAdr/adrLine", addressList.get(i));
            }
        }
    }

    /**
     * ① //RT//([A-Z]{5})(.*)
     * ② //([A-Z]{5})(.*)
     * ③ //RT//([A-Z]{2})(.*)
     * ④ //([A-Z]{2})(.*)
     * 第一个括号内容给mxSystemCode
     * 第二个括号内容给mxMemberId
     *
     * @param mtPartyIdentifier
     * @param maps
     */
    protected void mt_to_mxClearingIdentifier(String mtPartyIdentifier, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mtPartyIdentifier)) {
            return;
        }
        String mxClearingSystem = "";
        String mxClearingCode = "";
        String regex1 = "//RT//([A-Z]{5})(.*)";
        Pattern p1 = Pattern.compile(regex1);
        Matcher m1 = p1.matcher(mtPartyIdentifier);
        if (m1.find()) {
            mxClearingSystem = m1.group(1);
            mxClearingCode = m1.group(2);
        }
        String regex2 = "//([A-Z]{5})(.*)";
        Pattern p2 = Pattern.compile(regex2);
        Matcher m2 = p2.matcher(mtPartyIdentifier);
        if (m2.find()) {
            mxClearingSystem = m2.group(1);
            mxClearingCode = m2.group(2);
        }
        ClearingSystemMemberCode code = SwiftTransferUtil.getClsSysMemberByValue(mxClearingSystem);
        if (mxClearingCode.length() > 0 && code != null) {
            maps.put("mmbId", mxClearingCode);
            MapUtil.puts(maps, "clrSysId/cd", mxClearingSystem);
            return;
        }
        /**
         * 第2种情况
         */
        String mtClearingSystem = "";
        String mtClearingCode = "";
        String regex3 = "//RT//([A-Z]{2})(.*)";
        Pattern p3 = Pattern.compile(regex3);
        Matcher m3 = p3.matcher(mtPartyIdentifier);
        if (m3.find()) {
            mtClearingSystem = m3.group(1);
            mtClearingCode = m3.group(2);
        }else {
            String regex4 = "//([A-Z]{2})(.*)";
            Pattern p4 = Pattern.compile(regex4);
            Matcher m4 = p4.matcher(mtPartyIdentifier);
            if (m4.find()) {
                mtClearingSystem = m4.group(1);
                mtClearingCode = m4.group(2);
            }
        }
        code = SwiftTransferUtil.getClsSysMemberByDesc(mtClearingSystem);
        if (mtClearingCode.length() > 0 && code != null) {
            if ("SW".equals(mtClearingSystem)) {
                if (mtClearingCode.length() > 5) {
                    mxClearingSystem = ClearingSystemMemberCode.CHSIC.value();
                } else {
                    mxClearingSystem = ClearingSystemMemberCode.CHBCC.value();
                }
            } else {
                mxClearingSystem = code.value();
            }
            mxClearingCode = mtClearingCode;
            maps.put("mmbId", mxClearingCode);
            MapUtil.puts(maps, "clrSysId/cd", mxClearingSystem);
        }
    }

    /**
     * 判断 mt systemcode 在不在 MTClearingSystemCodeList 里面
     * 存在返回true
     * 不在返回false
     *
     * @param mtPartyIdentifier
     */
    protected boolean isMTClearingSystemCodeInList(String mtPartyIdentifier) {
        if (StringUtil.isEmpty(mtPartyIdentifier)) {
            return false;
        }
        boolean flag = false;
        String mxClearingSystem = "";
        String regex1 = "//RT//([A-Z]{5})(.*)";
        Pattern p1 = Pattern.compile(regex1);
        Matcher m1 = p1.matcher(mtPartyIdentifier);
        if (m1.find()) {
            mxClearingSystem = m1.group(1);
        }
        String regex2 = "//([A-Z]{5})(.*)";
        Pattern p2 = Pattern.compile(regex2);
        Matcher m2 = p2.matcher(mtPartyIdentifier);
        if (m2.find()) {
            mxClearingSystem = m2.group(1);
        }
        ClearingSystemMemberCode code = SwiftTransferUtil.getClsSysMemberByValue(mxClearingSystem);
        if (code != null) {
            flag = true;
        } else {
            /**
             * 第2种情况
             */
            String mtClearingSystem = "";
            String regex3 = "//RT//([A-Z]{2})(.*)";
            Pattern p3 = Pattern.compile(regex3);
            Matcher m3 = p3.matcher(mtPartyIdentifier);
            if (m3.find()) {
                mtClearingSystem = m3.group(1);
            }else{
                String regex4 = "//([A-Z]{2})(.*)";
                Pattern p4 = Pattern.compile(regex4);
                Matcher m4 = p4.matcher(mtPartyIdentifier);
                if (m4.find()) {
                    mtClearingSystem = m4.group(1);
                }
            }
            code = SwiftTransferUtil.getClsSysMemberByDesc(mtClearingSystem);
            if (code != null) {
                flag = true;
            }
        }
        return flag;
    }

    protected void mt_to_mxClearingSystemToNameAndAddressLine(String mtPartyIdentifier, String location, Map<String, Object> maps, String path) {
        if (StringUtil.isEmpty(mtPartyIdentifier)) {
            return;
        }
        String nameAddress = "";
        String regex1 = "//RT//(.*)";
        Pattern p1 = Pattern.compile(regex1);
        Matcher m1 = p1.matcher(mtPartyIdentifier);
        if (m1.find()) {
            nameAddress = m1.group(1);
        }
        String regex2 = "//(.*)";
        Pattern p2 = Pattern.compile(regex2);
        Matcher m2 = p2.matcher(mtPartyIdentifier);
        if (m2.find()) {
            nameAddress = m2.group(1);
        }
        if (StringUtil.isNotEmpty(nameAddress) && !"RT".equals(nameAddress)) {
            nameAddress = "//" + nameAddress;
            buildSTErrorInfo(15, path, null);
        } else {
            buildSTErrorInfo(16, path, null);
            nameAddress = Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE;
        }
        maps.put("nm", nameAddress);
        List<String> addressList = new ArrayList<String>();
        addressList.add(nameAddress);
        if (StringUtil.isNotEmpty(location)) {
            addressList.add(location);
        }
        for (int i = 0; i < addressList.size(); i++) {
            MapUtil.adds(maps, "pstlAdr/adrLine", addressList.get(i));
        }
    }

    protected void mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(String nameAddress, Map<String, Object> maps) {
        if (StringUtil.isEmpty(nameAddress)) {
            return;
        }
        Map<String, Object> finInstnIdMaps = null;
        if (maps.containsKey("finInstnId")) {
            finInstnIdMaps = (Map<String, Object>) maps.get("finInstnId");
        } else {
            finInstnIdMaps = new HashMap<String, Object>();
            maps.put("finInstnId", finInstnIdMaps);
        }
        String[] nameAddresses = nameAddress.split(Mx2MtConstants.NEW_LINE);
        String name = "";
        List<String> addressList = new ArrayList<String>();
        for (int i = 0; i < nameAddresses.length; i++) {
            if (i == 0) {
                name = nameAddresses[i];
            } else {
                addressList.add(nameAddresses[i]);
            }
        }
        finInstnIdMaps.put("nm", name);
        if (addressList.size() == 0) {
            MapUtil.adds(maps, "finInstnId/pstlAdr/adrLine", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
        }
        for (int i = 0; i < addressList.size(); i++) {
            MapUtil.adds(maps, "finInstnId/pstlAdr/adrLine", addressList.get(i));
        }
    }

    /**
     * @param mt72
     * @param maps
     */
    protected void mt72_to_mxNextAgent(String mt72, List<String> excludeCodeList, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mt72)) {
            return;
        }
        String mt72Value = refreshValue(mt72, "//", 35).replace("\r\n//", "");
        String[] lines = mt72Value.split("\r\n");
        String nextAgtInfo = "";
        for (int i = 0; i < lines.length; i++) {
            boolean found = false;
            for (String excludeCode : excludeCodeList) {
                if (lines[i].startsWith(excludeCode)) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                nextAgtInfo += lines[i];
            }
        }
        VoSettlementMethodHelper setlMtdHelper = context.get(VoSettlementMethodHelper.class);
        String nextAgeFin53 = setlMtdHelper.getInstructionForNextAgent();
        if (StringUtil.isNotEmpty(nextAgeFin53)) {
            nextAgtInfo = nextAgeFin53 + nextAgtInfo;
        }
        if (StringUtil.isNotEmpty(nextAgtInfo)) {
            if (nextAgtInfo.length() > 210) {
                buildSTErrorInfo(12, "InstrForNxtAgt/InstrInf", nextAgtInfo);
                nextAgtInfo = nextAgtInfo.substring(0, 209) + "+";
            }
            JSONArray instrForNxtAgtJsonArray = null;
            if (maps.containsKey("instrForNxtAgt")) {
                instrForNxtAgtJsonArray = (JSONArray) maps.get("instrForNxtAgt");
            } else {
                instrForNxtAgtJsonArray = new JSONArray();
                maps.put("instrForNxtAgt", instrForNxtAgtJsonArray);
            }
            List<String> nextAgtInfoList = StringUtil.outStringList(nextAgtInfo, 35, "");
            for (int i = 0; i < nextAgtInfoList.size(); i++) {
                Map<String, Object> instrForNxtAgtMaps = new HashMap<String, Object>();
                instrForNxtAgtMaps.put("instrInf", nextAgtInfoList.get(i));
                instrForNxtAgtJsonArray.add(instrForNxtAgtMaps);
            }
        }
    }

    /**
     * @param mt72
     * @param maps
     */
    protected void mt72ins_to_mxAgent(String mt72, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mt72)) {
            return;
        }
        boolean INSCode = false;
        String mt72Value = refreshValue(mt72, "//", 35).replace("\r\n//", "");
        String[] lines = mt72Value.split("\r\n");
        List<String> insList = new ArrayList<String>();
        for (int i = 0; i < lines.length; i++) {
            if (lines[i].startsWith("/INS/")) {
                insList.add(lines[i]);
            }
        }
        for (int i = 0; i < insList.size(); i++) {
            String insLine = insList.get(i);
            insLine = insLine.substring(5);
            if(i==2&&insLine.length()>0){
                INSCode = true;
            }
            if(i>=3){
                buildSTErrorInfo(7, "Block4/:72:", null);
            }else {
                String key = "prvsInstgAgt" + (i + 1);
                if (SwiftTransferUtil.isBIC(insLine)) {
                    MapUtil.puts(maps, key+"/finInstnId/bicfi", insLine);
                } else {
                    if (insLine.length() > 140) {
                        buildSTErrorInfo(12, "PrvsInstgAgt" + (i + 1) + "/FinInstnId/Nm", insLine);
                        insLine = insLine.substring(0, 139) + "+";
                    }
                    MapUtil.puts(maps, key+"/finInstnId/nm", insLine);
                    MapUtil.adds(maps, key+"/finInstnId/pstlAdr/adrLine", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                }
            }
        }
    }

    /**
     * @param mtPartyIdentifier
     * @param maps
     */
    protected void mt_to_mxFinancialInstitutionAccount(String mtPartyIdentifier, Map<String, Object> maps, String path) {
        if (StringUtil.isEmpty(mtPartyIdentifier)) {
            return;
        }
        boolean CUIDIndicator = false;
        if (mtPartyIdentifier.startsWith("//CH") && mtPartyIdentifier.length() > 4) {
            CUIDIndicator = true;
            mtPartyIdentifier = mtPartyIdentifier.substring(4);
            MapUtil.puts(maps, "id/othr/id", mtPartyIdentifier);
        } else if ((mtPartyIdentifier.startsWith("/C/") || mtPartyIdentifier.startsWith("/D/")) && mtPartyIdentifier.length() > 3) {
            buildSTErrorInfo(21, path, null);
            mtPartyIdentifier = mtPartyIdentifier.substring(3);
        } else if((mtPartyIdentifier.startsWith("/C") || mtPartyIdentifier.startsWith("/D")) && mtPartyIdentifier.length() == 2){
            buildSTErrorInfo(21, path, null);
            mtPartyIdentifier = "";
        } else if (mtPartyIdentifier.length() > 1) {
            mtPartyIdentifier = mtPartyIdentifier.substring(1);
        }
        if (SwiftTransferUtil.checkIbanAccount(mtPartyIdentifier)) {
            MapUtil.puts(maps, "id/iban", mtPartyIdentifier);
        } else {
            if(mtPartyIdentifier.length()>0){
                MapUtil.puts(maps, "id/othr/id", mtPartyIdentifier);
                if(CUIDIndicator){
                    MapUtil.puts(maps, "id/othr/schmeNm/cd", "CUID");
                }
            }
        }
    }

    protected void mt_to_mxRemittanceInformation(String mt70, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mt70)) {
            return;
        }
        String[] mt70Lines = mt70.split(Mx2MtConstants.NEW_LINE);
        String ustrd = "";
        for (int i = 0; i < mt70Lines.length; i++) {
            String mt70Line = mt70Lines[i];
            if (mt70Line.length() < 35 && i < (mt70Lines.length - 1)) {
                ustrd += mt70Line + " ";
            } else {
                ustrd += mt70Line;
            }
        }
        List<String> ustrdList = new ArrayList<String>();
        ustrdList.add(ustrd);
        maps.put("ustrd", ustrdList);
    }

    /**
     * 获取 EndToEndId
     *
     * @param mt70Value
     * @return
     */
    protected String mt70roc_to_mx35Text(String mt70Value) {
        if (StringUtil.isEmpty(mt70Value)) {
            return null;
        }
        String endToEndId = "";
        String tmpMtValue = mt70Value.replace(Mx2MtConstants.NEW_LINE, "");
        String regex = "/ROC/(.*)///";
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(tmpMtValue);
        if (m.find()) {
            endToEndId = m.group(1);
        } else {
            regex = "/ROC/(.*)";
            p = Pattern.compile(regex);
            m = p.matcher(tmpMtValue);
            if (m.find()) {
                endToEndId = m.group(1);
            }
        }
        if (endToEndId.length() > 35) {
            buildSTErrorInfo(12, "PmtId/EndToEndId", endToEndId);
            endToEndId = endToEndId.substring(0, 34) + "+";
        }
        return endToEndId;
    }

    /**
     * 提取72域中指定mtcode的值
     *
     * @param mt72
     * @param mtCode
     * @param maxLength
     * @return
     */
    protected String mt72_to_mxText(String mt72, String mtCode, int maxLength) {
        if (StringUtil.isEmpty(mt72)) {
            return "";
        }
        String mxCode = "";
        String mt72Value = refreshValue(mt72, "//", 35).replace("\r\n//", "");
        String[] lines = mt72Value.split("\r\n");
        for (int i = 0; i < lines.length; i++) {
            if (lines[i].startsWith(mtCode)) {
                mxCode = lines[i];
            }
        }
        if (StringUtil.isNotEmpty(mxCode)) {
            mxCode = mxCode.substring(mtCode.length());
            if (mxCode.length() > maxLength) {
                mxCode = mxCode.substring(0, maxLength - 1) + "+";
            }
        }
        return mxCode;
    }

    /**
     * 23E 72 转换成 Instruction For Creditor Agent
     *
     * @param field23EList (HOLD, PHOB, CHQB,TELB)
     * @param mt72
     * @param maps
     */
    protected void mt_to_mxInstructionForCreditorAgent(List<Field23E> field23EList, String mt72, Map<String, Object> maps) {
        if (field23EList.size() == 0 && StringUtil.isEmpty(mt72)) {
            return;
        }
        Map<String, Object> instrForCdtrAgt1 = new HashMap<String, Object>();
        Map<String, Object> instrForCdtrAgt2 = new HashMap<String, Object>();
        String mxText = mt72_to_mxText(mt72, "/ACC/", 205);
        String configIndicator = "";
        if(mxText.length()>0){
            if(field23EList.size()>0){
                configIndicator = "ACCCode";
            }else {
                configIndicator = "ACCOnly";
            }
        }else {
            if(field23EList.size()>0){
                configIndicator = "CodeOnly";
            }else {
                return;
            }
        }
        if ("ACCOnly".equals(configIndicator)) {
            if (mxText.length() > 140) {
                instrForCdtrAgt1.put("instrInf", mxText.substring(0, 140));
                instrForCdtrAgt2.put("instrInf", mxText.substring(140));
            } else {
                instrForCdtrAgt1.put("instrInf", mxText);
            }
        } else if ("ACCCode".equals(configIndicator)) {
            if (mxText.length() > 140) {
                buildSTErrorInfo(12, "InstrForCdtrAgt/InstrInf", mxText);
                instrForCdtrAgt1.put("instrInf", mxText.substring(0, 139) + "+");
            } else {
                instrForCdtrAgt1.put("instrInf", mxText);
            }
            String instructionCode = "";
            String instructionInfo = "";
            for (int i = 0; i < field23EList.size(); i++) {
                Field23E field23E = field23EList.get(i);
                String code = field23E.getCode();
                String info = field23E.getAdditionalInformation();
                if (i == 0) {
                    instructionCode = code;
                    if (StringUtil.isNotEmpty(info)) {
                        instructionInfo += info;
                    }
                } else {
                    instructionInfo += "/" + code + "/";
                    if (StringUtil.isNotEmpty(info)) {
                        instructionInfo += info;
                    }
                }
            }
            instrForCdtrAgt2.put("cd", instructionCode);
            if (StringUtil.isNotEmpty(instructionInfo)) {
                instrForCdtrAgt2.put("instrInf", instructionInfo);
            }
        } else if ("CodeOnly".equals(configIndicator)) {
            String instructionCode = "";
            String instructionInfo = "";
            for (int i = 0; i < field23EList.size(); i++) {
                Field23E field23E = field23EList.get(i);
                String code = field23E.getCode();
                String info = field23E.getAdditionalInformation();
                if (i == 0) {
                    instrForCdtrAgt1.put("cd", code);
                    if (StringUtil.isNotEmpty(info)) {
                        instrForCdtrAgt1.put("instrInf", info);
                    }
                } else if (i == 1) {
                    instructionCode = code;
                    if (StringUtil.isNotEmpty(info)) {
                        instructionInfo += info;
                    }
                } else {
                    instructionInfo += "/" + code + "/";
                    if (StringUtil.isNotEmpty(info)) {
                        instructionInfo += info;
                    }
                }
            }
            if (StringUtil.isNotEmpty(instructionCode)) {
                instrForCdtrAgt2.put("cd", instructionCode);
            }
            if (StringUtil.isNotEmpty(instructionInfo)) {
                instrForCdtrAgt2.put("instrInf", instructionInfo);
            }
        }
        JSONArray instrForCdtrAgtJsonArray = null;
        if (maps.containsKey("instrForCdtrAgt")) {
            instrForCdtrAgtJsonArray = (JSONArray) maps.get("instrForCdtrAgt");
        } else {
            instrForCdtrAgtJsonArray = new JSONArray();
            maps.put("instrForCdtrAgt", instrForCdtrAgtJsonArray);
        }
        if (instrForCdtrAgt1.size() > 0) {
            instrForCdtrAgtJsonArray.add(instrForCdtrAgt1);
        }
        if (instrForCdtrAgt2.size() > 0) {
            instrForCdtrAgtJsonArray.add(instrForCdtrAgt2);
        }
    }


    protected void mt_to_mxInstructionForCreditorAgent2(String mt72, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mt72)) {
            return;
        }
        String mxUdlcText = mt72_to_mxText(mt72, "/UDLC/", 204);
        String mxAccText = mt72_to_mxText(mt72, "/ACC/", 205);
        String mxPhobText = mt72_to_mxText(mt72, "/PHONBEN/", 140);
        String mxTelbText = mt72_to_mxText(mt72, "/TELEBEN/", 140);
        Map<String, Object> instrForCdtrAgt1 = new HashMap<String, Object>();
        Map<String, Object> instrForCdtrAgt2 = new HashMap<String, Object>();
        String configIndicator = "";
        if (StringUtil.isEmpty(mxUdlcText)) {  //不包含UDLC
            if (StringUtil.isEmpty(mxAccText)) { //不包含ACC
                if (StringUtil.isNotEmpty(mxPhobText) || StringUtil.isNotEmpty(mxTelbText)) {
                    configIndicator = "CodeOnly";
                } else {
                    return;
                }
            } else { // 包含ACC
                if (StringUtil.isNotEmpty(mxPhobText) || StringUtil.isNotEmpty(mxTelbText)) {
                    configIndicator = "ACCCode";
                } else {
                    configIndicator = "ACCOnly";
                }
            }
        } else {
            if (StringUtil.isEmpty(mxAccText)) {
                configIndicator = "UDLC";
            } else {
                configIndicator = "UDLCACC";
            }
        }
        if ("ACCOnly".equals(configIndicator)) {
            if (mxAccText.length() > 140) {
                instrForCdtrAgt1.put("instrInf", mxAccText.substring(0, 140));
                instrForCdtrAgt2.put("instrInf", mxAccText.substring(140));
            } else {
                instrForCdtrAgt1.put("instrInf", mxAccText);
            }
        } else if ("ACCCode".equals(configIndicator)) {
            if (mxAccText.length() > 140) {
                buildSTErrorInfo(12, "InstrForCdtrAgt/InstrInf", mxAccText);
                instrForCdtrAgt1.put("instrInf", mxAccText.substring(0, 139) + "+");
            } else {
                instrForCdtrAgt1.put("instrInf", mxAccText);
            }
            if (StringUtil.isNotEmpty(mxPhobText)) {
                String instructionInfo = mxPhobText;
                int remainLength = 140 - instructionInfo.length();
                if (StringUtil.isNotEmpty(mxTelbText) && remainLength >= 7) {
                    String tmpInstructionInfo = "/TELB/" + mxTelbText;
                    if (tmpInstructionInfo.length() > remainLength) {
                        buildSTErrorInfo(12, "InstrForCdtrAgt/InstrInf", tmpInstructionInfo);
                        tmpInstructionInfo = tmpInstructionInfo.substring(0, remainLength - 1) + "+";
                    }
                    instructionInfo += tmpInstructionInfo;
                }
                instrForCdtrAgt2.put("cd", "PHOB");
                instrForCdtrAgt2.put("instrInf", instructionInfo);
            } else { // /PHONBEN/不存在,/TELEBEN/必须存在
                instrForCdtrAgt2.put("cd", "TELB");
                instrForCdtrAgt2.put("instrInf", mxTelbText);
            }
        } else if ("CodeOnly".equals(configIndicator)) {
            if (StringUtil.isNotEmpty(mxPhobText)) {
                instrForCdtrAgt1.put("cd", "PHOB");
                instrForCdtrAgt1.put("instrInf", mxPhobText);
            }
            if (StringUtil.isNotEmpty(mxTelbText)) {
                if (instrForCdtrAgt1.size() == 0) {
                    instrForCdtrAgt1.put("cd", "TELB");
                    instrForCdtrAgt1.put("instrInf", mxTelbText);
                } else {
                    instrForCdtrAgt2.put("cd", "TELB");
                    instrForCdtrAgt2.put("instrInf", mxTelbText);
                }
            }
        } else if ("UDLCACC".equals(configIndicator)) {
            if (mxAccText.length() > 140) {
                buildSTErrorInfo(12, "InstrForCdtrAgt/InstrInf", mxAccText);
                instrForCdtrAgt1.put("instrInf", mxAccText.substring(0, 139) + "+");
            } else {
                instrForCdtrAgt1.put("instrInf", mxAccText);
            }
            mxUdlcText = "/UDLC/" + mxUdlcText;
            if (mxUdlcText.length() > 140) {
                buildSTErrorInfo(12, "InstrForCdtrAgt/InstrInf", mxUdlcText);
                instrForCdtrAgt2.put("instrInf", mxUdlcText.substring(0, 139) + "+");
            } else {
                instrForCdtrAgt2.put("instrInf", mxUdlcText);
            }
        } else if ("UDLC".equals(configIndicator)) {
            mxUdlcText = "/UDLC/" + mxUdlcText;
            if (mxUdlcText.length() > 140) {
                buildSTErrorInfo(12, "InstrForCdtrAgt/InstrInf", mxUdlcText);
                mxUdlcText = mxUdlcText.substring(0, 139) + "+";
            }
            if (StringUtil.isNotEmpty(mxPhobText) || StringUtil.isNotEmpty(mxTelbText)) {
                if (StringUtil.isNotEmpty(mxPhobText)) {
                    String instructionInfo = mxPhobText;
                    int remainLength = 140 - instructionInfo.length();
                    if (StringUtil.isNotEmpty(mxTelbText) && remainLength >= 7) {
                        String tmpInstructionInfo = "/TELB/" + mxTelbText;
                        if (tmpInstructionInfo.length() > remainLength) {
                            buildSTErrorInfo(12, "InstrForCdtrAgt/InstrInf", tmpInstructionInfo);
                            tmpInstructionInfo = tmpInstructionInfo.substring(0, remainLength - 1) + "+";
                        }
                        instructionInfo += tmpInstructionInfo;
                    }
                    instrForCdtrAgt1.put("cd", "PHOB");
                    instrForCdtrAgt1.put("instrInf", instructionInfo);
                } else { // /PHONBEN/不存在,/TELEBEN/必须存在
                    instrForCdtrAgt1.put("cd", "TELB");
                    instrForCdtrAgt1.put("instrInf", mxTelbText);
                }
                instrForCdtrAgt2.put("instrInf", mxUdlcText);
            } else {
                instrForCdtrAgt1.put("instrInf", mxUdlcText);
            }
        }
        JSONArray instrForCdtrAgtJsonArray = null;
        if (maps.containsKey("instrForCdtrAgt")) {
            instrForCdtrAgtJsonArray = (JSONArray) maps.get("instrForCdtrAgt");
        } else {
            instrForCdtrAgtJsonArray = new JSONArray();
            maps.put("instrForCdtrAgt", instrForCdtrAgtJsonArray);
        }
        if (instrForCdtrAgt1.size() > 0) {
            instrForCdtrAgtJsonArray.add(instrForCdtrAgt1);
        }
        if (instrForCdtrAgt2.size() > 0) {
            instrForCdtrAgtJsonArray.add(instrForCdtrAgt2);
        }
    }

    protected void mt_to_mxBNF_TSU_RI(String mt72, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mt72)) {
            return;
        }
        String mxBnfText = mt72_to_mxText(mt72, "/BNF/", 135);
        int remainLength = 140; //初始长度140
        String mxUnstructuredRemittanceInformation = "";
        if (StringUtil.isNotEmpty(mxBnfText)) {
            mxUnstructuredRemittanceInformation = "/BNF/" + mxBnfText;
            remainLength = 140 - mxUnstructuredRemittanceInformation.length();
        }
        String mxTsuText = mt72_to_mxText(mt72, "/TSU/", 135);
        if (StringUtil.isNotEmpty(mxTsuText) && remainLength >= 6) {
            mxTsuText = "/TSU/" + mxTsuText;
            if (mxTsuText.length() > remainLength) {
                buildSTErrorInfo(12, "RmtInf/Ustrd", mxTsuText);
                mxUnstructuredRemittanceInformation += mxTsuText.substring(0, remainLength - 1) + "+";
            }
        }
        if (StringUtil.isNotEmpty(mxUnstructuredRemittanceInformation)) {
            MapUtil.adds(maps, "rmtInf/ustrd", mxUnstructuredRemittanceInformation);
        }
    }

    /**
     * 前提条件是 mtPartyIdentifier 不以/开始
     *
     * @param mtPartyIdentifier
     * @param nameAddress
     * @param maps
     */
    protected void mt_to_mxFATFIdentification(String mtPartyIdentifier, String nameAddress, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mtPartyIdentifier)) {
            return;
        }
        String mtCode8 = "";
        if (StringUtil.isNotEmpty(nameAddress)) {
            String[] nameAndAddresses = nameAddress.split(Mx2MtConstants.NEW_LINE);
            for (int i = 0; i < nameAndAddresses.length; i++) {
                if (nameAndAddresses[i].startsWith("8/")) {
                    mtCode8 = nameAndAddresses[i].substring(2);
                }
            }
        }
        String mtFullId = mtPartyIdentifier;
        if (mtPartyIdentifier.length() == 35) {
            mtFullId += mtCode8;
        }
        //格式类似 mtSchemeCode + "/" + mtCountryCode + "/" + mtIssuer + "/" + mtIdentifier
        String mtFATFCode = mtFullId.substring(0, 4);
        String mtFATFId = mtFullId.substring(5);
        String mtCountry = "";
        String mtIssuer = "";
        String mtId = "";
        boolean flagOrgId = false;
        boolean flagPrivID = false;
        String mxOrgSchemeCode = "";
        String mXPrivSchemeCode= "";
        if ("CUST".equals(mtFATFCode) || "EMPL".equals(mtFATFCode) || "DRLC".equals(mtFATFCode)) {
            mtCountry = mtFATFId.substring(0, 2);
            String regex = "(.*)/(.*)";
            Pattern p = Pattern.compile(regex);
            Matcher m = p.matcher(mtFATFId.substring(3));
            if (m.find()) {
                mtIssuer = m.group(1);
                mtId = m.group(2);
                if (StringUtil.isEmpty(mtId)) {
                    mtId = Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE;
                }
                if (mtIssuer.length() > 4 && "CUST".equals(mtFATFCode)) {
                    OrganisationIdentificationCode code = SwiftTransferUtil.getOrganisationIdentificationCodeByCode(mtIssuer.substring(0, 4));
                    if (code != null
                            && !OrganisationIdentificationCode.CUST.equals(code)
                            && !OrganisationIdentificationCode.TXID.equals(code)
                            && !OrganisationIdentificationCode.EMPL.equals(code)) {
                        mxOrgSchemeCode = code.value();
                        mtIssuer = mtIssuer.substring(5);
                        flagOrgId = true;
                    }
                }
                if (mtIssuer.length() > 4 && "CUST".equals(mtFATFCode)) {
                    PersonIdentificationCode code = SwiftTransferUtil.getPersonIdentificationCodeByCode(mtIssuer.substring(0, 4));
                    if (code != null
                            && !PersonIdentificationCode.CUST.equals(code)
                            && !PersonIdentificationCode.TXID.equals(code)
                            && !PersonIdentificationCode.EMPL.equals(code)) {
                        mXPrivSchemeCode = code.value();
                        mtIssuer = mtIssuer.substring(5);
                        flagPrivID = true;
                    }
                }
            } else {
                mtId = mtFATFId.substring(3);
            }
            if (mtId.length() > 35) mtId = mtId.substring(0, 34) + "+";
        } else { //MTFATFCode 不在 {EMPL, CUST, DRLC}
            mtCountry = mtFATFId.substring(0, 2);
            mtId = mtFATFId.substring(3);
            if (StringUtil.isEmpty(mtId)) {
                mtId = Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE;
            }
            if (mtId.length() > 35) mtId = mtId.substring(0, 34) + "+";
        }
        String mxIssuer = mtCountry;
        if (StringUtil.isNotEmpty(mtIssuer)) {
            mxIssuer += "/" + mtIssuer;
        }
        //开始转换
        Map<String, Object> orgIdMaps = null;
        if (maps.containsKey("orgId")) {
            orgIdMaps = (Map<String, Object>) maps.get("orgId");
        } else {
            orgIdMaps = new HashMap<String, Object>();
            //如果orgIdMaps 长度大于0,put到maps
        }
        Map<String, Object> prvtIdMaps = null;
        if (maps.containsKey("prvtId")) {
            prvtIdMaps = (Map<String, Object>) maps.get("prvtId");
        } else {
            prvtIdMaps = new HashMap<String, Object>();
            //如果prvtIdMaps 长度大于0,put到maps
        }
        if (PersonIdentificationCode.ARNU.value().equals(mtFATFCode)
                || PersonIdentificationCode.CCPT.value().equals(mtFATFCode)
                || PersonIdentificationCode.DRLC.value().equals(mtFATFCode)
                || PersonIdentificationCode.EMPL.value().equals(mtFATFCode)
                || PersonIdentificationCode.NIDN.value().equals(mtFATFCode)
                || PersonIdentificationCode.SOSE.value().equals(mtFATFCode)
                || PersonIdentificationCode.TXID.value().equals(mtFATFCode)
                || (PersonIdentificationCode.CUST.value().equals(mtFATFCode) && !flagOrgId&&!flagPrivID)) {
            Map<String, Object> othrMaps = new HashMap<String, Object>();
            MapUtil.adds(prvtIdMaps, "othr", othrMaps);
            MapUtil.puts(othrMaps, "schmeNm/cd", mtFATFCode);
            othrMaps.put("id", mtId);
            othrMaps.put("issr", mxIssuer);
        } else if (PersonIdentificationCode.CUST.value().equals(mtFATFCode) && flagOrgId) {
            Map<String, Object> othrMaps = new HashMap<String, Object>();
            MapUtil.adds(orgIdMaps, "othr", othrMaps);
            MapUtil.puts(othrMaps, "schmeNm/cd", mxOrgSchemeCode);
            othrMaps.put("id", mtId);
            othrMaps.put("issr", mxIssuer);
        }else if (PersonIdentificationCode.CUST.value().equals(mtFATFCode) && flagPrivID) {
            Map<String, Object> othrMaps = new HashMap<String, Object>();
            MapUtil.adds(prvtIdMaps, "othr", othrMaps);
            MapUtil.puts(othrMaps, "schmeNm/cd", mXPrivSchemeCode);
            othrMaps.put("id", mtId);
            othrMaps.put("issr", mxIssuer);
        }
        if (orgIdMaps.size() > 0) {
            maps.put("orgId", orgIdMaps);
        }
        if (prvtIdMaps.size() > 0) {
            maps.put("prvtId", prvtIdMaps);
        }
    }

    /**
     * @param mtPartyIdentifier
     * @param nameAddress
     * @param maps
     */
    protected void mt_to_mxFATFNameAndAddress(String mtPartyIdentifier, String nameAddress, Map<String, Object> maps) {
        mt_to_mxPartyNameAndStructuredAddress(nameAddress, maps);
        String mtCode4 = "";
        String mtCode5 = "";
        String mtCode6 = "";
        String mtCode7 = "";
        String mtCode8 = "";
        if (StringUtil.isNotEmpty(nameAddress)) {
            String[] nameAndAddresses = nameAddress.split(Mx2MtConstants.NEW_LINE);
            for (int i = 0; i < nameAndAddresses.length; i++) {
                if (nameAndAddresses[i].startsWith("4/")) {
                    mtCode4 = nameAndAddresses[i].substring(2);
                }
                if (nameAndAddresses[i].startsWith("5/")) {
                    mtCode5 = nameAndAddresses[i].substring(2);
                }
                if (nameAndAddresses[i].startsWith("6/")) {
                    mtCode6 = nameAndAddresses[i].substring(2);
                }
                if (nameAndAddresses[i].startsWith("7/")) {
                    mtCode7 = nameAndAddresses[i].substring(2);
                }
                if (nameAndAddresses[i].startsWith("8/")) {
                    mtCode8 = nameAndAddresses[i].substring(2);
                }
            }
        }
        if (StringUtil.isEmpty(mtCode4)
                && StringUtil.isEmpty(mtCode5)
                && StringUtil.isEmpty(mtCode6)
                && StringUtil.isEmpty(mtCode7)) {
            return;  //找不到mtcode4 5 6 7，终止转换
        }
        if (StringUtil.isNotEmpty(mtCode6)) {
            boolean flagOrgId = false;
            String regex = "[A-Z]{2}/LEIC/([A-Z0-9]{18,18}[0-9]{2,2})";
            String lei = "";
            Pattern p = Pattern.compile(regex);
            Matcher m = p.matcher(mtCode6);
            if (m.find()) {
                lei = m.group(1);
                if (mtPartyIdentifier.startsWith("/")) {
                    flagOrgId = true;
                } else {
                    String regex1 = "CUST/[A-Z]{2}/([A-Z0-9]{4})";
                    Pattern p1 = Pattern.compile(regex1);
                    Matcher m1 = p1.matcher(mtPartyIdentifier);
                    if (m1.find()) {
                        String orgIdCode = m1.group(1);
                        OrganisationIdentificationCode code = SwiftTransferUtil.getOrganisationIdentificationCodeByCode(orgIdCode);
                        if (code != null
                                && !OrganisationIdentificationCode.CUST.equals(code)
                                && !OrganisationIdentificationCode.TXID.equals(code)
                                && !OrganisationIdentificationCode.EMPL.equals(code)) {
                            flagOrgId = true;
                        }
                    }
                }
            }
            if (flagOrgId) {
                if(StringUtil.isNotEmpty(lei)){
                    MapUtil.puts(maps, "id/orgId/lei", lei);
                }
                return;  //组织信息转换了,私人信息将不会转换.
            }
        }
        if (StringUtil.isNotEmpty(mtCode4) && StringUtil.isNotEmpty(mtCode5)) {
            try {
                Date brithDate = DateUtil.parseDate(mtCode4, "yyyyMMdd");
                String brithDateStr = DateUtil.format(brithDate, "yyyy-MM-dd");
                MapUtil.puts(maps, "id/prvtId/dtAndPlcOfBirth/birthDt", brithDateStr);
                String regex = "([A-Z]{2})/(.*)";
                Pattern p = Pattern.compile(regex);
                Matcher m = p.matcher(mtCode5);
                if (m.find()) {
                    MapUtil.puts(maps, "id/prvtId/dtAndPlcOfBirth/ctryOfBirth", m.group(1));
                    MapUtil.puts(maps, "id/prvtId/dtAndPlcOfBirth/cityOfBirth", m.group(2));
                }
            } catch (ParseException e) {
                throw new SwiftException(e.getMessage());
            }
        }
        if (StringUtil.isNotEmpty(mtPartyIdentifier) && !mtPartyIdentifier.startsWith("CUST")) {
            if (StringUtil.isNotEmpty(mtCode6)) {
                String mtFullId = mtCode6;
                if ((mtPartyIdentifier.startsWith("/") || mtPartyIdentifier.length() < 35) && mtCode8.length() > 0 && mtCode6.length() == 33) {
                    mtFullId += mtCode8;
                }
                String regex = "(.*)/(.*)";
                Pattern p = Pattern.compile(regex);
                Matcher m = p.matcher(mtFullId.substring(3));
                String mtCountry = mtFullId.substring(0, 2);
                String mtIssuer = "";
                String mtNum = "";
                if (m.find()) {
                    mtIssuer = m.group(1);
                    mtNum = m.group(2);
                    if (StringUtil.isEmpty(mtNum)) {
                        mtNum = Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE;
                    }
                } else {
                    mtNum = mtFullId.substring(3);
                }
                if (mtNum.length() > 35) {
                    mtNum = mtNum.substring(0, 34) + "+";
                }
                Map<String, Object> othrMaps = new HashMap<String, Object>();
                MapUtil.adds(maps, "id/prvtId/othr", othrMaps);
                MapUtil.puts(othrMaps, "id", mtNum);
                MapUtil.puts(othrMaps, "schmeNm/cd", "CUST");
                if (StringUtil.isNotEmpty(mtIssuer)) {
                    mtCountry += "/" + mtIssuer;
                }
                othrMaps.put("issr", mtCountry);
            }
        }
        if (StringUtil.isNotEmpty(mtPartyIdentifier) && !mtPartyIdentifier.startsWith("NIDN")) {
            if (StringUtil.isNotEmpty(mtCode7)) {
                Map<String, Object> idMaps = null;
                Map<String, Object> prvtIdMaps = null;
                JSONArray othrJsonArray = null;
                if (maps.containsKey("id")) {
                    idMaps = (Map<String, Object>) maps.get("id");
                    if (idMaps.containsKey("prvtId")) {
                        prvtIdMaps = (Map<String, Object>) idMaps.get("prvtId");
                        if (prvtIdMaps.containsKey("othr")) {
                            othrJsonArray = (JSONArray) prvtIdMaps.get("othr");
                        }
                    }
                }
                if (othrJsonArray!=null&&othrJsonArray.size() >= 2) {
                    return;
                }
                Map<String, Object> othrMaps = new HashMap<String, Object>();
                MapUtil.adds(maps, "id/prvtId/othr", othrMaps);
                String mtFullId = mtCode7;
                if ((mtPartyIdentifier.startsWith("/") || mtPartyIdentifier.length() < 35) && mtCode8.length() > 0
                        && mtCode6.length() != 33 && mtCode7.length() == 33) {
                    mtFullId += mtCode8;
                }
                if (mtFullId.length() > 3 && mtFullId.indexOf("/") == 2) {
                    String mtNum = mtFullId.substring(3);
                    if (mtNum.length() > 35) {
                        mtNum = mtNum.substring(0, 34) + "+";
                    }
                    othrJsonArray.add(othrMaps);
                    othrMaps.put("id", mtNum);
                    MapUtil.puts(othrMaps, "schmeNm/cd", "NIDN");
                    othrMaps.put("issr", mtFullId.substring(0, 2));
                }
            }
        }
    }

    /**
     * /INTA/ -> MXIntermediaryAgent2, MXIntermediaryAgent3
     * /SVCLVL/ ->  Payment Type Information/Service Level
     * /LOCINS/ ->  Payment Type Information/Local Instrument
     * /CATPURP/ -> Payment Type Information/Category Purpose
     *
     * @param mt72
     * @param maps
     */
    protected void mt_to_mxField72NewCodeWords(String mt72, boolean pacs009001Flag, Map<String, Object> maps) {
        //长度不足不加空格
        String mt72Value = mt72.replace("\r\n//", "");
        String[] lines = mt72Value.split("\r\n");
        List<String> intermediaryAgentLines = new ArrayList<String>();
        List<String> serviceLevelLines = new ArrayList<String>();
        List<String> localInstrumentLines = new ArrayList<String>();
        List<String> categoryPurposeLines = new ArrayList<String>();
        List<String> purposeLines = new ArrayList<String>();
        //长度不足需要加空格
        String refreshMt72Value = refreshValue(mt72, "//", 35).replace("\r\n//", "");
        String[] refreshLines = refreshMt72Value.split("\r\n");
        for (int i = 0; i < refreshLines.length; i++) {
            if (refreshLines[i].startsWith("/INTA/")) {
                String line = refreshLines[i].substring(6);
                if (StringUtil.isNotEmpty(line)) {
                    intermediaryAgentLines.add(line);
                }
            }
        }
        for (int i = 0; i < lines.length; i++) {
            if (lines[i].startsWith("/SVCLVL/")) {
                String line = lines[i].substring(8);
                if (StringUtil.isNotEmpty(line)) {
                    serviceLevelLines.add(line);
                }
            }
            if (lines[i].startsWith("/LOCINS/")) {
                String line = lines[i].substring(8);
                if (StringUtil.isNotEmpty(line)) {
                    localInstrumentLines.add(line);
                }
            }
            if (lines[i].startsWith("/CATPURP/")) {
                String line = lines[i].substring(9);
                if (StringUtil.isNotEmpty(line)) {
                    categoryPurposeLines.add(line);
                }
            }
            if (lines[i].startsWith("/PURP/")) {
                String line = lines[i].substring(6);
                if (StringUtil.isNotEmpty(line)) {
                    purposeLines.add(line);
                }
            }
        }
        //开始转换
        if (intermediaryAgentLines.size() > 0) {
            for (int i = 0; i < intermediaryAgentLines.size(); i++) {
                if (i == 2) break; //最多转换2次 IntermediaryAgent2, IntermediaryAgent3
                String line = intermediaryAgentLines.get(i);
                Map<String, Object> intrmyAgtMaps = new HashMap<String, Object>();
                mt_to_mxIntermediaryAgent(line, intrmyAgtMaps, "intrmyAgt" + (i + 2));
                maps.put("intrmyAgt" + (i + 2), intrmyAgtMaps);
            }
        }
        if (serviceLevelLines.size() > 0) {
            for (int i = 0; i < serviceLevelLines.size(); i++) {
                Map<String, Object> svcLvlMaps = new HashMap<String, Object>();
                MapUtil.adds(maps, "pmtTpInf/svcLvl", svcLvlMaps);
                if (i >= 3) break; //Service Level 最多转换3次
                String line = serviceLevelLines.get(i);
                if (ServiceLevelCode.SDVA.value().equals(line)) {
                    continue;
                }
                if (line.matches("G00[1-9]{1}")) {
                    continue;
                }
                ServiceLevelCode code = SwiftTransferUtil.getServiceLevelByValue(line);
                if (code != null) {
                    svcLvlMaps.put("cd", code.value());
                } else {
                    if (line.length() > 35) {
                        buildSTErrorInfo(12, "PmtTpInf/SvcLvl/Prtry", line);
                        line = line.substring(0, 34) + "+";
                    }
                    if(StringUtil.isNotEmpty(line)){
                        svcLvlMaps.put("prtry", line);
                    }
                }
            }
        }
        //NOT IsInList {CRED,CRTS,SPAY, SPRI, SSTD}
        if (localInstrumentLines.size() > 0) {
            for (int i = 0; i < localInstrumentLines.size(); i++) {
                if (i == 1) break; //只取第一条
                String line = localInstrumentLines.get(i);
                if ("CRED".equals(line) || "CRTS".equals(line)
                        || "SPAY".equals(line) || "SPRI".equals(line) || "SSTD".equals(line)) {
                    continue;
                }
                LocalInstrumentCode code = SwiftTransferUtil.getLocalInstrumentByValue(line);
                if (code != null) {
                    MapUtil.puts(maps, "pmtTpInf/lclInstrm/cd", code.value());
                } else {
                    if (line.length() > 35) {
                        buildSTErrorInfo(12, "PmtTpInf/LclInstrm/Prtry", line);
                        line = line.substring(0, 34) + "+";
                    }
                    if(StringUtil.isNotEmpty(line)){
                        MapUtil.puts(maps, "pmtTpInf/lclInstrm/prtry", line);
                    }
                }
            }
        }
        if (categoryPurposeLines.size() > 0) {
            for (int i = 0; i < categoryPurposeLines.size(); i++) {
                if (i == 1) break; //只取第一条
                String line = categoryPurposeLines.get(i);
                if (CategoryPurposeCode.INTC.value().equals(line)
                        || CategoryPurposeCode.CORT.value().equals(line)) {
                    continue;
                }
                CategoryPurposeCode code = SwiftTransferUtil.getCategoryPurposeByValue(line);
                if (code != null) {
                    String cd = MapUtil.gets(maps, "pmtTpInf/ctgyPurp/cd");
                    if(StringUtil.isEmpty(cd)){
                        MapUtil.puts(maps, "pmtTpInf/ctgyPurp/cd", code.value());
                    }
                } else {
                    if (line.length() > 35) {
                        buildSTErrorInfo(12, "PmtTpInf/CtgyPurp/Prtry", line);
                        line = line.substring(0, 34) + "+";
                    }
                    if(StringUtil.isNotEmpty(line)){
                        MapUtil.puts(maps, "pmtTpInf/ctgyPurp/prtry", line);
                    }
                }
            }
        }
        if (purposeLines.size() > 0 && pacs009001Flag) {
            String purpCode = purposeLines.get(0);
            PurposeCode code = SwiftTransferUtil.getPurposeByValue(purpCode);
            if (code != null) {
                MapUtil.puts(maps, "purp/cd", purpCode);
            } else {
                if (purpCode.length() > 35) {
                    buildSTErrorInfo(12, "Purp/Prtry", purpCode);
                    purpCode = purpCode.substring(0, 34) + "+";
                }
                if(StringUtil.isNotEmpty(purpCode)){
                    MapUtil.puts(maps, "purp/prtry", purpCode);
                }
            }
        }
    }

    /**
     * @param mtAgent
     * @param maps
     */
    protected void mt_to_mxIntermediaryAgent(String mtAgent, Map<String, Object> maps, String path) {
        if (StringUtil.isEmpty(mtAgent)) {
            return;
        }
        if (SwiftTransferUtil.isBIC(mtAgent)) {
            mt_to_mxBICFI(mtAgent, maps);
            return;
        }
        String regex = "([A-Z]{5})(.*)";
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(mtAgent);
        if (m.find()) {
            String mxCode = m.group(1);
            String mxMemId = m.group(2);
            if (SwiftTransferUtil.getClsSysMemberByValue(mxCode) != null) {
                if (mxMemId.length() > 28) {
                    buildSTErrorInfo(12, path + "/FinInstnId/ClrSysMmbId/MmbId", mxMemId);
                    mxMemId = mxMemId.substring(0, 27) + "+";
                }
                if(StringUtil.isNotEmpty(mxMemId)){
                    MapUtil.puts(maps, "finInstnId/clrSysMmbId/mmbId", mxMemId);
                }
                MapUtil.puts(maps, "finInstnId/clrSysMmbId/clrSysId/cd", mxCode);
                return;
            }
        }
        String mxName = "";
        String mxCountryCode = "";
        String mxTownName = "";
        regex = "(.*)\\(([A-Z]{2})\\((.*)";
        p = Pattern.compile(regex);
        m = p.matcher(mtAgent);
        if (m.find()) {
            mxName = m.group(1);
            mxCountryCode = m.group(2);
            mxTownName = m.group(3);
        } else {
            regex = "(.*)\\(([A-Z]{2})";
            p = Pattern.compile(regex);
            m = p.matcher(mtAgent);
            if (m.find()) {
                mxName = m.group(1);
                mxCountryCode = m.group(2);
            } else {
                mxName = mtAgent;
            }
        }
        if(StringUtil.isNotEmpty(mxName)){
            MapUtil.puts(maps, "finInstnId/nm", mxName);
        }
        if (StringUtil.isEmpty(mxCountryCode) && StringUtil.isEmpty(mxTownName)) {
            MapUtil.adds(maps, "finInstnId/pstlAdr/adrLine", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
            buildSTErrorInfo(17, path, null);
        } else {
            if (StringUtil.isNotEmpty(mxCountryCode)) {
                MapUtil.puts(maps, "finInstnId/pstlAdr/ctry", mxCountryCode);
            }
            if (StringUtil.isNotEmpty(mxTownName)) {
                MapUtil.puts(maps, "finInstnId/pstlAdr/twnNm", mxTownName);
            }
        }
    }

    /**
     * 1、如果mtField76第一行不以4c开头，错误退出
     * 2、如果mtField76每行都以4c或//开头，走有结构逻辑，否则走无结构
     * 3、有结构的附加信息以4c开始到非//开头的行结束为一段，每段中间无空格连接
     * 4、无结构的附加信息以普通方式连接，每行小于35时以空格连接
     *
     * @param mtField76
     * @param maps
     * @param stsMaps
     * @return void
     */
    protected void mt_To_mxField76(String mtField76, Map<String, Object> maps, Map<String, Object> stsMaps) {
        if (StringUtil.isNotEmpty(mtField76)) {
            String lineRgx = "\\r?\\n";
            String[] lines = mtField76.split(lineRgx);
            String stsRgx = "/((CNCL)|(PDCR)|(RJCR))/";
            Matcher stsMatcher = Pattern.compile("^" + stsRgx + ".*$").matcher(lines[0]);
            if (!stsMatcher.matches()) {
                //Translation Preconditions
                buildSTErrorInfo(80, "Block4:76:", null);
                return;
            }
            String sts = stsMatcher.group(1);
            String validStr = extractLines(lines, "/" + sts + "/", "//",35, true);
            boolean structured = true;
            String struRgx = "/[A-Z\\d]{4}/";
            for (int i = 1; i < lines.length; i++) {
                if (!lines[i].matches("^((//)|(" + struRgx + ")).*$")) {
                    structured = false;
                    break;
                }
            }
            if (structured) {
                //SubfunctionStructured76
                StringBuilder sb = new StringBuilder(validStr);
                Pattern struPattern = Pattern.compile(struRgx);
                String[] remainLines = deleteExtractLines(lines, "/" + sts + "/", "//");
                while (remainLines.length > 0) {
                    for (String remainLine : remainLines) {
                        Matcher remainMatcher = struPattern.matcher(remainLine);
                        if (remainMatcher.find()) {
                            String keyWord = remainMatcher.group();
                            String tmpStr = extractLines(remainLines, keyWord, "//",35, true);
                            sb.append(tmpStr);
                            remainLines = deleteExtractLines(remainLines, keyWord, "//");
                        }
                    }
                }
                validStr = sb.toString();
            } else {
                //SubfunctionUnStructured76
                validStr = extractLines(lines, null, "//",35, false);
            }
            String rsnCd = null;
            String addtlInf = null;
            validStr = validStr.replaceFirst(stsRgx, "");
            if (validStr.length() > 3 && Arrays.asList("AC04", "AGNT", "AM04", "ARDT", "CUST", "INDM", "LEGL", "NOAS", "NOOR", "PTNA", "RQDA").contains(validStr.substring(0, 4))) {
                rsnCd = validStr.substring(0, 4);
                if (validStr.length() > 4) {
                    if (validStr.charAt(4) == '/') {
                        if (validStr.length() > 5) {
                            addtlInf = validStr.substring(5);
                        }
                    } else {
                        addtlInf = validStr.substring(4);
                    }
                }
            } else {
                if ("RJCR".equals(sts)) {
                    rsnCd = "NARR";
                }
                addtlInf = validStr;
            }
            stsMaps.put("conf", sts);
            Map<String, Object> rsnMap = new HashMap<String, Object>();
            if (StringUtil.isNotEmpty(rsnCd)) {
                rsnMap = MapUtil.puts("rsn/cd", rsnCd);
            }
            if (StringUtil.isNotEmpty(addtlInf)) {
                if (addtlInf.length() > 210) {
                    buildSTErrorInfo(12, "CxlDtls/TxInfAndSts/CxlStsRsnInf/AddtlInf", addtlInf);
                    addtlInf = addtlInf.substring(0, 209) + "+";
                }
                if (addtlInf.length() > 105) {
                    List<Object> follwList = MapUtil.adds(rsnMap, "addtlInf", addtlInf.substring(0, 105));
                    follwList.add(addtlInf.substring(105));
                } else {
                    MapUtil.adds(rsnMap, "addtlInf", addtlInf);
                }
            }
            if (!rsnMap.isEmpty()) {
                MapUtil.adds(maps, "cxlStsRsnInf", rsnMap);
            }
        }
    }

    protected String getMxType(String mtType) {
        String mxType = "";
        if ("103".equals(mtType)) {
            mxType = "pacs.008";
        } else if ("104".equals(mtType)) {
            mxType = "pacs.003";
        } else if ("202".equals(mtType) || "205".equals(mtType)) {
            mxType = "pacs.009";
        } else if ("204".equals(mtType)) {
            mxType = "pacs.010";
        } else if (mtType.matches("10[0-9]{1}") || mxType.matches("20[0-9]{1}")) {
            mxType = "MT" + mtType;
        } else {
            mxType = "MT" + mtType;
            buildSTErrorInfo(78, "Undrlyg/TxInf/OrgnlGrpInf/OrgnlMsgId", null);
        }
        return mxType;
    }

    /**
     * 从lines中提取以from开始的行连接以continuation开始的后继行
     * continuation替换为空，小于35的行与后继行以空格连接
     *
     * @param lines
     * @param from
     * @param continuation
     * @param length       需要加空格的长度
     * @param strict       严格模式下遇到非continuation开始的行停止提取
     * @return String
     */
    protected String extractLines(String[] lines, String from, String continuation, int length,boolean strict) {
        StringBuilder sb = new StringBuilder();
        for (String line : lines) {
            if (line.length() < length) {
                line += " ";
            }
            if (sb.length() > 0) {
                if (line.startsWith(continuation)) {
                    sb.append(line.substring(continuation.length()));
                } else if (strict || StringUtil.isEmpty(line)) {
                    break;
                } else {
                    sb.append(line);
                }
            } else if (from == null || line.startsWith(from)) {
                sb.append(line);
            }
        }
        return sb.toString().trim();
    }

    /**
     * 返回删除extractLines结果后的行
     *
     * @param lines
     * @param from
     * @param continuation
     * @return String
     */
    protected String[] deleteExtractLines(String[] lines, String from, String continuation) {
        short status = -1;
        List<String> list = new ArrayList<String>();
        for (String line : lines) {
            if (status == -1 && line.startsWith(from)) {
                status = 0;
            } else if (status == 0 && !line.startsWith(continuation)) {
                status = 1;
            }
            if (status == -1 || status == 1) {
                list.add(line);
            }
        }
        return list.toArray(new String[0]);
    }

    /**
     * 与extractLines相似，返回结果保留continuation
     *
     * @param lines
     * @param from
     * @param continuation
     * @param strict
     * @return String
     */
    protected String extractLinesAsIs(String[] lines, String from, String continuation, boolean strict) {
        StringBuilder sb = new StringBuilder();
        for (String line : lines) {
            if (sb.length() > 0) {
                if (line.startsWith(continuation)) {
                    sb.append(line);
                } else if (strict || StringUtil.isEmpty(line)) {
                    break;
                } else {
                    sb.append(line);
                }
            } else if (from == null || line.startsWith(from)) {
                sb.append(line);
            }
        }
        return sb.toString().trim();
    }

    /**
     * 1、如果mtField79为空，赋默认值
     * 2、如果mtField79不含uetr且mtUETR为空，错误退出，否则提取uetr
     * 3、从第一行提取rsnCd，剩下内容以普通连接方式提取为附加信息
     *
     * @param mtField79
     * @param mtUETR
     * @param maps
     * @return void
     */
    protected void mt_To_mxField79(String mtField79, String mtUETR, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mtField79)) {
            Map<String, Object> rsnMap = MapUtil.puts("rsn/cd", "NARR");
            MapUtil.adds(rsnMap, "addtlInf", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            MapUtil.adds(maps, "cxlRsnInf", rsnMap);
        } else {
            //OriginalUETR
            String lineRgx = "\\r?\\n";
            String narMtField79 = mtField79;  //不含mtCode时需替换uetr为空格
            String uetrRgx = "/UETR/([a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89ab][a-f0-9]{3}-[a-f0-9]{12})";
            Matcher uetrMatcher = Pattern.compile(uetrRgx).matcher(mtField79);
            if (uetrMatcher.find()) {
                String field79UETR = uetrMatcher.group(1);
                String orgnlUETR = (mtUETR == null || mtUETR.length() == 0) ? field79UETR : mtUETR;
                maps.put("orgnlUETR", orgnlUETR);
                narMtField79 = mtField79.replaceFirst("(" + lineRgx + ")?" + uetrRgx, " ");
                mtField79 = mtField79.replaceFirst(uetrRgx, "");
            } else if (StringUtil.isEmpty(mtUETR)) {
                //Translation Preconditions
                buildSTErrorInfo(76, "Block3/EndToEndReference/UniqueEndToEndTransactionReference", null);
            }
            String[] lines = mtField79.split(lineRgx);
            //CancellationReasonInformation
            String mtCode = null;
            String cxlRsnInf = lines[0];
            if (cxlRsnInf.length() > 5 && cxlRsnInf.charAt(0) == '/' && cxlRsnInf.charAt(5) == '/') {
                String subStr = cxlRsnInf.substring(1, 5);
                List<String> expectedCodes = Arrays.asList("AGNT", "AM09", "COVR", "CURR", "CUST", "CUTA", "DUPL", "FRAD", "TECH", "UPAY", "NARR");
                if (expectedCodes.contains(subStr)) {
                    mtCode = subStr;
                }
            }
            if (mtCode == null) {
                //SubfunctionNarrative79
                cxlRsnInf = extractLines(narMtField79.split(lineRgx), null, "//", 50,false);
                mtCode = "NARR";
            } else {
                //Subfunction79
                cxlRsnInf = extractLines(lines, cxlRsnInf.substring(0, 6), "//", 50,true).substring(6);
            }
            Map<String, Object> rsnMap = MapUtil.puts("rsn/cd", mtCode);
            String mtAddtlInf = cxlRsnInf;
            if(cxlRsnInf.length()>210){
                buildSTErrorInfo(12, "FIToFIPmtCxlReq/Undrlyg/TxInf/CxlRsnInf/AddtlInf", mtAddtlInf);
                mtAddtlInf = cxlRsnInf.substring(0, 209) + "+";
            }
            if (mtAddtlInf.length() > 105) {
                List<Object> followList = MapUtil.adds(rsnMap, "addtlInf", mtAddtlInf.substring(0, 105));
                followList.add(mtAddtlInf.substring(105));
            } else if (mtAddtlInf.length() > 0) {
                MapUtil.adds(rsnMap, "addtlInf", mtAddtlInf);
            } else if ("NARR".equals(mtCode)) {
                MapUtil.adds(rsnMap, "addtlInf", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            }
            MapUtil.adds(maps, "cxlRsnInf", rsnMap);
        }
    }

    protected void mt_to_mxReject72(String mt72, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mt72)) {
            return;
        }
        JSONArray stsRsnInfJsonArray = null;
        if (maps.containsKey("stsRsnInf")) {
            stsRsnInfJsonArray = (JSONArray) maps.get("stsRsnInf");
        } else {
            stsRsnInfJsonArray = new JSONArray();
        }
        Map<String, Object> stsRsnInfMap = new HashMap<String, Object>();

        String mxReasonCodeOrProprietary = "";
        String additionalInformation = "";
        int maxLength = 0;
        String mtReasonCode = "";
        String[] mt72s = mt72.split(Mx2MtConstants.NEW_LINE);
        String line2 = "";
        if (mt72s.length >= 2) {
            line2 = mt72s[1];
        }
        String regex = "/XT99/(.*)/(.*)";
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(line2);
        if (m.find()) {
            mxReasonCodeOrProprietary = m.group(1);
            additionalInformation = m.group(2);
            maxLength = 35 - 6 - mxReasonCodeOrProprietary.length() - 1;
        } else {
            regex = "/([0-9A-Z]{2}[0-9]{2})/(.*)";
            p = Pattern.compile(regex);
            m = p.matcher(line2);
            if (m.find()) {
                mtReasonCode = m.group(1);
                additionalInformation = m.group(2);
                maxLength = 29;
            } else {
                additionalInformation = line2;
                maxLength = 35;
            }
        }
        String line6 = "";
        String refreshMt72 = refreshValue(mt72, "//", 35).replace(Mx2MtConstants.NEW_LINE + "//", "");
        String[] refreshMt72s = refreshMt72.split(Mx2MtConstants.NEW_LINE);
        for (int i = 0; i < refreshMt72s.length; i++) {
            if (refreshMt72s[i].startsWith("/TEXT/")) {
                line6 = refreshMt72s[i].substring(6);
                break;
            }
        }
        if (line6.length() > 0) {
            if (additionalInformation.length() > 0) {
                if (additionalInformation.length() < maxLength) {
                    additionalInformation += " " + line6;
                } else {
                    additionalInformation += line6;
                }
            } else {
                additionalInformation = line6;
            }
        }
        Map<String, Object> rsnMaps = new HashMap<String, Object>();
        if (mxReasonCodeOrProprietary.length() > 0) {
            StatusReasonCode code = SwiftTransferUtil.getStatusReasonCodeByCode(mxReasonCodeOrProprietary);
            if (code != null) {
                rsnMaps.put("cd", mxReasonCodeOrProprietary);
            } else {
                rsnMaps.put("cd", StatusReasonCode.NARR.value());
                if (additionalInformation.length() > 0) {
                    additionalInformation = mxReasonCodeOrProprietary + "/" + additionalInformation;
                } else {
                    additionalInformation = mxReasonCodeOrProprietary;
                }
            }
        } else if (mtReasonCode.length() > 0) {
            List<MxMtReasonCodeInfo> mxMtReasonCodeInfos = SwiftTransferUtil.getMxMtReasonCodes();
            String mxReasonCode = "";
            boolean isMxErrorCodePresent = false;
            if (mxMtReasonCodeInfos != null && mxMtReasonCodeInfos.size() > 0) {
                for (int i = 0; i < mxMtReasonCodeInfos.size(); i++) {
                    MxMtReasonCodeInfo mxMtReasonCodeInfo = mxMtReasonCodeInfos.get(i);
                    if (mtReasonCode.equals(mxMtReasonCodeInfo.getMtCode())) {
                        isMxErrorCodePresent = mxMtReasonCodeInfo.isMxErrorCodePresent();
                        mxReasonCode = mxMtReasonCodeInfo.getIsoCode();
                        break;
                    }
                }
            }
            if (!isMxErrorCodePresent) {
                rsnMaps.put("prtry", mtReasonCode);
            } else {
                rsnMaps.put("cd", mxReasonCode);
            }
        } else {
            rsnMaps.put("cd", ReturnReasonCode.NARR.value());
        }
        if (additionalInformation.length() > 0) {
            if (additionalInformation.length() > 210) {
                buildSTErrorInfo(12, "TxInfAndSts/StsRsnInf/AddtlInf", additionalInformation);
                additionalInformation = additionalInformation.substring(0, 209) + "+";
            }
            List<String> list = new ArrayList<String>();
            if (additionalInformation.length() > 105) {
                String addInf1 = additionalInformation.substring(0, 105);
                String addInf2 = additionalInformation.substring(105);
                list.add(addInf1);
                list.add(addInf2);
            } else {
                list.add(additionalInformation);
            }
            stsRsnInfMap.put("addtlInf", list);
        }
        if (rsnMaps.size() > 0) {
            stsRsnInfMap.put("rsn", rsnMaps);
        }
        if (stsRsnInfMap.size() > 0) {
            stsRsnInfJsonArray.add(stsRsnInfMap);
        }
        if (stsRsnInfJsonArray.size() > 0) {
            maps.put("stsRsnInf", stsRsnInfJsonArray);
        }
    }
    protected void mt_to_mxUltimateParty(String mtFiled75, String fromPattern, Map<String, Object> partyMap) {
        if (StringUtil.isEmpty(mtFiled75)) {
            return;
        }
        String[] patternTable = {"/ULTB/", "/ULTD/", "/PURP/", "/ROC/", "/URI/", "/RELID/", "/SRI/", "/INV/", "/IPI/", "/RFB/", "/ROC/", "/TSU/"};
        String patterns = StringUtil.joinString(patternTable, "(", "|", ")");
        String rgx = "^.*?" + fromPattern + "(.*?)" + patterns + ".*$";
        Matcher partyMatcher = Pattern.compile(rgx).matcher(mtFiled75);
        if (partyMatcher.find()) {
            String partyStr = partyMatcher.group(1);
            if (StringUtil.isNotEmpty(partyStr)) {
                if (partyStr.endsWith("//")) {
                    partyStr = partyStr.substring(0, partyStr.length() - 2);
                }
                if (SwiftTransferUtil.isBIC(partyStr)) {
                    MapUtil.puts(partyMap, "id/orgId/anyBIC", partyStr);
                } else {
                    //Name/Country/TownName
                    String name;
                    String country = null;
                    String townName = null;
                    String other = null;
                    Matcher addrMatcher = Pattern.compile("^([^/]*)/([A-Z]{2})/([^/]*)$").matcher(partyStr);
                    if (addrMatcher.matches()) {
                        name = addrMatcher.group(1);
                        country = addrMatcher.group(2);
                        townName = addrMatcher.group(3);
                    } else {
                        //Name/Other” or Name/Country
                        addrMatcher = Pattern.compile("^(.*?)/(.*)$").matcher(partyStr);
                        if (addrMatcher.matches()) {
                            name = addrMatcher.group(1);
                            String oth = addrMatcher.group(2);
                            if (oth.matches("[A-Z]{2}")) {
                                country = oth;
                            } else {
                                other = oth;
                            }
                        } else {
                            name = partyStr;
                        }
                    }
                    if (StringUtil.isNotEmpty(name)) {
                        partyMap.put("nm", name.length() > 140 ? (name.substring(0, name.length() - 1) + "+") : name);
                        if (StringUtil.isNotEmpty(country)) {
                            MapUtil.puts(partyMap, "pstlAdr/ctry", country);
                            if (StringUtil.isNotEmpty(townName)) {
                                MapUtil.puts(partyMap, "pstlAdr/twnNm", townName.length() > 35 ? (townName.substring(0, townName.length() - 1) + "+") : townName);
                            } else if ("/ULTD/".equals(fromPattern)) {
                                MapUtil.puts(partyMap, "pstlAdr/twnNm", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                            }
                        } else if (StringUtil.isNotEmpty(other)) {
                            Map<String, Object> otherMap = MapUtil.puts("id", other.length() > 35 ? (other.substring(0, other.length() - 1) + "+") : other);
                            MapUtil.adds(partyMap, "id/prvtId/othr", otherMap);
                        }
                    }
                }
            }
        }
    }

    protected void mt_to_mxReturn72(String mt72, Map<String, Object> maps) {
        if (StringUtil.isEmpty(mt72)) {
            return;
        }
        JSONArray rtrRsnInfJsonArray = null;
        if (maps.containsKey("rtrRsnInf")) {
            rtrRsnInfJsonArray = (JSONArray) maps.get("rtrRsnInf");
        } else {
            rtrRsnInfJsonArray = new JSONArray();
        }
        Map<String, Object> rtrRsnInfMap = new HashMap<String, Object>();

        String mxReasonCodeOrProprietary = "";
        String additionalInformation = "";
        int maxLength = 0;
        String mtReasonCode = "";
        String[] mt72s = mt72.split(Mx2MtConstants.NEW_LINE);
        String line2 = "";
        if (mt72s.length >= 2) {
            line2 = mt72s[1];
        }
        String regex = "/XT99/(.*)/(.*)";
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(line2);
        if (m.find()) {
            mxReasonCodeOrProprietary = m.group(1);
            additionalInformation = m.group(2);
            maxLength = 35 - 6 - mxReasonCodeOrProprietary.length() - 1;
        } else {
            regex = "/([0-9A-Z]{2}[0-9]{2})/(.*)";
            p = Pattern.compile(regex);
            m = p.matcher(line2);
            if (m.find()) {
                mtReasonCode = m.group(1);
                additionalInformation = m.group(2);
                maxLength = 29;
            } else {
                additionalInformation = line2;
                maxLength = 35;
            }
        }
        String line6 = "";
        String refreshMt72 = refreshValue(mt72, "//", 35).replace(Mx2MtConstants.NEW_LINE + "//", "");
        String[] refreshMt72s = refreshMt72.split(Mx2MtConstants.NEW_LINE);
        for (int i = 0; i < refreshMt72s.length; i++) {
            if (refreshMt72s[i].startsWith("/TEXT/")) {
                line6 = refreshMt72s[i].substring(6);
                break;
            }
        }
        if (line6.length() > 0) {
            if (additionalInformation.length() > 0) {
                if (additionalInformation.length() < maxLength) {
                    additionalInformation += " " + line6;
                } else {
                    additionalInformation += line6;
                }
            } else {
                additionalInformation = line6;
            }
        }
        Map<String, Object> rsnMaps = new HashMap<String, Object>();
        if (mxReasonCodeOrProprietary.length() > 0) {
            ReturnReasonCode code = SwiftTransferUtil.getReturnReasonCodeByCode(mxReasonCodeOrProprietary);
            if (code != null) {
                rsnMaps.put("cd", mxReasonCodeOrProprietary);
            } else {
                rsnMaps.put("cd", ReturnReasonCode.NARR.value());
                if (additionalInformation.length() > 0) {
                    additionalInformation = mxReasonCodeOrProprietary + "/" + additionalInformation;
                } else {
                    additionalInformation = mxReasonCodeOrProprietary;
                }
            }
        } else if (mtReasonCode.length() > 0) {
            List<MxMtReasonCodeInfo> mxMtReasonCodeInfos = SwiftTransferUtil.getMxMtReasonCodes();
            String mxReasonCode = "";
            boolean isMxErrorCodePresent = false;
            if (mxMtReasonCodeInfos != null && mxMtReasonCodeInfos.size() > 0) {
                for (int i = 0; i < mxMtReasonCodeInfos.size(); i++) {
                    MxMtReasonCodeInfo mxMtReasonCodeInfo = mxMtReasonCodeInfos.get(i);
                    if (mtReasonCode.equals(mxMtReasonCodeInfo.getMtCode())) {
                        isMxErrorCodePresent = mxMtReasonCodeInfo.isMxErrorCodePresent();
                        mxReasonCode = mxMtReasonCodeInfo.getIsoCode();
                        break;
                    }
                }
            }
            if (!isMxErrorCodePresent) {
                if (additionalInformation.length() > 0) {
                    additionalInformation = mtReasonCode + "/" + additionalInformation;
                } else {
                    additionalInformation = mtReasonCode;
                }
            }
            if (StringUtil.isEmpty(mxReasonCode)) {
                mxReasonCode = ReturnReasonCode.NARR.value();
            }
            rsnMaps.put("cd", mxReasonCode);
        } else {
            rsnMaps.put("cd", ReturnReasonCode.NARR.value());
        }
        if (additionalInformation.length() > 0) {
            if (additionalInformation.length() > 210) {
                buildSTErrorInfo(12, "TxInf/RtrRsnInf/AddtlInf", additionalInformation);
                additionalInformation = additionalInformation.substring(0, 209) + "+";
            }
            List<String> list = new ArrayList<String>();
            if (additionalInformation.length() > 105) {
                String addInf1 = additionalInformation.substring(0, 105);
                String addInf2 = additionalInformation.substring(105);
                list.add(addInf1);
                list.add(addInf2);
            } else {
                list.add(additionalInformation);
            }
            rtrRsnInfMap.put("addtlInf", list);
        }
        if (rsnMaps.size() > 0) {
            rtrRsnInfMap.put("rsn", rsnMaps);
        }
        if (rtrRsnInfMap.size() > 0) {
            rtrRsnInfJsonArray.add(rtrRsnInfMap);
        }
        if (rtrRsnInfJsonArray.size() > 0) {
            maps.put("rtrRsnInf", rtrRsnInfJsonArray);
        }
    }

    protected String mt_to_mxDate(String mtDate) {
        if(StringUtil.isEmpty(mtDate)){
            return null;
        }
        SimpleDateFormat mtFormat = new SimpleDateFormat("yyyyMMdd");
        mtFormat.setLenient(false);
        SimpleDateFormat mxFormat = new SimpleDateFormat("yyyy-MM-dd");
        try {
            return mxFormat.format(mtFormat.parse(mtDate));
        } catch (NullPointerException e) {
            throw new SwiftException("MT日期转MX日期失败：MT日期为空", e);
        } catch (ParseException e) {
            try {
                return mxFormat.format(mtFormat.parse("20" + mtDate));
            } catch (ParseException ex) {
                throw new SwiftException("MT日期转MX日期失败：MT日期（" + mtDate + "）格式不合法", ex);
            }
        }
    }

    protected String mt_to_mxDateTime(String mtDateTime) {
        if (StringUtil.isEmpty(mtDateTime)) {
            throw new SwiftException("MT日期时间转MX日期时间失败：MT日期时间为空");
        }
        String[] split = mtDateTime.split("[+-]", -1);
        if (split.length == 2) {
            SimpleDateFormat mtFormat = new SimpleDateFormat("yyyyMMddHHmm");
            mtFormat.setLenient(false);
            SimpleDateFormat mxFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:00");
            String mxDateTime = null;
            try {
                mxDateTime = mxFormat.format(mtFormat.parse(split[0]));
            } catch (ParseException e) {
                try {
                    mxDateTime = mxFormat.format(mtFormat.parse("20" + split[0]));
                } catch (ParseException ignore) {
                }
            }
            if (mxDateTime != null && split[1].matches("((0\\d)|(1[0-3]))([0-5]\\d)?")) {
                mxDateTime = mxDateTime + mtDateTime.charAt(mtDateTime.length() - split[1].length() - 1) + split[1].substring(0, 2) + ":";
                mxDateTime += (split[1].length() > 2 ? split[1].substring(2) : "00");
                return mxDateTime;
            }
        }
        throw new SwiftException("MT日期时间转MX日期时间失败：MT日期时间（" + mtDateTime + "）格式不合法");
    }

    protected String mt_to_mxDateTime(String mtDateTime, int zoneOffset) {
        if (StringUtil.isEmpty(mtDateTime)) {
            throw new SwiftException("MT日期时间转MX日期时间失败：MT日期时间为空");
        }
        SimpleDateFormat mtFormat = new SimpleDateFormat("yyyyMMddHHmmZ");
        mtFormat.setLenient(false);
        Date mtDate;
        try {
            mtDate = mtFormat.parse(mtDateTime);
        } catch (ParseException e) {
            try {
                mtDate = mtFormat.parse("20" + mtDateTime);
            } catch (ParseException ex) {
                throw new SwiftException("MT日期时间转MX日期时间失败：MT日期时间（" + mtDateTime + "）格式不合法", ex);
            }
        }
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTime(mtDate);
        XMLGregorianCalendar xmlGregorianCalendar;
        try {
            xmlGregorianCalendar = DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
        } catch (DatatypeConfigurationException e) {
            throw new SwiftException("MT日期时间转MX日期时间失败", e);
        }
        xmlGregorianCalendar.setTimezone(zoneOffset * 60);
        return xmlGregorianCalendar.toXMLFormat();
    }

    /**
     * MT 转 MX 转换函数结束
     */

}
