package com.brilliance.swift.mt2mx.camt052001;

import com.brilliance.swift.mt2mx.AbstractMt2MxCreator;
import com.brilliance.swift.mt2mx.Mt2MxParseField;
import com.brilliance.swift.mt2mx.camt052001.impl.*;
import com.brilliance.swift.util.MapUtil;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field90C;
import com.prowidesoftware.swift.model.field.Field90D;
import com.prowidesoftware.swift.model.mt.AbstractMT;
import com.prowidesoftware.swift.model.mx.dic.CreditDebitCode;
import com.prowidesoftware.swift.utils.SwiftFormatUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Mt2MxCamt052001Creator extends AbstractMt2MxCreator {

    @Override
    public void preProcess() {
        super.preProcess();
        jsonMaps.put("identifier", "camt.052.001.08");
        MapUtil.puts(jsonMaps, "appHdr/msgDefIdr", "camt.052.001.08");
        Map<String, Object> rptMaps = new HashMap<String, Object>();
        MapUtil.adds(jsonMaps, "bkToCstmrAcctRpt/rpt", rptMaps);
        MapUtil.puts(jsonMaps, "bkToCstmrAcctRpt/grpHdr/creDtTm", getCreateDateStr());
        AbstractMT abstractMT = context.get(AbstractMT.class);
        //这个操作是将借记和贷记的笔数和金额进行计算
        Tag tag90C = abstractMT.getSwiftMessage().getBlock4().getTagByName("90C");
        Tag tag90D = abstractMT.getSwiftMessage().getBlock4().getTagByName("90D");
        if(tag90C!=null||tag90D!=null){
            int field90CNumber = 0;
            BigDecimal field90CSum = BigDecimal.valueOf(0);
            if(tag90C!=null){
                Field90C field90C = (Field90C)tag90C.asField();
                field90CNumber = Integer.parseInt(field90C.getNumber());
                Number number = SwiftFormatUtils.getNumber(field90C.getAmount());
                field90CSum = new BigDecimal(number.toString()).stripTrailingZeros();
            }
            int field90DNumber = 0;
            BigDecimal field90DSum = BigDecimal.valueOf(0);
            if(tag90D!=null){
                Field90D field90D = (Field90D)tag90D.asField();
                field90DNumber = Integer.parseInt(field90D.getNumber());
                Number number = SwiftFormatUtils.getNumber(field90D.getAmount());
                field90DSum = new BigDecimal(number.toString()).stripTrailingZeros();
            }
            BigDecimal subtract = field90CSum.subtract(field90DSum);
            int i = subtract.compareTo(BigDecimal.ZERO);
            MapUtil.puts(rptMaps, "txsSummry/ttlNtries/nbOfNtries", field90CNumber+field90DNumber);
            if(i>=0){
                MapUtil.puts(rptMaps, "txsSummry/ttlNtries/ttlNetNtry/cdtDbtInd", CreditDebitCode.CRDT.value());
                MapUtil.puts(rptMaps, "txsSummry/ttlNtries/ttlNetNtry/amt", subtract);
            }else {
                MapUtil.puts(rptMaps, "txsSummry/ttlNtries/ttlNetNtry/cdtDbtInd", CreditDebitCode.DBIT.value());
                MapUtil.puts(rptMaps, "txsSummry/ttlNtries/ttlNetNtry/amt", subtract.negate());
            }


        }

    }

    @Override
    public List<Mt2MxParseField> getParseFieldList() {
        List<Mt2MxParseField> list = new ArrayList<Mt2MxParseField>();
        list.add(new Camt052001Parse20Field());
        list.add(new Camt052001Parse21Field());
        list.add(new Camt052001Parse25Field());
        list.add(new Camt052001Parse28CField());
        list.add(new Camt052001Parse34FField());
        list.add(new Camt052001Parse13DField());
        list.add(new Camt052001Parse61Field());
        list.add(new Camt052001Parse90CField());
        list.add(new Camt052001Parse90DField());
        list.add(new Camt052001Parse86Field());
        return list;
    }
}
