package com.brilliance.swift.mt2mx.camt052001.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.camt052001.AbstractMt2MxCamt052001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.CdtDbtCode;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field61;
import com.prowidesoftware.swift.model.mt.AbstractMT;
import com.prowidesoftware.swift.utils.SwiftFormatUtils;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.brilliance.swift.constants.Mx2MtConstants.*;

//这个域是对账单细目
public class Camt052001Parse61Field extends AbstractMt2MxCamt052001ParseField {

    private static final String NAME_61 = "61";
    private static final String NAME_86 = "86";

    @Override
    public void parseField() throws SwiftException {
        SwiftMessage swiftMessage = context.get(AbstractMT.class).getSwiftMessage();
        List<Tag> tags = swiftMessage.getBlock4().getTags();
        int index61 = 0;
        for (int i = 0; i < tags.size(); i++) {
            if (!NAME_61.equals(tags.get(i).getName())) {
                continue;
            }
            index61++;
            Field61 field61 = (Field61) tags.get(i).asField();
            String valueDate = mt_to_mxDate(field61.getValueDate());
            Map<String, Object> ntryMaps = new HashMap<String, Object>();;
            MapUtil.adds(rptMaps, "ntry", ntryMaps);
            Map<String, Object> ntryDtlsMaps = new HashMap<String, Object>();;
            MapUtil.adds(ntryMaps, "ntryDtls", ntryDtlsMaps);
            Map<String, Object> txDtlsMaps = new HashMap<String, Object>();
            MapUtil.adds(ntryDtlsMaps, "txDtls", txDtlsMaps);
            MapUtil.puts(ntryMaps, "sts/cd", "book");
            MapUtil.puts(ntryMaps, "bkTxCd/prtry/cd", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            MapUtil.puts(ntryMaps, "bkTxCd/prtry/issr", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            MapUtil.puts(ntryMaps,"valDt/dt",valueDate);
            String entryDate = field61.getEntryDate();
            if (StringUtil.isNotEmpty(entryDate)) {
                if(entryDate.length()==4){
                    entryDate = Calendar.getInstance().get(Calendar.YEAR) + entryDate;
                }else if(entryDate.length()==6){
                    entryDate = "20" + entryDate;
                }
                entryDate = mt_to_mxDate(entryDate);
                MapUtil.puts(ntryMaps, "bookgDt/dt", entryDate);
            }
            //General Information
            String dcMark = field61.getDCMark();
            if (StringUtil.isNotEmpty(dcMark)) {
                if (DRCRMARK_RD.equals(dcMark) || DRCRMARK_RC.equals(dcMark) ||
                        DRCRMARK_C.equals(dcMark) || DRCRMARK_D.equals(dcMark)) {
                    MapUtil.puts(ntryMaps, "sts/cd", "BOOK");
                } else if (DRCRMARK_EC.equals(dcMark) || DRCRMARK_ED.equals(dcMark)) {
                    MapUtil.puts(ntryMaps, "sts/cd", "PDNG");
                }
                if (DRCRMARK_RD.equals(dcMark) || DRCRMARK_RC.equals(dcMark)) {
                    ntryMaps.put("rvslInd", Boolean.TRUE);
                } else if (DRCRMARK_C.equals(dcMark) || DRCRMARK_D.equals(dcMark)) {
                    ntryMaps.put("rvslInd", Boolean.FALSE);
                }
                if (DRCRMARK_RD.equals(dcMark) || DRCRMARK_C.equals(dcMark) || DRCRMARK_EC.equals(dcMark)) {
                    ntryMaps.put("cdtDbtInd", CdtDbtCode.CRDT.value());
                    MapUtil.puts(txDtlsMaps, "refs/endToEndId", field61.getReferenceForTheAccountOwner());
                } else if (DRCRMARK_RC.equals(dcMark) || DRCRMARK_D.equals(dcMark) || DRCRMARK_ED.equals(dcMark)) {
                    ntryMaps.put("cdtDbtInd", CdtDbtCode.DBIT.value());
                    MapUtil.puts(txDtlsMaps, "refs/instrId", field61.getReferenceForTheAccountOwner());
                }
            }
            if (StringUtil.isNotEmpty(field61.getFundsCode())) {
                buildSTErrorInfo(7, "Block4:61:[" + index61 + "]", field61.getFundsCode());
            }
            Number number = SwiftFormatUtils.getNumber(field61.getAmount());
            BigDecimal amt = new BigDecimal(number.toString()).stripTrailingZeros();
            MapUtil.puts(ntryMaps, "amt/value", amt.toPlainString());
            String ccy = MapUtil.gets(rptMaps, "acct/ccy");
            MapUtil.puts(ntryMaps, "amt/ccy", ccy);
            if (StringUtil.isNotEmpty(field61.getTransactionType()) || StringUtil.isNotEmpty(field61.getIdentificationCode())) {
                buildSTErrorInfo(7, "Block4:61:[" + index61 + "]", field61.getTransactionType() + field61.getIdentificationCode());
            }
            if (StringUtil.isNotEmpty(field61.getReferenceOfTheAccountServicingInstitution())) {
                MapUtil.puts(txDtlsMaps, "refs/acctSvcrRef", field61.getReferenceOfTheAccountServicingInstitution());
            }
            if (StringUtil.isNotEmpty(field61.getSupplementaryDetails())) {
                txDtlsMaps.put("addtlTxInf", field61.getSupplementaryDetails());
            }
            if (i < tags.size() - 1 && NAME_86.equals(tags.get(i + 1).getName())) {
                String tag86Value = tags.get(i + 1).getValue();
                ntryMaps.put("addtlNtryInf", tag86Value.replaceAll(NEW_LINE, ""));
            }
        }
    }
}
