package com.brilliance.swift.mt2mx.camt053001.impl;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.camt053001.AbstractMt2MxCamt053001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.vo.common.BalanceSubTypeCode;
import com.brilliance.swift.vo.common.BalanceTypeCode;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field;
import com.prowidesoftware.swift.model.field.Field60F;
import com.prowidesoftware.swift.model.field.Field60M;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.HashMap;
import java.util.Map;

import static com.brilliance.swift.constants.Mx2MtConstants.DRCRMARK_C;
import static com.brilliance.swift.vo.common.CdtDbtCode.CRDT;
import static com.brilliance.swift.vo.common.CdtDbtCode.DBIT;

//起始余额
public class Camt053001Parse60Field extends AbstractMt2MxCamt053001ParseField {

    private static final String NAME_F = "60F";

    private static final String NAME_M = "60M";

    @Override
    public void parseField() throws SwiftException {
        SwiftMessage swiftMessage = context.get(AbstractMT.class).getSwiftMessage();
        Tag tag60F = swiftMessage.getBlock4().getTagByName(NAME_F);
        Tag tag60M = swiftMessage.getBlock4().getTagByName(NAME_M);
        if (tag60F != null || tag60M != null) {
            Map<String, Object> balMap = new HashMap<String, Object>();
            MapUtil.adds(stmtMaps, "bal", balMap);
            MapUtil.puts(balMap, "tp/cdOrPrtry/cd", BalanceTypeCode.OPBD.value());
            Field60DataHelper field60DataHelper = new Field60DataHelper(tag60F != null ? tag60F.asField() : tag60M.asField());
            balMap.put("cdtDbtInd", DRCRMARK_C.equals(field60DataHelper.dcMark) ? CRDT.value() : DBIT.value());
            String mxDate = mt_to_mxDate(field60DataHelper.date);
            MapUtil.puts(balMap, "dt/dt", mxDate);
            MapUtil.puts(balMap, "amt/ccy", field60DataHelper.currency);
            MapUtil.puts(balMap, "amt/value", field60DataHelper.amount);
            if(tag60M!=null){
                MapUtil.puts(balMap, "tp/subTp/cd", BalanceSubTypeCode.INTM.value());
            }
        }
    }

    private static class Field60DataHelper {
        String dcMark;
        String date;
        String currency;
        String amount;

        Field60DataHelper(Field field) {
            if (field instanceof Field60F) {
                Field60F field60F = (Field60F) field;
                dcMark = field60F.getDCMark();
                date = field60F.getDate();
                currency = field60F.getCurrency();
                amount = field60F.amount().toPlainString();
            } else if (field instanceof Field60M) {
                Field60M field60M = (Field60M) field;
                dcMark = field60M.getDCMark();
                date = field60M.getDate();
                currency = field60M.getCurrency();
                amount = field60M.amount().toPlainString();
            }
        }
    }
}
