package com.brilliance.swift.mt2mx.camt053001.impl;

import com.alibaba.fastjson.JSONArray;
import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.camt053001.AbstractMt2MxCamt053001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.CdtDbtCode;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field61;
import com.prowidesoftware.swift.model.mt.AbstractMT;
import com.prowidesoftware.swift.utils.SwiftFormatUtils;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.brilliance.swift.constants.Mx2MtConstants.*;

//对账单细目
public class Camt053001Parse61Field extends AbstractMt2MxCamt053001ParseField {

    private static final String NAME_61 = "61";
    private static final String NAME_86 = "86";

    @Override
    public void parseField() throws SwiftException {
        AbstractMT abstractMT = context.get(AbstractMT.class);
        SwiftMessage swiftMessage = context.get(AbstractMT.class).getSwiftMessage();
        List<Tag> tags = swiftMessage.getBlock4().getTags();
        int index61 = 0;
        for (int i = 0; i < tags.size(); i++) {
            if (!NAME_61.equals(tags.get(i).getName())) {
                continue;
            }
            index61++;
            Field61 field61 = (Field61) tags.get(i).asField();
            String valueDate = mt_to_mxDate(field61.getValueDate());
            JSONArray ntryJsonArray = (JSONArray) stmtMaps.get("ntry");
            Map<String, Object> ntryMaps = null;
            if(index61==1){
                ntryMaps = (Map<String, Object>)ntryJsonArray.get(index61-1);
            }else {
                ntryMaps = new HashMap<String, Object>();
                MapUtil.adds(stmtMaps, "ntry", ntryMaps);
            }
            JSONArray ntryDtlsArray = (JSONArray) ntryMaps.get("ntryDtls");
            Map<String, Object> ntryDtlsMaps = null;
            if(ntryDtlsArray!=null&&ntryDtlsArray.size()>0){
                ntryDtlsMaps = (Map<String, Object>) ntryDtlsArray.get(0);
            }else {
                ntryDtlsMaps = new HashMap<String, Object>();
                MapUtil.adds(ntryMaps, "ntryDtls", ntryDtlsMaps);
            }
            JSONArray txDtlsArray = (JSONArray) ntryDtlsMaps.get("txDtls");
            Map<String, Object> txDtlsMaps = null;
            if(txDtlsArray!=null&&txDtlsArray.size()>0){
                txDtlsMaps = (Map<String, Object>) txDtlsArray.get(0);
            }else {
                txDtlsMaps = new HashMap<String, Object>();
                MapUtil.adds(ntryDtlsMaps, "txDtls", txDtlsMaps);
            }
            MapUtil.puts(ntryMaps, "sts/cd", "book");
            MapUtil.puts(ntryMaps, "bkTxCd/prtry/cd", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            MapUtil.puts(ntryMaps, "bkTxCd/prtry/issr", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            MapUtil.puts(ntryMaps,"valDt/dt",valueDate);
            String entryDate = field61.getEntryDate();
            if (StringUtil.isNotEmpty(entryDate)) {
                if(entryDate.length()==4){
                    entryDate = Calendar.getInstance().get(Calendar.YEAR) + entryDate;
                }else if(entryDate.length()==6){
                    entryDate = "20" + entryDate;
                }
                entryDate = mt_to_mxDate(entryDate);
                MapUtil.puts(ntryMaps, "bookgDt/dt", entryDate);
            }
            //General Information
            String dcMark = field61.getDCMark();
            if (StringUtil.isNotEmpty(dcMark)) {
                if (DRCRMARK_RD.equals(dcMark) || DRCRMARK_RC.equals(dcMark) ||
                        DRCRMARK_C.equals(dcMark) || DRCRMARK_D.equals(dcMark)) {
                    MapUtil.puts(ntryMaps, "sts/cd", "BOOK");
                } else if (DRCRMARK_EC.equals(dcMark) || DRCRMARK_ED.equals(dcMark)) {
                    MapUtil.puts(ntryMaps, "sts/cd", "PDNG");
                }
                if (DRCRMARK_RD.equals(dcMark) || DRCRMARK_RC.equals(dcMark)) {
                    ntryMaps.put("rvslInd", Boolean.TRUE);
                } else if (DRCRMARK_C.equals(dcMark) || DRCRMARK_D.equals(dcMark)) {
                    ntryMaps.put("rvslInd", Boolean.FALSE);
                }
                if (DRCRMARK_RD.equals(dcMark) || DRCRMARK_C.equals(dcMark) || DRCRMARK_EC.equals(dcMark)) {
                    ntryMaps.put("cdtDbtInd", CdtDbtCode.CRDT.value());
                    MapUtil.puts(txDtlsMaps, "refs/endToEndId", field61.getReferenceForTheAccountOwner());
                } else if (DRCRMARK_RC.equals(dcMark) || DRCRMARK_D.equals(dcMark) || DRCRMARK_ED.equals(dcMark)) {
                    ntryMaps.put("cdtDbtInd", CdtDbtCode.DBIT.value());
                    MapUtil.puts(txDtlsMaps, "refs/instrId", field61.getReferenceForTheAccountOwner());
                }
            }
            Number number = SwiftFormatUtils.getNumber(field61.getAmount());
            if (StringUtil.isNotEmpty(field61.getFundsCode())) {
                buildSTErrorInfo(7, "Block4:61:[" + index61 + "]", field61.getFundsCode());
            }
            String ccy = MapUtil.gets(stmtMaps, "acct/ccy");
            BigDecimal amt = new BigDecimal(number.toString()).stripTrailingZeros();
            MapUtil.puts(ntryMaps, "amt/value", amt.toPlainString());
            MapUtil.puts(ntryMaps, "amt/ccy", ccy);
            if (StringUtil.isNotEmpty(field61.getTransactionType()) || StringUtil.isNotEmpty(field61.getIdentificationCode())) {
                buildSTErrorInfo(7, "Block4:61:[" + index61 + "]", field61.getTransactionType() + field61.getIdentificationCode());
            }
            if (StringUtil.isNotEmpty(field61.getReferenceOfTheAccountServicingInstitution())) {
                MapUtil.puts(ntryMaps, "acctSvcrRef", field61.getReferenceOfTheAccountServicingInstitution());
            }
            if (StringUtil.isNotEmpty(field61.getSupplementaryDetails())) {
                txDtlsMaps.put("addtlTxInf", field61.getSupplementaryDetails());
            }
            if("940".equals(abstractMT.getMessageType())){
                if (i < tags.size() - 1 && NAME_86.equals(tags.get(i + 1).getName())) {
                    String tag86Value = tags.get(i + 1).getValue();
                    ntryMaps.put("addtlNtryInf", tag86Value.replaceAll(NEW_LINE, ""));
                }
            }
        }
    }
}
