package com.brilliance.swift.mt2mx.camt053001.impl;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.camt053001.AbstractMt2MxCamt053001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.vo.common.BalanceSubTypeCode;
import com.brilliance.swift.vo.common.BalanceTypeCode;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field;
import com.prowidesoftware.swift.model.field.Field62F;
import com.prowidesoftware.swift.model.field.Field62M;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.HashMap;
import java.util.Map;

import static com.brilliance.swift.constants.Mx2MtConstants.DRCRMARK_C;
import static com.brilliance.swift.vo.common.CdtDbtCode.CRDT;
import static com.brilliance.swift.vo.common.CdtDbtCode.DBIT;

//结束余额
public class Camt053001Parse62Field extends AbstractMt2MxCamt053001ParseField {

    private static final String NAME_F = "62F";

    private static final String NAME_M = "62M";

    @Override
    public void parseField() throws SwiftException {
        SwiftMessage swiftMessage = context.get(AbstractMT.class).getSwiftMessage();
        Tag tag62F = swiftMessage.getBlock4().getTagByName(NAME_F);
        Tag tag62M = swiftMessage.getBlock4().getTagByName(NAME_M);
        if (tag62F != null || tag62M != null) {
            Map<String, Object> balMap = new HashMap<String, Object>();
            MapUtil.adds(stmtMaps, "bal", balMap);
            MapUtil.puts(balMap, "tp/cdOrPrtry/cd", BalanceTypeCode.CLBD.value());
            Field62DataHelper field62DataHelper = new Field62DataHelper(tag62F != null ? tag62F.asField() : tag62M.asField());
            balMap.put("cdtDbtInd", DRCRMARK_C.equals(field62DataHelper.dcMark) ? CRDT.value() : DBIT.value());
            String mxDate = mt_to_mxDate(field62DataHelper.date);
            MapUtil.puts(balMap, "dt/dt", mxDate);
            MapUtil.puts(balMap, "amt/ccy", field62DataHelper.currency);
            MapUtil.puts(balMap, "amt/value", field62DataHelper.amount);
            MapUtil.puts(stmtMaps, "stmtPgntn/lastPgInd", field62DataHelper.lastPageIndicator);
            if (!field62DataHelper.lastPageIndicator) {
                MapUtil.puts(balMap, "tp/subTp/cd", BalanceSubTypeCode.INTM.value());
            }
        }
    }

    private static class Field62DataHelper {
        String dcMark;
        String date;
        String currency;
        String amount;
        boolean lastPageIndicator;

        Field62DataHelper(Field field) {
            if (field instanceof Field62F) {
                Field62F field62F = (Field62F) field;
                dcMark = field62F.getDCMark();
                date = field62F.getDate();
                currency = field62F.getCurrency();
                amount = field62F.amount().toPlainString();
                lastPageIndicator = true;
            } else if (field instanceof Field62M) {
                Field62M field62M = (Field62M) field;
                dcMark = field62M.getDCMark();
                date = field62M.getDate();
                currency = field62M.getCurrency();
                amount = field62M.amount().toPlainString();
                lastPageIndicator = false;
            }
        }
    }
}
