package com.brilliance.swift.mt2mx.pacs002001;

import com.brilliance.swift.mt2mx.AbstractMt2MxCreator;
import com.brilliance.swift.mt2mx.Mt2MxContextIdentifier;
import com.brilliance.swift.mt2mx.Mt2MxParseField;
import com.brilliance.swift.mt2mx.pacs002001.mt103rejt.Pacs002001Parse20Field;
import com.brilliance.swift.mt2mx.pacs002001.mt103rejt.Pacs002001Parse72Field;
import com.brilliance.swift.mt2mx.pacs002001.mt202rejt.Pacs002001Parse21Field;
import com.brilliance.swift.mt2mx.pacs002001.mtn99rejt.Pacs002001Parse21About99Field;
import com.brilliance.swift.mt2mx.pacs002001.mtn99rejt.Pacs002001Parse79Field;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Mt2MxPacs002001Creator extends AbstractMt2MxCreator {

    @Override
    public void preProcess() {
        super.preProcess();
        jsonMaps.put("identifier", "pacs.002.001.10");
        MapUtil.puts(jsonMaps, "appHdr/msgDefIdr", "pacs.002.001.10");
        Map<String, Object> txInfAndStsMaps = new HashMap<String, Object>();
        MapUtil.adds(jsonMaps, "fiToFIPmtStsRpt/txInfAndSts", txInfAndStsMaps);
        //Default Values
        MapUtil.puts(jsonMaps, "fiToFIPmtStsRpt/grpHdr/creDtTm", getCreateDateStr());
        //TR001
        String sendBic = (String) context.get(Mt2MxContextIdentifier.MT_SEND_BIC, true);
        if (StringUtil.isNotEmpty(sendBic)) {
            MapUtil.puts(txInfAndStsMaps, "instgAgt/finInstnId/bicfi", sendBic);
        }
        String recvBic = (String) context.get(Mt2MxContextIdentifier.MT_RECEIVE_BIC, true);
        if (StringUtil.isNotEmpty(recvBic)) {
            MapUtil.puts(txInfAndStsMaps, "instdAgt/finInstnId/bicfi", recvBic);
        }
        AbstractMT abstractMT = context.get(AbstractMT.class);
        //设置原始消息类型
        String messageType = abstractMT.getSwiftMessage().isCOV() ? abstractMT.getMessageType() + " COVE" : abstractMT.getMessageType();
        MapUtil.puts(txInfAndStsMaps, "orgnlGrpInf/orgnlMsgNmId", "MT" + messageType);
        //设置UETR
        String uetr = abstractMT.getSwiftMessage().getUETR();
        if (StringUtil.isNotEmpty(uetr)) {
            txInfAndStsMaps.put("orgnlUETR", uetr);
        }
    }

    @Override
    public List<Mt2MxParseField> getParseFieldList() {
        AbstractMT abstractMT = context.get(AbstractMT.class);
        List<Mt2MxParseField> list = new ArrayList<Mt2MxParseField>();
        if ("103".equals(abstractMT.getMessageType())) {
            list.add(new Pacs002001Parse20Field());
            list.add(new Pacs002001Parse72Field());
        } else if ("202".equals(abstractMT.getMessageType()) || "205".equals(abstractMT.getMessageType())) {
            list.add(new Pacs002001Parse20Field());
            list.add(new Pacs002001Parse21Field());
            list.add(new Pacs002001Parse72Field());
        } else if ("199".equals(abstractMT.getMessageType()) || "299".equals(abstractMT.getMessageType())) {
            list.add(new Pacs002001Parse20Field());
            list.add(new Pacs002001Parse21About99Field());
            list.add(new Pacs002001Parse79Field());
        }
        return list;
    }
}
