package com.brilliance.swift.mt2mx.pacs002001.mtn99rejt;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.mt2mx.pacs002001.AbstractMt2MxPacs002001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.StatusReasonCode;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field79;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Pacs002001Parse79Field extends AbstractMt2MxPacs002001ParseField {
    private static final String NAME = "79";

    @Override
    public void parseField() {
        SwiftMessage swiftMessage = context.get(AbstractMT.class).getSwiftMessage();
        Tag tag79 = swiftMessage.getBlock4().getTagByName(NAME);
        if (tag79 != null) {
            Field79 field79 = (Field79) tag79.asField();
            List<String> lines = field79.getLines();
            for (int i = 0; i < lines.size(); i++) {
                String line = lines.get(i);
                if (line.startsWith("/MREF/")) {
                    MapUtil.puts(txInfAndStsMaps, "orgnlGrpInf/orgnlMsgId", line.substring(6));
                } else if (line.startsWith("/TREF/")) {
                    txInfAndStsMaps.put("orgnlEndToEndId", line.substring(6));
                } else if (line.startsWith("/XT99/")) {
                    //可能是rsnPrtry或rsnCd
                    String rsnPrtry = line.substring(6);
                    if (rsnPrtry.charAt(rsnPrtry.length() - 1) == '/') {
                        rsnPrtry = rsnPrtry.substring(0, rsnPrtry.length() - 1);
                    }
                    Map<String, Object> stsRsnInfMap = MapUtil.puts("rsn/prtry", rsnPrtry);
                    MapUtil.adds(txInfAndStsMaps, "stsRsnInf", stsRsnInfMap);
                } else if (line.startsWith("/TEXT/")) {
                    String addtlInf = line.substring(6);
                    Matcher uetrMatcher = Pattern.compile("(//)?/UETR/(.*)").matcher(addtlInf);
                    if (uetrMatcher.find()) {
                        txInfAndStsMaps.put("orgnlUETR", uetrMatcher.group(2));
                        addtlInf = addtlInf.substring(0, uetrMatcher.start()) + addtlInf.substring(uetrMatcher.end());
                        addtlInf = addtlInf.replaceAll(Mx2MtConstants.NEW_LINE + "//", "");
                    }
                    if (StringUtil.isNotEmpty(addtlInf)) {
                        String[] split = addtlInf.split(Mx2MtConstants.NEW_LINE);
                        Map<String, Object> stsRsnInfMap = new HashMap<String, Object>();
                        MapUtil.adds(txInfAndStsMaps, "stsRsnInf", stsRsnInfMap);
                        for (String str : split) {
                            MapUtil.adds(stsRsnInfMap, "addtlInf", str);
                        }
                    }
                } else {
                    Matcher matcher = Pattern.compile("^/(.{4})/(.*)$").matcher(line);
                    if (matcher.matches() && i == 1) {
                        String code = matcher.group(1);
                        if (StatusReasonCode.isValidCode(code)) {
                            Map<String, Object> stsRsnInfMap = MapUtil.puts("rsn/cd", code);
                            MapUtil.adds(txInfAndStsMaps, "stsRsnInf", stsRsnInfMap);
                            String addInfo = matcher.group(2);
                            if (StringUtil.isNotEmpty(addInfo)) {
                                MapUtil.adds(stsRsnInfMap, "addtlInf", addInfo);
                            }
                        }
                    }
                }
            }
        }
    }
}
