package com.brilliance.swift.mt2mx.pacs004001;

import com.brilliance.swift.mt2mx.AbstractMt2MxCreator;
import com.brilliance.swift.mt2mx.Mt2MxContextIdentifier;
import com.brilliance.swift.mt2mx.Mt2MxParseField;
import com.brilliance.swift.mt2mx.pacs004001.mt103retn.*;
import com.brilliance.swift.mt2mx.pacs004001.mt202retn.Pacs004001Parse21Field;
import com.brilliance.swift.mt2mx.pacs004001.mt202retn.Pacs004001Parse58Field;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.ChargeForEnum;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Mt2MxPacs004001Creator extends AbstractMt2MxCreator {

    @Override
    public void preProcess() {
        super.preProcess();
        jsonMaps.put("identifier", "pacs.004.001.09");
        MapUtil.puts(jsonMaps, "appHdr/msgDefIdr", "pacs.004.001.09");
        Map<String, Object> txInfMap = new HashMap<String, Object>();
        MapUtil.adds(jsonMaps, "pmtRtr/txInf", txInfMap);
        //Default Values
        MapUtil.puts(jsonMaps, "pmtRtr/grpHdr/nbOfTxs", "1");
        MapUtil.puts(jsonMaps, "pmtRtr/grpHdr/creDtTm", getCreateDateStr());
        //TR001
        String sendBic = (String) context.get(Mt2MxContextIdentifier.MT_SEND_BIC, true);
        if (StringUtil.isNotEmpty(sendBic)) {
            MapUtil.puts(txInfMap, "instgAgt/finInstnId/bicfi", sendBic);
        }
        String recvBic = (String) context.get(Mt2MxContextIdentifier.MT_RECEIVE_BIC, true);
        if (StringUtil.isNotEmpty(recvBic)) {
            MapUtil.puts(txInfMap, "instdAgt/finInstnId/bicfi", recvBic);
        }
        AbstractMT abstractMT = context.get(AbstractMT.class);
        String mtUETR = abstractMT.getSwiftMessage().getUETR();
        if (StringUtil.isNotEmpty(mtUETR)) {
            txInfMap.put("orgnlUETR", mtUETR);
        } else {
            //PREC002
            buildSTErrorInfo(76, "Block3/{121}", null);
        }
        //Block3/EndToEndReference/ServiceTypeIdentifier
        Tag tag111 = abstractMT.getSwiftMessage().getBlock3().getTagByName("111");
        if (tag111 != null) {
            String serTypId = tag111.getValue();
            if (serTypId.matches("00\\d")) {
                Map<String, Object> svcLvlMap = MapUtil.puts("cd", "G" + serTypId);
                MapUtil.adds(txInfMap, "orgnlTxRef/pmtTpInf/svcLvl", svcLvlMap);
            }
        }

        //202,205
        //Default Values
        if ("202".equals(abstractMT.getMessageType()) || "205".equals(abstractMT.getMessageType())) {
            txInfMap.put("chrgBr", ChargeForEnum.SHAR.value());
            buildSTErrorInfo(81, "PmtRtr/TxInf/ChrgBr", null);
        }
    }

    @Override
    public List<Mt2MxParseField> getParseFieldList() {
        AbstractMT abstractMT = context.get(AbstractMT.class);
        List<Mt2MxParseField> list = new ArrayList<Mt2MxParseField>();
        if ("103".equals(abstractMT.getMessageType())) {
            list.add(new Pacs004001Parse20Field());
            list.add(new Pacs004001Parse13CField());
            list.add(new Pacs004001Parse23BField());
            list.add(new Pacs004001Parse23EField());
            list.add(new Pacs004001Parse26TField());
            list.add(new Pacs004001Parse32AField());
            list.add(new Pacs004001Parse33BField());
            list.add(new Pacs004001Parse36Field());
            list.add(new Pacs004001Parse50Field());
            list.add(new Pacs004001Parse52Field());
            list.add(new Pacs004001ParseSetlMtdField());
            list.add(new Pacs004001ParseSetlAcctField());
            list.add(new Pacs004001Parse53Field());
            list.add(new Pacs004001Parse54Field());
            list.add(new Pacs004001Parse55Field());
            list.add(new Pacs004001Parse56Field());
            list.add(new Pacs004001Parse57Field());
            list.add(new Pacs004001Parse59Field());
            list.add(new Pacs004001Parse70Field());
            list.add(new Pacs004001Parse71Field());
            list.add(new Pacs004001Parse72Field());
            list.add(new Pacs004001Parse77BField());
        } else if ("202".equals(abstractMT.getMessageType())
                || "205".equals(abstractMT.getMessageType())) {
            list.add(new Pacs004001Parse20Field());
            list.add(new Pacs004001Parse21Field());
            list.add(new Pacs004001Parse13CField());
            list.add(new Pacs004001Parse32AField());
            list.add(new com.brilliance.swift.mt2mx.pacs004001.mt202retn.Pacs004001Parse52Field());
            list.add(new Pacs004001ParseSetlMtdField());
            list.add(new Pacs004001ParseSetlAcctField());
            list.add(new Pacs004001Parse53Field());
            list.add(new Pacs004001Parse54Field());
            list.add(new Pacs004001Parse56Field());
            list.add(new Pacs004001Parse57Field());
            list.add(new Pacs004001Parse58Field());
            list.add(new Pacs004001Parse72Field());
        }
        return list;
    }
}
