package com.brilliance.swift.mt2mx.pacs004001.mt103retn;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs004001.AbstractMt2MxPacs004001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.OrganisationIdentificationCode;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field50A;
import com.prowidesoftware.swift.model.field.Field50F;
import com.prowidesoftware.swift.model.field.Field50K;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Pacs004001Parse50Field extends AbstractMt2MxPacs004001ParseField {

    private static final String NAME_A = "50A";

    private static final String NAME_F = "50F";

    private static final String NAME_K = "50K";

    @Override
    public void parseField() throws SwiftException {
        SwiftMessage swiftMessage = context.get(AbstractMT.class).getSwiftMessage();
        Tag tag50A = swiftMessage.getBlock4().getTagByName(NAME_A);
        Tag tag50F = swiftMessage.getBlock4().getTagByName(NAME_F);
        Tag tag50K = swiftMessage.getBlock4().getTagByName(NAME_K);
        if (tag50A != null) {
            Field50A field50A = (Field50A) tag50A.asField();
            MapUtil.puts(txInfMaps, "rtrChain/dbtr/pty/id/orgId/anyBIC", field50A.getBIC());
            Map<String, Object> othrMap = MapUtil.puts("id", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            MapUtil.puts(othrMap, "schmeNm/cd", OrganisationIdentificationCode.TXID);
            MapUtil.adds(txInfMaps, "rtrChain/dbtr/pty/id/orgId/othr", othrMap);
        }
        if (tag50F != null) {
            Field50F field50F = (Field50F) tag50F.asField();
            String partyIdentifier = field50F.getPartyIdentifier();
            List<String> lines = field50F.getLines().subList(1, field50F.getLines().size());
            String nameAddress = StringUtil.joinString(lines.toArray(new String[0]), Mx2MtConstants.NEW_LINE);
            if (partyIdentifier.startsWith("/")) {
                //(Account)
                Map<String, Object> othrMap = MapUtil.puts("id", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                MapUtil.adds(txInfMaps, "rtrChain/dbtr/pty/id/orgId/othr", othrMap);
                MapUtil.puts(othrMap, "schmeNm/cd", OrganisationIdentificationCode.TXID);
            } else {
                //(Code)(Country Code)(Identifier)
                Map<String, Object> idMap = new HashMap<String, Object>();
                mt_to_mxFATFIdentification(partyIdentifier, nameAddress, idMap);
                MapUtil.puts(txInfMaps, "rtrChain/dbtr/pty/id", idMap);
            }
            Map<String, Object> orgPtyMap = MapUtil.getsMap(txInfMaps, "rtrChain/dbtr/pty");
            Map<String, Object> ptyMap = orgPtyMap == null ? new HashMap<String, Object>() : orgPtyMap;
            mt_to_mxFATFNameAndAddress(partyIdentifier, nameAddress, ptyMap);
            if (orgPtyMap == null) {
                MapUtil.puts(txInfMaps, "rtrChain/dbtr/pty", ptyMap);
            }
        }
        if (tag50K != null) {
            Field50K field50K = (Field50K) tag50K.asField();
            String nameAndAddressLine1 = field50K.getNameAndAddressLine1();
            if (Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE.equals(field50K.getComponent1()) && nameAndAddressLine1.startsWith("//")) {
                Map<String, Object> clrSysMmbIdMap = new HashMap<String, Object>();
                if (isMTClearingSystemCodeInList(nameAndAddressLine1)) {
                    mt_to_mxClearingIdentifier(nameAndAddressLine1, clrSysMmbIdMap);
                } else {
                    mt_to_mxClearingSystemToNameAndAddressLine(field50K.getNameAndAddress(), null, clrSysMmbIdMap, "Dbtr/Agt/FinInstnId");
                    buildSTErrorInfo(101, "PmtRtr/TxInf/RtrChain/Dbtr/Agt/FinInstnId", null);
                }
                MapUtil.puts(txInfMaps, "rtrChain/dbtr/agt/finInstnId/clrSysMmbId", clrSysMmbIdMap);
            } else {
                Map<String, Object> othrMap = MapUtil.puts("id", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                MapUtil.adds(txInfMaps, "rtrChain/dbtr/pty/id/orgId/othr", othrMap);
                MapUtil.puts(othrMap, "schmeNm/cd", OrganisationIdentificationCode.TXID);
                Map<String, Object> ptyMap = new HashMap<String, Object>();
                mt_to_mxPartyNameAndAddress(field50K.getNameAndAddress(), ptyMap);
                MapUtil.puts(txInfMaps, "rtrChain/dbtr/pty", ptyMap);
            }
        }
    }
}
