package com.brilliance.swift.mt2mx.pacs004001.mt103retn;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs004001.AbstractMt2MxPacs004001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field56A;
import com.prowidesoftware.swift.model.field.Field56D;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.HashMap;
import java.util.Map;

public class Pacs004001Parse56Field extends AbstractMt2MxPacs004001ParseField {

    private static final String NAME_A = "56A";

    private static final String NAME_C = "56C";

    private static final String NAME_D = "56D";

    @Override
    public void parseField() throws SwiftException {
        SwiftMessage swiftMessage = context.get(AbstractMT.class).getSwiftMessage();
        Tag tag56A = swiftMessage.getBlock4().getTagByName(NAME_A);
        Tag tag56C = swiftMessage.getBlock4().getTagByName(NAME_C);
        Tag tag56D = swiftMessage.getBlock4().getTagByName(NAME_D);
        if (tag56A != null) {
            Field56A field56A = (Field56A) tag56A.asField();
            String ptyId = field56A.getComponent1() == null ? field56A.getComponent2() : null;
            if (ptyId != null && ptyId.startsWith("/") && !ptyId.startsWith("/CH") && !ptyId.startsWith("/FW") && !ptyId.startsWith("/RT")) {
                Map<String, Object> clrSysMmbIdMap = new HashMap<String, Object>();
                mt_to_mxClearingIdentifier("/" + ptyId, clrSysMmbIdMap);
                MapUtil.puts(txInfMaps, "rtrChain/intrmyAgt1/finInstnId/clrSysMmbId", clrSysMmbIdMap);
            }
            MapUtil.puts(txInfMaps, "rtrChain/intrmyAgt1/finInstnId/bicfi", field56A.getBIC());
        }
        if (tag56C != null) {
            String partyIdentifier = tag56C.getValue();
            if (partyIdentifier.startsWith("//") && !partyIdentifier.startsWith("//CH")) {
                if (isMTClearingSystemCodeInList(partyIdentifier)) {
                    Map<String, Object> clrSysMmbIdMap = new HashMap<String, Object>();
                    mt_to_mxClearingIdentifier(partyIdentifier, clrSysMmbIdMap);
                    MapUtil.puts(txInfMaps, "rtrChain/intrmyAgt1/finInstnId/clrSysMmbId", clrSysMmbIdMap);
                    MapUtil.puts(txInfMaps, "rtrChain/intrmyAgt1/finInstnId/nm", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                    MapUtil.adds(txInfMaps, "rtrChain/intrmyAgt1/finInstnId/pstlAdr/adrLine", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                } else {
                    Map<String, Object> finInstnIdMaps = new HashMap<String, Object>();
                    mt_to_mxClearingSystemToNameAndAddressLine(partyIdentifier, null, finInstnIdMaps, "IntrmyAgt1/FinInstnId");
                    MapUtil.puts(txInfMaps, "rtrChain/intrmyAgt1/finInstnId", finInstnIdMaps);
                }
            }
        }
        if (tag56D != null) {
            Field56D field56D = (Field56D) tag56D.asField();
            String ptyId = field56D.getComponent1() == null ? field56D.getComponent2() : null;
            if (ptyId != null && ptyId.startsWith("/") && !ptyId.startsWith("/CH")) {
                Map<String, Object> clrSysMmbIdMap = new HashMap<String, Object>();
                mt_to_mxClearingIdentifier("/" + ptyId, clrSysMmbIdMap);
                MapUtil.puts(txInfMaps, "rtrChain/intrmyAgt1/finInstnId/clrSysMmbId", clrSysMmbIdMap);
            }
            Map<String, Object> intrmyAgt1Map = new HashMap<String, Object>();
            mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(field56D.getNameAndAddress(), intrmyAgt1Map);
            MapUtil.puts(txInfMaps, "rtrChain/intrmyAgt1", intrmyAgt1Map);
        }
    }
}
