package com.brilliance.swift.mt2mx.pacs004001.mt103retn;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs004001.AbstractMt2MxPacs004001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.SwiftMessage;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field57A;
import com.prowidesoftware.swift.model.field.Field57B;
import com.prowidesoftware.swift.model.field.Field57C;
import com.prowidesoftware.swift.model.field.Field57D;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.HashMap;
import java.util.Map;

public class Pacs004001Parse57Field extends AbstractMt2MxPacs004001ParseField {

    private static final String NAME_A = "57A";

    private static final String NAME_B = "57B";

    private static final String NAME_C = "57C";

    private static final String NAME_D = "57D";

    @Override
    public void parseField() throws SwiftException {
        SwiftMessage swiftMessage = context.get(AbstractMT.class).getSwiftMessage();
        Tag tag57A = swiftMessage.getBlock4().getTagByName(NAME_A);
        Tag tag57B = swiftMessage.getBlock4().getTagByName(NAME_B);
        Tag tag57C = swiftMessage.getBlock4().getTagByName(NAME_C);
        Tag tag57D = swiftMessage.getBlock4().getTagByName(NAME_D);

        if (tag57A != null) {
            Field57A field57A = (Field57A) tag57A.asField();
            String ptyId = field57A.getComponent1() == null ? field57A.getComponent2() : null;
            if (ptyId != null && ptyId.startsWith("/") && !ptyId.startsWith("/CH") && !ptyId.startsWith("/FW") && !ptyId.startsWith("/RT")) {
                Map<String, Object> clrSysMmbIdMap = new HashMap<String, Object>();
                mt_to_mxClearingIdentifier("/" + ptyId, clrSysMmbIdMap);
                MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/clrSysMmbId", clrSysMmbIdMap);
            }
            MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/bicfi", field57A.getBIC());
        }
        if (tag57B != null) {
            Field57B field57B = (Field57B) tag57B.asField();
            String ptyId = field57B.getComponent1() == null ? field57B.getComponent2() : null;
            if (field57B.getComponent1() == null && field57B.getComponent2() == null && StringUtil.isNotEmpty(field57B.getLocation())) {
                buildSTErrorInfo(14, "Block4/:57B:", null);
                MapUtil.adds(txInfMaps, "rtrChain/cdtrAgt/finInstnId/pstlAdr/adrLine", field57B.getLocation());
                MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/nm", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            } else if (ptyId != null && ptyId.startsWith("/") && !ptyId.startsWith("/CH")) {
                ptyId = "/" + ptyId;
                if (isMTClearingSystemCodeInList(ptyId)) {
                    Map<String, Object> clrSysMmbIdMap = new HashMap<String, Object>();
                    mt_to_mxClearingIdentifier(ptyId, clrSysMmbIdMap);
                    MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/clrSysMmbId", clrSysMmbIdMap);
                    MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/nm", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                    MapUtil.adds(txInfMaps, "rtrChain/cdtrAgt/finInstnId/pstlAdr/adrLine", StringUtil.isEmpty(field57B.getLocation()) ? Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE : field57B.getLocation());
                } else {
                    Map<String, Object> finInstnIdMap = new HashMap<String, Object>();
                    mt_to_mxClearingSystemToNameAndAddressLine(ptyId, field57B.getLocation(), finInstnIdMap, "CdtrAgt/FinInstnId");
                    MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId", finInstnIdMap);
                }
            } else if (StringUtil.isNotEmpty(field57B.getLocation())) {
                MapUtil.adds(txInfMaps, "rtrChain/cdtrAgt/finInstnId/pstlAdr/adrLine", field57B.getLocation());
                MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/nm", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            }
        }
        if (tag57C != null) {
            Field57C field57C = (Field57C) tag57C.asField();
            String component1 = field57C.getComponent1();
            if (component1.startsWith("/") && !component1.startsWith("/CH")) {
                component1 = "/" + component1;
                if (isMTClearingSystemCodeInList(component1)) {
                    Map<String, Object> clrSysMmbIdMap = new HashMap<String, Object>();
                    mt_to_mxClearingIdentifier(component1, clrSysMmbIdMap);
                    MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/clrSysMmbId", clrSysMmbIdMap);
                    MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/nm", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                    MapUtil.adds(txInfMaps, "rtrChain/cdtrAgt/finInstnId/pstlAdr/adrLine", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                } else {
                    Map<String, Object> finInstnIdMap = new HashMap<String, Object>();
                    mt_to_mxClearingSystemToNameAndAddressLine(component1, null, finInstnIdMap, "CdtrAgt/FinInstnId");
                    MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId", finInstnIdMap);
                }
            } else {
                MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/nm", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                MapUtil.adds(txInfMaps, "rtrChain/cdtrAgt/finInstnId/pstlAdr/adrLine", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            }
        }
        if (tag57D != null) {
            Field57D field57D = (Field57D) tag57D.asField();
            String ptyId = field57D.getComponent1() == null ? field57D.getComponent2() : null;
            if (ptyId != null && ptyId.startsWith("/") && !ptyId.startsWith("/CH")) {
                ptyId = "/" + ptyId;
                if (isMTClearingSystemCodeInList(ptyId)) {
                    Map<String, Object> clrSysMmbIdMap = new HashMap<String, Object>();
                    mt_to_mxClearingIdentifier(ptyId, clrSysMmbIdMap);
                    MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt/finInstnId/clrSysMmbId", clrSysMmbIdMap);
                }
            }
            Map<String, Object> cdtrAgtMap = new HashMap<String, Object>();
            mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(field57D.getNameAndAddress(), cdtrAgtMap);
            MapUtil.puts(txInfMaps, "rtrChain/cdtrAgt", cdtrAgtMap);
        }
    }
}
