package com.brilliance.swift.mt2mx.pacs004001.mt103retn;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs004001.AbstractMt2MxPacs004001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.SwiftBlock4;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field72;
import com.prowidesoftware.swift.model.field.SwiftParseUtils;
import com.prowidesoftware.swift.model.mt.AbstractMT;
import com.prowidesoftware.swift.utils.SwiftFormatUtils;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Pacs004001Parse72Field extends AbstractMt2MxPacs004001ParseField {

    private static final String NAME = "72";

    @Override
    public void parseField() throws SwiftException {
        AbstractMT abstractMT = context.get(AbstractMT.class);
        SwiftBlock4 block4 = abstractMT.getSwiftMessage().getBlock4();
        Tag tag72 = block4.getTagByName(NAME);
        if (tag72 != null) {
            Field72 field72 = (Field72) tag72.asField();
            List<String> lines = field72.getLines();
            String chargInfo = null;
            String orgnlInstrId = null;
            String orgnlEndToEndId = null;
            for (String line : lines) {
                if (line.startsWith("/MREF/")) {
                    orgnlInstrId = line.substring(6);
                } else if (line.startsWith("/TREF/")) {
                    orgnlEndToEndId = line.substring(6);
                } else if (line.startsWith("/CHGS/")) {
                    chargInfo = line.substring(6);
                }
            }
            if (StringUtil.isNotEmpty(orgnlInstrId)) {
                txInfMaps.put("orgnlInstrId", orgnlInstrId);
            }
            if (!"202".equals(abstractMT.getMessageType()) && !"205".equals(abstractMT.getMessageType())) {
                txInfMaps.put("orgnlEndToEndId", StringUtil.isEmpty(orgnlEndToEndId) ? Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE : orgnlEndToEndId);
            }
            if (StringUtil.isNotEmpty(chargInfo) && block4.getTagByName("71G") == null && block4.getTagByName("71F") == null) {
                Number amtNumber = SwiftFormatUtils.getNumber(SwiftParseUtils.getNumericSuffix(chargInfo));
                String amount = new BigDecimal(amtNumber.toString()).stripTrailingZeros().toPlainString();
                Map<String, Object> chrgsInfMap = new HashMap<String, Object>();
                MapUtil.adds(txInfMaps, "chrgsInf", chrgsInfMap);
                chrgsInfMap.put("amt/ccy", SwiftParseUtils.getAlphaPrefix(chargInfo));
                chrgsInfMap.put("amt/value", amount);
                MapUtil.puts(chrgsInfMap, "agt/finInstnId/nm", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                MapUtil.adds(chrgsInfMap, "pstlAdr/adrLine", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            }
            mt_to_mxReturn72(tag72.getValue(), txInfMaps);
        }
    }
}
