package com.brilliance.swift.mt2mx.pacs008001.impl;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs008001.AbstractMt2MxPacs008001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.vo.common.CategoryPurposeCode;
import com.brilliance.swift.vo.common.ExternalCreditorAgentInstructionCode;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field23E;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Pacs00801Parse23EField extends AbstractMt2MxPacs008001ParseField {

    private static final String NAME_23E = "23E";

    private static final String NAME_72 = "72";

    @Override
    public void parseField() throws SwiftException {
        AbstractMT abstractMT = context.get(AbstractMT.class);
        Tag[] tags = abstractMT.getSwiftMessage().getBlock4().getTagsByName(NAME_23E);
        List<Field23E> field23EList = new ArrayList<Field23E>();
        if (tags != null && tags.length > 0) {
            Field23E intcField23E = null;
            Field23E cortField23E = null;
            for (int i=0; i<tags.length; i++) {
                Field23E field23E = (Field23E)tags[i].asField();
                if (ExternalCreditorAgentInstructionCode.HOLD.value().equals(field23E.getCode())
                    || ExternalCreditorAgentInstructionCode.PHOB.value().equals(field23E.getCode())
                    || ExternalCreditorAgentInstructionCode.CHQB.value().equals(field23E.getCode())
                    || ExternalCreditorAgentInstructionCode.TELB.value().equals(field23E.getCode())) {
                    field23EList.add(field23E);
                } else if ("SDVA".equals(field23E.getCode())) {
                    Map<String, Object> svcLvlMaps = new HashMap<String, Object>();
                    MapUtil.adds(cdtTrfTxInfMaps, "pmtTpInf/svcLvl", svcLvlMaps);
                    svcLvlMaps.put("cd", field23E.getCode());
                } else if (CategoryPurposeCode.INTC.value().equals(field23E.getCode())) {
                    intcField23E = field23E;
                } else if (CategoryPurposeCode.CORT.value().equals(field23E.getCode())) {
                    cortField23E = field23E;
                }
            }
            if (intcField23E != null && cortField23E != null) {
                MapUtil.puts(cdtTrfTxInfMaps, "pmtTpInf/ctgyPurp/prtry", CategoryPurposeCode.INTC.value() + " " + CategoryPurposeCode.CORT.value());
            } else if (intcField23E != null) {
                MapUtil.puts(cdtTrfTxInfMaps, "pmtTpInf/ctgyPurp/cd", CategoryPurposeCode.INTC.value());
            } else if (cortField23E != null) {
                MapUtil.puts(cdtTrfTxInfMaps, "pmtTpInf/ctgyPurp/cd", CategoryPurposeCode.CORT.value());
            }
        }
        Tag tag72 = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_72);
        String mt72 = "";
        if (tag72 != null) {
            mt72 = tag72.getValue();
        }
        mt_to_mxInstructionForCreditorAgent(field23EList, mt72, cdtTrfTxInfMaps);
    }
}
