package com.brilliance.swift.mt2mx.pacs008001.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs008001.AbstractMt2MxPacs008001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.OrganisationIdentificationCode;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field50A;
import com.prowidesoftware.swift.model.field.Field50F;
import com.prowidesoftware.swift.model.field.Field50K;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Pacs00801Parse50Field extends AbstractMt2MxPacs008001ParseField {

    private static final String NAME_A = "50A";

    private static final String NAME_F = "50F";

    private static final String NAME_K = "50K";

    @Override
    public void parseField() throws SwiftException {
        Field50A field50A = null;
        Field50F field50F = null;
        Field50K field50K = null;
        AbstractMT abstractMT = context.get(AbstractMT.class);
        Tag tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_A);
        if (tag != null) {
            field50A = (Field50A)tag.asField();
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_F);
        if (tag != null) {
            field50F = (Field50F)tag.asField();
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_K);
        if (tag != null) {
            field50K = (Field50K)tag.asField();
        }
        Map<String, Object> dbtrMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "dbtr");
        Map<String, Object> dbtrAcctMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "dbtrAcct");
        String account = "";
        if (field50A != null) {
            String bicCode = field50A.getBIC();
            if (StringUtil.isNotEmpty(field50A.getComponent1())) {
                account += "/" + field50A.getComponent1();
            }
            mt_to_mxAnyBIC(bicCode, dbtrMaps);
            if (StringUtil.isEmpty(account)) {
                buildDefaultValue(dbtrMaps);
            } else {
                mt_to_mxPartyAccount(account, dbtrAcctMaps);
            }
        }
        if (field50F != null) {
            String partyIdentifier = field50F.getPartyIdentifier();
            String nameAddress = "";
            List<String> lines = field50F.getLines();
            for (int i=0; i<lines.size(); i++) {
                if (i == 0) {
                    continue;
                }
                if (i == lines.size()-1) {
                    nameAddress += lines.get(i);
                } else {
                    nameAddress += lines.get(i) + Mx2MtConstants.NEW_LINE;
                }
            }
            if (partyIdentifier.startsWith("/")) {
                mt_to_mxPartyAccount(partyIdentifier, dbtrAcctMaps);
            } else {
                Map<String, Object> idMaps = MapUtil.getAndCreateNotPut(dbtrMaps, "id");
                mt_to_mxFATFIdentification(partyIdentifier, nameAddress, idMaps);
                if(idMaps.size()>0){
                    dbtrMaps.put("id", idMaps);
                }
            }
            mt_to_mxFATFNameAndAddress(partyIdentifier, nameAddress, dbtrMaps);
        }
        if (field50K != null) {
            String nameAddress = field50K.getNameAndAddress();
            if (StringUtil.isNotEmpty(field50K.getComponent1())) {
                account += "/" + field50K.getComponent1();
            }
            mt_to_mxPartyNameAndAddress(nameAddress, dbtrMaps);
            if (StringUtil.isEmpty(account)) {
                buildDefaultValue(dbtrMaps);
            } else {
                mt_to_mxPartyAccount(account, dbtrAcctMaps);
            }
        }
        if(dbtrMaps.size()>0){
            cdtTrfTxInfMaps.put("dbtr",dbtrMaps);
        }
        if(dbtrAcctMaps.size()>0){
            cdtTrfTxInfMaps.put("dbtrAcct",dbtrAcctMaps);
        }
    }

    /**
     * IF 50A/Account 50K/Account IsAbsent
     * THEN Copy "NOTPROVIDED" to Debtor/Id/OrgID/Other/ID
     * and "TXID" to SchemeName/Code
     * @param dbtrMaps
     */
    private void buildDefaultValue(Map<String, Object> dbtrMaps) {
        Map<String, Object> othrMaps = new HashMap<String, Object>();
        MapUtil.adds(dbtrMaps, "id/orgId/othr", othrMaps);
        othrMaps.put("id", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
        MapUtil.puts(othrMaps, "schmeNm/cd", OrganisationIdentificationCode.TXID.value());
    }
}
