package com.brilliance.swift.mt2mx.pacs008001.impl;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.Mt2MxContextIdentifier;
import com.brilliance.swift.mt2mx.pacs008001.AbstractMt2MxPacs008001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field52A;
import com.prowidesoftware.swift.model.field.Field52D;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.Map;

public class Pacs00801Parse52Field extends AbstractMt2MxPacs008001ParseField {

    private static final String NAME_A = "52A";

    private static final String NAME_D = "52D";

    @Override
    public void parseField() throws SwiftException {
        Field52A field52A = null;
        Field52D field52D = null;
        AbstractMT abstractMT = context.get(AbstractMT.class);
        Tag tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_A);
        if (tag != null) {
            field52A = (Field52A)tag.asField();
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_D);
        if (tag != null) {
            field52D = (Field52D)tag.asField();
        }
        Map<String, Object> dbtrAgtMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "dbtrAgt");
        Map<String, Object> dbtrAgtAcctMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "dbtrAgtAcct");
        String account = "";
        if (field52A != null) {
            String bicCode = field52A.getBIC();
            mt_to_mxBICFI(bicCode, dbtrAgtMaps);
            if (StringUtil.isNotEmpty(field52A.getComponent1())) {
                account += "/" + field52A.getComponent1();
            }
            if (StringUtil.isNotEmpty(field52A.getComponent2())) {
                account += "/" + field52A.getComponent2();
            }
            if (StringUtil.isNotEmpty(account)) {
                if (account.startsWith("//") && !account.startsWith("//CH") && !account.startsWith("//FW")) {
                    Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(dbtrAgtMaps, "finInstnId");
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                    }
                    if(finInstnIdMaps.size()>0){
                        dbtrAgtMaps.put("finInstnId", finInstnIdMaps);
                    }
                } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                    mt_to_mxFinancialInstitutionAccount(account, dbtrAgtAcctMaps, "DbtrAgtAcct/Id");
                }
            }
        } else if (field52D != null) {
            if (StringUtil.isNotEmpty(field52D.getComponent1())) {
                account += "/" + field52D.getComponent1();
            }
            if (StringUtil.isNotEmpty(field52D.getComponent2())) {
                account += "/" + field52D.getComponent2();
            }
            String nameAddress = field52D.getNameAndAddress();
            mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(nameAddress, dbtrAgtMaps);
            if (StringUtil.isNotEmpty(account)) {
                if (account.startsWith("//") && !account.startsWith("//CH")) {
                    Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(dbtrAgtMaps, "finInstnId");
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                    }
                    if(finInstnIdMaps.size()>0){
                        dbtrAgtMaps.put("finInstnId", finInstnIdMaps);
                    }
                } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                    mt_to_mxFinancialInstitutionAccount(account, dbtrAgtAcctMaps, "DbtrAgtAcct/Id");
                }
            }
        } else {
            String sendBic = (String) context.get(Mt2MxContextIdentifier.MT_SEND_BIC, true);
            mt_to_mxBICFI(sendBic, dbtrAgtMaps);
        }
        if (dbtrAgtMaps.size() > 0) {
            cdtTrfTxInfMaps.put("dbtrAgt", dbtrAgtMaps);
        }
        if (dbtrAgtAcctMaps.size() > 0) {
            cdtTrfTxInfMaps.put("dbtrAgtAcct", dbtrAgtAcctMaps);
        }
    }
}
