package com.brilliance.swift.mt2mx.pacs008001.impl;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs008001.AbstractMt2MxPacs008001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.VoSettlementMethodHelper;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field53A;
import com.prowidesoftware.swift.model.field.Field53D;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.Map;

public class Pacs00801Parse53Field extends AbstractMt2MxPacs008001ParseField {

    private static final String NAME_A = "53A";

    //private static final String NAME_B = "53B";

    private static final String NAME_D = "53D";

    @Override
    public void parseField() throws SwiftException {
        VoSettlementMethodHelper setlMtdHelper = context.get(VoSettlementMethodHelper.class);
        if (setlMtdHelper.isTranslate53Flag()) {
            //53B 只能翻译成 Group Header -> Settlement Information -> Settlement Account/Settlement Method
            Field53A field53A = null;
            Field53D field53D = null;
            AbstractMT abstractMT = context.get(AbstractMT.class);
            Tag tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_A);
            if (tag != null) {
                field53A = (Field53A)tag.asField();
            }
            tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_D);
            if (tag != null) {
                field53D = (Field53D)tag.asField();
            }
            Map<String, Object> sttlmInfMaps = MapUtil.getAndCreateNotPut(grpHdrMaps, "sttlmInf");
            Map<String, Object> instgRmbrsmntAgtMaps = MapUtil.getAndCreateNotPut(sttlmInfMaps, "instgRmbrsmntAgt");
            String account = "";
            if (field53A != null) {
                String bicCode = field53A.getBIC();
                mt_to_mxBICFI(bicCode, instgRmbrsmntAgtMaps);
                if (StringUtil.isNotEmpty(field53A.getComponent1())) {
                    account += "/" + field53A.getComponent1();
                }
                if (StringUtil.isNotEmpty(field53A.getComponent2())) {
                    account += "/" + field53A.getComponent2();
                }
            } else if (field53D != null) {
                String nameAddress = field53D.getNameAndAddress();
                mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(nameAddress, instgRmbrsmntAgtMaps);
                if (StringUtil.isNotEmpty(field53D.getComponent1())) {
                    account += "/" + field53D.getComponent1();
                }
                if (StringUtil.isNotEmpty(field53D.getComponent2())) {
                    account += "/" + field53D.getComponent2();
                }
            }
            if (StringUtil.isNotEmpty(account)) {
                Map<String, Object> instgRmbrsmntAgtAcctMaps = MapUtil.getAndCreateNotPut(sttlmInfMaps, "instgRmbrsmntAgtAcct");
                if (account.startsWith("//") && !account.startsWith("//CH")) {
                    Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(instgRmbrsmntAgtMaps, "finInstnId");
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                    }
                    if(finInstnIdMaps.size()>0){
                        instgRmbrsmntAgtMaps.put("finInstnId", finInstnIdMaps);
                    }
                } else {
                    mt_to_mxFinancialInstitutionAccount(account, instgRmbrsmntAgtAcctMaps, "InstgRmbrsmntAgtAcct/Id");
                }
                if(instgRmbrsmntAgtAcctMaps.size()>0){
                    sttlmInfMaps.put("instgRmbrsmntAgtAcct", instgRmbrsmntAgtAcctMaps);
                }
            }
            if(instgRmbrsmntAgtMaps.size()>0){
                sttlmInfMaps.put("instgRmbrsmntAgt", instgRmbrsmntAgtMaps);
            }
            if(sttlmInfMaps.size()>0){
                grpHdrMaps.put("sttlmInf", sttlmInfMaps);
            }
        }

    }
}
