package com.brilliance.swift.mt2mx.pacs008001.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs008001.AbstractMt2MxPacs008001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.VoSettlementMethodHelper;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field54A;
import com.prowidesoftware.swift.model.field.Field54B;
import com.prowidesoftware.swift.model.field.Field54D;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.Map;

public class Pacs00801Parse54Field extends AbstractMt2MxPacs008001ParseField {
    private static final String NAME_A = "54A";

    private static final String NAME_B = "54B";

    private static final String NAME_D = "54D";

    @Override
    public void parseField() throws SwiftException {
        VoSettlementMethodHelper setlMtdHelper = context.get(VoSettlementMethodHelper.class);
        if (setlMtdHelper.isTranslate54Flag()) {
            Field54A field54A = null;
            Field54B field54B = null;
            Field54D field54D = null;
            AbstractMT abstractMT = context.get(AbstractMT.class);
            Tag tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_A);
            if (tag != null) {
                field54A = (Field54A)tag.asField();
            }
            tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_B);
            if (tag != null) {
                field54B = (Field54B)tag.asField();
            }
            tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_D);
            if (tag != null) {
                field54D = (Field54D)tag.asField();
            }
            Map<String, Object> sttlmInfMaps = MapUtil.getAndCreateNotPut(grpHdrMaps, "sttlmInf");
            Map<String, Object> instdRmbrsmntAgtMaps = MapUtil.getAndCreateNotPut(sttlmInfMaps, "instdRmbrsmntAgt");
            Map<String, Object> instdRmbrsmntAgtAcctMaps = MapUtil.getAndCreateNotPut(sttlmInfMaps, "instdRmbrsmntAgtAcct");
            String account = "";
            if (field54A != null) {
                String bicCode = field54A.getBIC();
                mt_to_mxBICFI(bicCode, instdRmbrsmntAgtMaps);
                if (StringUtil.isNotEmpty(field54A.getComponent1())) {
                    account += "/" + field54A.getComponent1();
                }
                if (StringUtil.isNotEmpty(field54A.getComponent2())) {
                    account += "/" + field54A.getComponent2();
                }
                if (account.startsWith("//") && !account.startsWith("//CH")) {
                    Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(instdRmbrsmntAgtMaps, "finInstnId");
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                    }
                    if(finInstnIdMaps.size()>0){
                        instdRmbrsmntAgtMaps.put("finInstnId", finInstnIdMaps);
                    }
                } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                    mt_to_mxFinancialInstitutionAccount(account, instdRmbrsmntAgtAcctMaps, "InstdRmbrsmntAgtAcct/Id");
                }
            } else if (field54B != null) {
                if (StringUtil.isNotEmpty(field54B.getComponent1())) {
                    account += "/" + field54B.getComponent1();
                }
                if (StringUtil.isNotEmpty(field54B.getComponent2())) {
                    account += "/" + field54B.getComponent2();
                }
                String location = field54B.getLocation();
                Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(instdRmbrsmntAgtMaps, "finInstnId");
                if (account.startsWith("//") && !account.startsWith("//CH")) {
                    if (isMTClearingSystemCodeInList(account)) {
                        Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                        mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                        if(clrSysMmbIdMaps.size()>0){
                            finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                        }
                        //name address 设置NOTPROVIDED
                        finInstnIdMaps.put("nm", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                        MapUtil.adds(finInstnIdMaps, "pstlAdr/adrLine", StringUtil.isEmpty(location)?Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE:location);
                    } else {
                        mt_to_mxClearingSystemToNameAndAddressLine(account, location, finInstnIdMaps, "InstdRmbrsmntAgt/FinInstnId");
                    }
                } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                    mt_to_mxFinancialInstitutionAccount(account, instdRmbrsmntAgtAcctMaps, "InstdRmbrsmntAgtAcct/Id");
                    if (StringUtil.isEmpty(account)) {
                        buildSTErrorInfo(14, "Block4/:54B:", null);
                    }
                    //name address 设置NOTPROVIDED
                    finInstnIdMaps.put("nm", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                    MapUtil.adds(finInstnIdMaps, "pstlAdr/adrLine", StringUtil.isEmpty(location)?Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE:location);
                }
                if(finInstnIdMaps.size()>0){
                    instdRmbrsmntAgtMaps.put("finInstnId",finInstnIdMaps);
                }
            } else if (field54D != null) {
                if (StringUtil.isNotEmpty(field54D.getComponent1())) {
                    account += "/" + field54D.getComponent1();
                }
                if (StringUtil.isNotEmpty(field54D.getComponent2())) {
                    account += "/" + field54D.getComponent2();
                }
                String nameAddress = field54D.getNameAndAddress();
                mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(nameAddress, instdRmbrsmntAgtMaps);
                if (account.startsWith("//") && !account.startsWith("//CH")) {
                    Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(instdRmbrsmntAgtMaps, "finInstnId");
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                    }
                    if(finInstnIdMaps.size()>0){
                        instdRmbrsmntAgtMaps.put("finInstnId", finInstnIdMaps);
                    }
                } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                    mt_to_mxFinancialInstitutionAccount(account, instdRmbrsmntAgtAcctMaps, "InstdRmbrsmntAgtAcct/Id");
                }
            }
            if (instdRmbrsmntAgtMaps.size() > 0) {
                sttlmInfMaps.put("instdRmbrsmntAgt", instdRmbrsmntAgtMaps);
            }
            if (instdRmbrsmntAgtAcctMaps.size() > 0) {
                sttlmInfMaps.put("instdRmbrsmntAgtAcct", instdRmbrsmntAgtAcctMaps);
            }
            if(sttlmInfMaps.size()>0){
                grpHdrMaps.put("sttlmInf", sttlmInfMaps);
            }
        }
    }
}
