package com.brilliance.swift.mt2mx.pacs008001.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.Mt2MxContextIdentifier;
import com.brilliance.swift.mt2mx.pacs008001.AbstractMt2MxPacs008001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field57A;
import com.prowidesoftware.swift.model.field.Field57B;
import com.prowidesoftware.swift.model.field.Field57C;
import com.prowidesoftware.swift.model.field.Field57D;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.Map;

public class Pacs00801Parse57Field extends AbstractMt2MxPacs008001ParseField {

    private static final String NAME_A = "57A";

    private static final String NAME_B = "57B";

    private static final String NAME_C = "57C";

    private static final String NAME_D = "57D";

    @Override
    public void parseField() throws SwiftException {
        Field57A field57A = null;
        Field57B field57B = null;
        Field57C field57C = null;
        Field57D field57D = null;
        AbstractMT abstractMT = context.get(AbstractMT.class);
        Tag tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_A);
        if (tag != null) {
            field57A = (Field57A)tag.asField();
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_B);
        if (tag != null) {
            field57B = (Field57B)tag.asField();
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_C);
        if (tag != null) {
            field57C = (Field57C)tag.asField();
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName(NAME_D);
        if (tag != null) {
            field57D = (Field57D)tag.asField();
        }
        Map<String, Object> cdtrAgtMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "cdtrAgt");
        Map<String, Object> cdtrAgtAcctMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "cdtrAgtAcct");
        boolean clearingChannelRTGSFlag = false;
        String account = "";
        if (field57A != null) {
            String bicCode = field57A.getBIC();
            mt_to_mxBICFI(bicCode, cdtrAgtMaps);
            if (StringUtil.isNotEmpty(field57A.getComponent1())) {
                account += "/" + field57A.getComponent1();
            }
            if (StringUtil.isNotEmpty(field57A.getComponent2())) {
                account += "/" + field57A.getComponent2();
            }
            if (account.startsWith("//RT") || account.startsWith("//FW")) {
                clearingChannelRTGSFlag = true;
            }
            if (account.startsWith("//") && !account.startsWith("//CH") && !account.startsWith("//FW") && !account.startsWith("//RT")) {
                Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(cdtrAgtMaps, "finInstnId");
                Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                if(clrSysMmbIdMaps.size()>0){
                    finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                }
                if(finInstnIdMaps.size()>0){
                    cdtrAgtMaps.put("finInstnId", finInstnIdMaps);
                }
            } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                mt_to_mxFinancialInstitutionAccount(account, cdtrAgtAcctMaps, "CdtrAgtAcct/Id");
            }
        } else if (field57B != null) {
            if (StringUtil.isNotEmpty(field57B.getComponent1())) {
                account += "/" + field57B.getComponent1();
            }
            if (StringUtil.isNotEmpty(field57B.getComponent2())) {
                account += "/" + field57B.getComponent2();
            }
            String location = field57B.getLocation();
            Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(cdtrAgtMaps, "finInstnId");
            if (account.startsWith("//") && !account.startsWith("//CH")) {
                if (isMTClearingSystemCodeInList(account)) {
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                    }
                    //name address 设置NOTPROVIDED
                    finInstnIdMaps.put("nm", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                    MapUtil.adds(finInstnIdMaps, "pstlAdr/adrLine", StringUtil.isEmpty(location)?Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE:location);
                } else {
                    mt_to_mxClearingSystemToNameAndAddressLine(account, location, finInstnIdMaps, "CdtrAgt/FinInstnId");
                }
            } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                mt_to_mxFinancialInstitutionAccount(account, cdtrAgtAcctMaps, "CdtrAgtAcct/Id");
                if (StringUtil.isEmpty(account)) {
                    buildSTErrorInfo(14, "Block4/:57B:", null);
                }
                //name address 设置NOTPROVIDED
                finInstnIdMaps.put("nm", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                MapUtil.adds(finInstnIdMaps, "pstlAdr/adrLine", StringUtil.isEmpty(location)?Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE:location);
            }
            if(finInstnIdMaps.size()>0){
                cdtrAgtMaps.put("finInstnId", finInstnIdMaps);
            }
        } else if (field57C != null) {
            account = field57C.getValue();
            if (account.startsWith("//RT")) {
                clearingChannelRTGSFlag = true;
            }
            Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(cdtrAgtMaps, "finInstnId");
            if (account.startsWith("//") && !account.startsWith("//CH") && !account.startsWith("//FW")) {
                if (isMTClearingSystemCodeInList(account)) {
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                    }
                    //name address 设置NOTPROVIDED
                    finInstnIdMaps.put("nm", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                    MapUtil.adds(finInstnIdMaps, "pstlAdr/adrLine", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                } else {
                    mt_to_mxClearingSystemToNameAndAddressLine(account, null, finInstnIdMaps, "CdtrAgt/FinInstnId");
                }
            } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                mt_to_mxFinancialInstitutionAccount(account, cdtrAgtAcctMaps, "CdtrAgtAcct/Id");
                //name address 设置NOTPROVIDED
                finInstnIdMaps.put("nm", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                MapUtil.adds(finInstnIdMaps, "pstlAdr/adrLine", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
            }
            if(finInstnIdMaps.size()>0){
                cdtrAgtMaps.put("finInstnId", finInstnIdMaps);
            }
        } else if (field57D != null) {
            if (StringUtil.isNotEmpty(field57D.getComponent1())) {
                account += "/" + field57D.getComponent1();
            }
            if (StringUtil.isNotEmpty(field57D.getComponent2())) {
                account += "/" + field57D.getComponent2();
            }
            if (account.startsWith("//RT")) {
                clearingChannelRTGSFlag = true;
            }
            String nameAddress = field57D.getNameAndAddress();
            mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(nameAddress, cdtrAgtMaps);
            if (account.startsWith("//") && !account.startsWith("//CH") && !account.startsWith("//FW")) {
                Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(cdtrAgtMaps, "finInstnId");
                Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                if(clrSysMmbIdMaps.size()>0){
                    finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                }
                if(finInstnIdMaps.size()>0){
                    cdtrAgtMaps.put("finInstnId", finInstnIdMaps);
                }
            } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                mt_to_mxFinancialInstitutionAccount(account, cdtrAgtAcctMaps, "CdtrAgtAcct/Id");
            }
        } else {
            String receiverBic = (String) context.get(Mt2MxContextIdentifier.MT_RECEIVE_BIC, true);
            mt_to_mxBICFI(receiverBic, cdtrAgtMaps);
        }
        if (cdtrAgtMaps.size() > 0) {
            cdtTrfTxInfMaps.put("cdtrAgt", cdtrAgtMaps);
        }
        if (cdtrAgtAcctMaps.size() > 0) {
            cdtTrfTxInfMaps.put("cdtrAgtAcct", cdtrAgtAcctMaps);
        }
        if (clearingChannelRTGSFlag) {
            MapUtil.puts(cdtTrfTxInfMaps, "pmtTpInf/clrChanl", "RTGS");
        }
    }
}
