package com.brilliance.swift.mt2mx.pacs009001;

import com.brilliance.swift.mt2mx.AbstractMt2MxCreator;
import com.brilliance.swift.mt2mx.Mt2MxContextIdentifier;
import com.brilliance.swift.mt2mx.Mt2MxParseField;
import com.brilliance.swift.mt2mx.pacs009001.impl.*;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Mt2MxPacs009001Creator extends AbstractMt2MxCreator {

    @Override
    public void preProcess() {
        super.preProcess();
        jsonMaps.put("identifier", "pacs.009.001.08");
        MapUtil.puts(jsonMaps, "appHdr/msgDefIdr", "pacs.009.001.08");
        MapUtil.puts(jsonMaps, "fiCdtTrf/grpHdr/creDtTm", getCreateDateStr());
        MapUtil.puts(jsonMaps, "fiCdtTrf/grpHdr/nbOfTxs", "1");
        Map<String, Object> cdtTrfTxInfMaps = new HashMap<String, Object>();
        MapUtil.adds(jsonMaps, "fiCdtTrf/cdtTrfTxInf", cdtTrfTxInfMaps);
        AbstractMT abstractMT = context.get(AbstractMT.class);
        String uetr = abstractMT.getSwiftMessage().getUETR();
        if (StringUtil.isNotEmpty(uetr)) {
            MapUtil.puts(cdtTrfTxInfMaps, "pmtId/uetr", uetr);
        }
        //设置InstructingAgent InstructedAgent 取值从sendbic receivebic
        String sendBic = (String) context.get(Mt2MxContextIdentifier.MT_SEND_BIC, true);
        if (StringUtil.isNotEmpty(sendBic)) {
            MapUtil.puts(cdtTrfTxInfMaps, "instgAgt/finInstnId/bicfi", sendBic);
        }
        String receiverBic = (String) context.get(Mt2MxContextIdentifier.MT_RECEIVE_BIC, true);
        if (StringUtil.isNotEmpty(receiverBic)) {
            MapUtil.puts(cdtTrfTxInfMaps, "instdAgt/finInstnId/bicfi", receiverBic);
        }

        //设置Service Level 取值从block3 {111:00[0-9]}
        if (abstractMT.getSwiftMessage().getBlock3() != null) {
            Tag tag111 = abstractMT.getSwiftMessage().getBlock3().getTagByName("111");
            if (tag111 != null) {
                String svcLvlCode = "";
                String value = tag111.getValue();
                if (value.matches("00[0-9]{1}")) {
                    svcLvlCode = value;
                }
                if (StringUtil.isNotEmpty(svcLvlCode)) {
                    Map<String, Object> svcLvlMaps = new HashMap<String, Object>();
                    MapUtil.adds(cdtTrfTxInfMaps, "pmtTpInf/svcLvl", svcLvlMaps);
                    svcLvlMaps.put("cd", "G" + svcLvlCode);
                }
            }
        }
        //如果是cov，初始化Underlying Customer Credit Transfer map
        if (abstractMT.getSwiftMessage().isCOV()) {
            Map<String, Object> undrlygCstmrCdtTrfMaps = new HashMap<String, Object>();
            cdtTrfTxInfMaps.put("undrlygCstmrCdtTrf", undrlygCstmrCdtTrfMaps);
        }
    }

    @Override
    public List<Mt2MxParseField> getParseFieldList() {
        List<Mt2MxParseField> list = new ArrayList<Mt2MxParseField>();
        list.add(new Pacs009001Parse20Field());
        list.add(new Pacs009001Parse21Field());
        list.add(new Pacs009001Parse13CField());
        list.add(new Pacs009001Parse32AField());
        list.add(new Pacs009001Parse52Field());
        list.add(new Pacs009001ParseSetlMtdField());
        list.add(new Pacs009001ParseSetlAcctField());
        list.add(new Pacs009001Parse53Field());
        list.add(new Pacs009001Parse54Field());
        list.add(new Pacs009001Parse56Field());
        list.add(new Pacs009001Parse57Field());
        list.add(new Pacs009001Parse58Field());
        list.add(new Pacs009001Parse72Field());
        list.add(new Pacs009001ParseSeqB50Field());
        list.add(new Pacs009001ParseSeqB52Field());
        list.add(new Pacs009001ParseSeqB56Field());
        list.add(new Pacs009001ParseSeqB57Field());
        list.add(new Pacs009001ParseSeqB59Field());
        list.add(new Pacs009001ParseSeqB70Field());
        list.add(new Pacs009001ParseSeqB72Field());
        list.add(new Pacs009001ParseSeqB33BField());
        return list;
    }
}
