package com.brilliance.swift.mt2mx.pacs009001.impl;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.Mt2MxContextIdentifier;
import com.brilliance.swift.mt2mx.pacs009001.AbstractMt2MxPacs009001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field52A;
import com.prowidesoftware.swift.model.field.Field52D;

import java.util.Map;

public class Pacs009001Parse52Field extends AbstractMt2MxPacs009001ParseField {

    private static final String NAME_A = "52A";

    private static final String NAME_D = "52D";

    private static final String SEQUENCE_A = "A";

    @Override
    public void parseField() throws SwiftException {
        Field52A field52A = null;
        Field52D field52D = null;
        Tag tag = getTagByName(NAME_A, SEQUENCE_A);
        if (tag != null) {
            field52A = (Field52A)tag.asField();
        }
        tag = getTagByName(NAME_D, SEQUENCE_A);
        if (tag != null) {
            field52D = (Field52D)tag.asField();
        }
        Map<String, Object> dbtrMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "dbtr");
        Map<String, Object> dbtrAcctMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "dbtrAcct");
        String account = "";
        if (field52A != null) {
            String bicCode = field52A.getBIC();
            mt_to_mxBICFI(bicCode, dbtrMaps);
            if (StringUtil.isNotEmpty(field52A.getComponent1())) {
                account += "/" + field52A.getComponent1();
            }
            if (StringUtil.isNotEmpty(field52A.getComponent2())) {
                account += "/" + field52A.getComponent2();
            }
            if (StringUtil.isNotEmpty(account)) {
                if (account.startsWith("//") && !account.startsWith("//CH")&& !account.startsWith("//FW")) {
                    Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(dbtrMaps, "finInstnId");
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                    }
                    if(finInstnIdMaps.size()>0){
                        dbtrMaps.put("finInstnId", finInstnIdMaps);
                    }
                } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                    mt_to_mxFinancialInstitutionAccount(account, dbtrAcctMaps, "DbtrAcct/Id");
                }
            }
        } else if (field52D != null) {
            if (StringUtil.isNotEmpty(field52D.getComponent1())) {
                account += "/" + field52D.getComponent1();
            }
            if (StringUtil.isNotEmpty(field52D.getComponent2())) {
                account += "/" + field52D.getComponent2();
            }
            if (StringUtil.isNotEmpty(account)) {
                if (account.startsWith("//") && !account.startsWith("//CH")) {
                    Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(dbtrMaps, "finInstnId");
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                    }
                    if(finInstnIdMaps.size()>0){
                        dbtrMaps.put("finInstnId", finInstnIdMaps);
                    }
                } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                    mt_to_mxFinancialInstitutionAccount(account, dbtrAcctMaps, "DbtrAcct/Id");
                }
            }
            String nameAddress = field52D.getNameAndAddress();
            mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(nameAddress, dbtrMaps);
        } else {
            String sendBic = (String) context.get(Mt2MxContextIdentifier.MT_SEND_BIC, true);
            mt_to_mxBICFI(sendBic, dbtrMaps);
        }
        if (dbtrMaps.size() > 0) {
            cdtTrfTxInfMaps.put("dbtr", dbtrMaps);
        }
        if (dbtrAcctMaps.size() > 0) {
            cdtTrfTxInfMaps.put("dbtrAcct", dbtrAcctMaps);
        }
    }
}
