package com.brilliance.swift.mt2mx.pacs009001.impl;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs009001.AbstractMt2MxPacs009001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field56A;
import com.prowidesoftware.swift.model.field.Field56D;

import java.util.Map;

public class Pacs009001Parse56Field extends AbstractMt2MxPacs009001ParseField {

    private static final String NAME_A = "56A";

    //private static final String NAME_C = "56C";

    private static final String NAME_D = "56D";

    private static final String SEQUENCE_A = "A";

    @Override
    public void parseField() throws SwiftException {
        Field56A field56A = null;
        Field56D field56D = null;
        Tag tag = getTagByName(NAME_A, SEQUENCE_A);
        if (tag != null) {
            field56A = (Field56A)tag.asField();
        }
        tag = getTagByName(NAME_D, SEQUENCE_A);
        if (tag != null) {
            field56D = (Field56D)tag.asField();
        }
        Map<String, Object> intrmyAgt1Maps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "intrmyAgt1");
        Map<String, Object> intrmyAgt1AcctMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "intrmyAgt1Acct");
        boolean clearingChannelRTGSFlag = false;
        String account = "";
        if (field56A != null) {
            String bicCode = field56A.getBIC();
            mt_to_mxBICFI(bicCode, intrmyAgt1Maps);
            if (StringUtil.isNotEmpty(field56A.getComponent1())) {
                account += "/" + field56A.getComponent1();
            }
            if (StringUtil.isNotEmpty(field56A.getComponent2())) {
                account += "/" + field56A.getComponent2();
            }
            if (account.startsWith("//RT") || account.startsWith("//FW")) {
                clearingChannelRTGSFlag = true;
            }
            if (account.startsWith("//") && !account.startsWith("//CH") && !account.startsWith("//FW") && !account.startsWith("//RT")) {
                Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(intrmyAgt1Maps, "finInstnId");
                Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                if(clrSysMmbIdMaps.size()>0){
                    finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                }
                if(finInstnIdMaps.size()>0){
                    intrmyAgt1Maps.put("finInstnId", finInstnIdMaps);
                }
            } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                mt_to_mxFinancialInstitutionAccount(account, intrmyAgt1AcctMaps, "IntrmyAgt1Acct/Id");
            }
        } else if (field56D != null) {
            if (StringUtil.isNotEmpty(field56D.getComponent1())) {
                account += "/" + field56D.getComponent1();
            }
            if (StringUtil.isNotEmpty(field56D.getComponent2())) {
                account += "/" + field56D.getComponent2();
            }
            if (account.startsWith("//RT")) {
                clearingChannelRTGSFlag = true;
            }
            String nameAddress = field56D.getNameAndAddress();
            mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(nameAddress, intrmyAgt1Maps);
            if (account.startsWith("//") && !account.startsWith("//CH")) {
                Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(intrmyAgt1Maps, "finInstnId");
                Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                if(clrSysMmbIdMaps.size()>0){
                    finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                }
                if(finInstnIdMaps.size()>0){
                    intrmyAgt1Maps.put("finInstnId", finInstnIdMaps);
                }
            } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                mt_to_mxFinancialInstitutionAccount(account, intrmyAgt1AcctMaps, "IntrmyAgt1Acct/Id");
            }
        }
        if (intrmyAgt1Maps.size() > 0) {
            cdtTrfTxInfMaps.put("intrmyAgt1", intrmyAgt1Maps);
        }
        if (intrmyAgt1AcctMaps.size() > 0) {
            cdtTrfTxInfMaps.put("intrmyAgt1Acct", intrmyAgt1AcctMaps);
        }
        if (clearingChannelRTGSFlag) {
            MapUtil.puts(cdtTrfTxInfMaps, "pmtTpInf/clrChanl", "RTGS");
        }
    }
}
