package com.brilliance.swift.mt2mx.pacs009001.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mt2mx.pacs009001.AbstractMt2MxPacs009001ParseField;
import com.brilliance.swift.util.MapUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field56A;
import com.prowidesoftware.swift.model.field.Field56C;
import com.prowidesoftware.swift.model.field.Field56D;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.Map;

public class Pacs009001ParseSeqB56Field extends AbstractMt2MxPacs009001ParseField {

    private static final String NAME_A = "56A";

    private static final String NAME_C = "56C";

    private static final String NAME_D = "56D";

    private static final String SEQUENCE_B = "B";

    @Override
    public void parseField() throws SwiftException {
        AbstractMT abstractMT = context.get(AbstractMT.class);
        if (!abstractMT.getSwiftMessage().isCOV()) {
            return;
        }
        Field56A field56A = null;
        Field56C field56C = null;
        Field56D field56D = null;
        Tag tag = getTagByName(NAME_A, SEQUENCE_B);
        if (tag != null) {
            field56A = (Field56A)tag.asField();
        }
        tag = getTagByName(NAME_C, SEQUENCE_B);
        if (tag != null) {
            field56C = (Field56C)tag.asField();
        }
        tag = getTagByName(NAME_D, SEQUENCE_B);
        if (tag != null) {
            field56D = (Field56D)tag.asField();
        }
        Map<String, Object> intrmyAgt1Maps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "intrmyAgt1");
        Map<String, Object> intrmyAgt1AcctMaps = MapUtil.getAndCreateNotPut(cdtTrfTxInfMaps, "intrmyAgt1Acct");
        String account = "";
        if (field56A != null) {
            String bicCode = field56A.getBIC();
            mt_to_mxBICFI(bicCode, intrmyAgt1Maps);
            if (StringUtil.isNotEmpty(field56A.getComponent1())) {
                account += "/" + field56A.getComponent1();
            }
            if (StringUtil.isNotEmpty(field56A.getComponent2())) {
                account += "/" + field56A.getComponent2();
            }
            if (account.startsWith("//") && !account.startsWith("//CH") && !account.startsWith("//FW") && !account.startsWith("//RT")) {
                Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(intrmyAgt1Maps, "finInstnId");
                Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                if(clrSysMmbIdMaps.size()>0){
                    finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                }
                if(finInstnIdMaps.size()>0){
                    intrmyAgt1Maps.put("finInstnId", finInstnIdMaps);
                }
            } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                mt_to_mxFinancialInstitutionAccount(account, intrmyAgt1AcctMaps, "IntrmyAgt1Acct/Id");
            }
        } else if (field56C != null) {
            account = field56C.getValue();
            Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(intrmyAgt1Maps, "finInstnId");
            if (account.startsWith("//") && !account.startsWith("//CH") ) {
                if (isMTClearingSystemCodeInList(account)) {
                    Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                    mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                    if(clrSysMmbIdMaps.size()>0){
                        finInstnIdMaps.put("clrSysMmbId",clrSysMmbIdMaps);
                    }
                    //name address 设置NOTPROVIDED
                    finInstnIdMaps.put("nm", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                    MapUtil.adds(finInstnIdMaps, "pstlAdr/adrLine", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
                } else {
                    mt_to_mxClearingSystemToNameAndAddressLine(account, null, finInstnIdMaps, "IntrmyAgt1/FinInstnId");
                }
            } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                mt_to_mxFinancialInstitutionAccount(account, intrmyAgt1AcctMaps, "IntrmyAgt1Acct/Id");
                //name address 设置NOTPROVIDED
                finInstnIdMaps.put("nm", Mx2MtConstants.MX_TO_MT_DEFAULT_VALUE);
                MapUtil.adds(finInstnIdMaps, "pstlAdr/adrLine", Mx2MtConstants.MT_TO_MX_DEFAULT_VALUE);
            }
            if(finInstnIdMaps.size()>0){
                intrmyAgt1Maps.put("finInstnId",finInstnIdMaps);
            }
        } else if (field56D != null) {
            if (StringUtil.isNotEmpty(field56D.getComponent1())) {
                account += "/" + field56D.getComponent1();
            }
            if (StringUtil.isNotEmpty(field56D.getComponent2())) {
                account += "/" + field56D.getComponent2();
            }
            String nameAddress = field56D.getNameAndAddress();
            mt_to_mxFinancialInstitutionNameAndUnstructuredAddress(nameAddress, intrmyAgt1Maps);
            if (account.startsWith("//") && !account.startsWith("//CH")) {
                Map<String, Object> finInstnIdMaps = MapUtil.getAndCreateNotPut(intrmyAgt1Maps, "finInstnId");
                Map<String, Object> clrSysMmbIdMaps = MapUtil.getAndCreateNotPut(finInstnIdMaps, "clrSysMmbId");
                mt_to_mxClearingIdentifier(account, clrSysMmbIdMaps);
                if(clrSysMmbIdMaps.size()>0){
                    finInstnIdMaps.put("clrSysMmbId", clrSysMmbIdMaps);
                }
                if(finInstnIdMaps.size()>0){
                    intrmyAgt1Maps.put("finInstnId", finInstnIdMaps);
                }
            } else if (!account.startsWith("//") || account.startsWith("//CH")) {
                mt_to_mxFinancialInstitutionAccount(account, intrmyAgt1AcctMaps, "IntrmyAgt1Acct/Id");
            }
        }
        if (intrmyAgt1Maps.size() > 0) {
            undrlygCstmrCdtTrfMaps.put("intrmyAgt1", intrmyAgt1Maps);
        }
        if (intrmyAgt1AcctMaps.size() > 0) {
            undrlygCstmrCdtTrfMaps.put("intrmyAgt1Acct", intrmyAgt1AcctMaps);
        }
    }
}
