package com.brilliance.swift.mx2map;

import com.alibaba.fastjson.JSONArray;
import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.util.MessageUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.CdtDbtCode;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field53A;
import com.prowidesoftware.swift.model.field.Field53B;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.util.LinkedHashMap;
import java.util.Map;

public abstract class AbstractMx2MapCreator implements Mx2MapCreator{

    protected String xmlStr;

    protected String localCode;

    protected Map<String, Object> maps = new LinkedHashMap<>();

    /**
     * MX103报文是否是单双报逻辑
     */
    protected String doubleMessageFlag = "N";

    public String getXmlStr() {
        return xmlStr;
    }

    public void setXmlStr(String xmlStr) {
        this.xmlStr = xmlStr;
    }

    public String getLocalCode() {
        return localCode;
    }

    public void setLocalCode(String localCode) {
        this.localCode = localCode;
    }

    public String getDoubleMessageFlag() {
        return doubleMessageFlag;
    }

    public void setDoubleMessageFlag(String doubleMessageFlag) {
        this.doubleMessageFlag = doubleMessageFlag;
    }

    protected String getPropertyValue(String key) {
        return MessageUtil.getPropertyValue(localCode, key);
    }

    /**
     * 组装party信息
     * @param name 表示party的名字
     * @param elementMaps 平铺要素MAP
     */
    protected void buildPartyInfo(Map<String, Object> targetMaps, String name, Map<String, Object> elementMaps) {
        String[] keys = new String[]{name+"AcctIdIban", name+"AcctId", name+"Nm", name+"OrgAnyBIC",
                                     name+"PstlAdrCtry", name+"PstlAdrDept", name+"PstlAdrSubDept",
                                     name+"PstlAdrStrtNm", name+"PstlAdrBldgNb", name+"PstlAdrBldgNm",
                                     name+"PstlAdrFlr", name+"PstlAdrRoom", name+"PstlAdrTwnNm",
                                     name+"PstlAdrPstBx", name+"PstlAdrPstCd", name+"PstlAdrCtrySubDvsn",
                                     name+"PstlAdrDstrctNm", name+"PstlAdrTwnLctnNm", name+"AdrLines"
                                    };
        boolean existFlag = false;
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                existFlag = true;
                break;
            }
        }
        if (existFlag) {
            Map<String, Object> newMaps = new LinkedHashMap<>();
            targetMaps.put(Mx2MtConstants.NEW_LINE + getPropertyValue(name+"Info"), newMaps);
            for (String key : keys) {
                if (elementMaps.get(key) != null) {
                    Object obj = elementMaps.get(key);
                    String suffixKey = key.substring(key.indexOf(name) + name.length());
                    if (obj instanceof String) {
                        newMaps.put(getPropertyValue(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1)), obj);
                    } else if (obj instanceof JSONArray) {
                        JSONArray jsonArray = (JSONArray)obj;
                        for (int i=0; i<jsonArray.size(); i++) {
                            Object object = jsonArray.get(i);
                            newMaps.put(getPropertyValue(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1)) + "(" +(i+1) + ")", object);
                        }
                    }
                }
            }
        }else if (elementMaps.containsKey(name)) {
            Object vlu = elementMaps.get(name);
            if (vlu instanceof Map) {
                Map<String, String> agentMap = (Map<String, String>) vlu;
                Map<String, Object> newMaps = new LinkedHashMap<>();
                targetMaps.put(Mx2MtConstants.NEW_LINE + getPropertyValue(name + "Info"), newMaps);
                keys = new String[]{"acctIdIban", "acctId", "nm", "orgAnyBIC"};
                for (String key : keys) {
                    if (agentMap.get(key) != null) {
                        maps.put(getPropertyValue(key), agentMap.get(key));
                    }
                }
            }
        }
    }

    /**
     * 组装Agent信息
     * @param name 表示Agent的名字
     * @param elementMaps 平铺要素MAP
     */
    protected void buildAgentInfo(Map<String, Object> targetMaps, String name, Map<String, Object> elementMaps) {
        String[] keys = new String[]{name+"AcctIdIban", name+"AcctId", name+"Bicfi",
                name+"Nm", name+"ClrSysIdType", name+"ClrSysId",
                name+"PstlAdrCtry", name+"PstlAdrDept", name+"PstlAdrSubDept",
                name+"PstlAdrStrtNm", name+"PstlAdrBldgNb", name+"PstlAdrBldgNm",
                name+"PstlAdrFlr", name+"PstlAdrRoom", name+"PstlAdrTwnNm",
                name+"PstlAdrPstBx", name+"PstlAdrPstCd", name+"PstlAdrCtrySubDvsn",
                name+"PstlAdrDstrctNm", name+"PstlAdrTwnLctnNm", name+"AdrLines"
        };
        boolean existFlag = false;
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                existFlag = true;
                break;
            }
        }
        if (existFlag) {
            if ("instgRmbrsmntAgt".equals(name) || "instdRmbrsmntAgt".equals(name)) {
                doubleMessageFlag = "Y";
            }
            Map<String, Object> newMaps = new LinkedHashMap<>();
            targetMaps.put(Mx2MtConstants.NEW_LINE + getPropertyValue(name+"Info"), newMaps);
            for (String key : keys) {
                if (elementMaps.get(key) != null) {
                    Object obj = elementMaps.get(key);
                    String suffixKey = key.substring(key.indexOf(name) + name.length());
                    if (obj instanceof String) {
                        newMaps.put(getPropertyValue(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1)), obj);
                    } else if (obj instanceof JSONArray) {
                        JSONArray jsonArray = (JSONArray)obj;
                        for (int i=0; i<jsonArray.size(); i++) {
                            Object object = jsonArray.get(i);
                            newMaps.put(getPropertyValue(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1)) + "(" +(i+1) + ")", object);
                        }
                    }
                }
            }
        } else if (elementMaps.containsKey(name)) {
            Object vlu = elementMaps.get(name);
            if (vlu instanceof Map) {
                Map<String, String> agentMap = (Map<String, String>) vlu;
                Map<String, Object> newMaps = new LinkedHashMap<>();
                targetMaps.put(Mx2MtConstants.NEW_LINE + getPropertyValue(name + "Info"), newMaps);
                keys = new String[]{"bicfi", "nm"};
                for (String key : keys) {
                    if (agentMap.get(key) != null) {
                        maps.put(getPropertyValue(key), agentMap.get(key));
                    }
                }
            }
        }
    }

    protected void buildAgentInfo(Map<String, Object> targetMaps, String name, Map<String, Object> elementMaps, String subMapName) {
        String[] keys = new String[]{name+"AcctIdIban", name+"AcctId", name+"Bicfi",
                name+"Nm", name+"ClrSysIdType", name+"ClrSysId",
                name+"PstlAdrCtry", name+"PstlAdrDept", name+"PstlAdrSubDept",
                name+"PstlAdrStrtNm", name+"PstlAdrBldgNb", name+"PstlAdrBldgNm",
                name+"PstlAdrFlr", name+"PstlAdrRoom", name+"PstlAdrTwnNm",
                name+"PstlAdrPstBx", name+"PstlAdrPstCd", name+"PstlAdrCtrySubDvsn",
                name+"PstlAdrDstrctNm", name+"PstlAdrTwnLctnNm", name+"AdrLines"
        };
        boolean existFlag = false;
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                existFlag = true;
                break;
            }
        }
        if (existFlag) {
            if ("instgRmbrsmntAgt".equals(name) || "instdRmbrsmntAgt".equals(name)) {
                doubleMessageFlag = "Y";
            }
            Map<String, Object> newMaps = new LinkedHashMap<>();
            targetMaps.put(Mx2MtConstants.NEW_LINE + getPropertyValue(subMapName), newMaps);
            for (String key : keys) {
                if (elementMaps.get(key) != null) {
                    Object obj = elementMaps.get(key);
                    String suffixKey = key.substring(key.indexOf(name) + name.length());
                    if (obj instanceof String) {
                        newMaps.put(getPropertyValue(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1)), obj);
                    } else if (obj instanceof JSONArray) {
                        JSONArray jsonArray = (JSONArray)obj;
                        for (int i=0; i<jsonArray.size(); i++) {
                            Object object = jsonArray.get(i);
                            newMaps.put(getPropertyValue(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1)) + "(" +(i+1) + ")", object);
                        }
                    }
                }
            }
        }
    }

    protected String addSuffixWithMessageType(Map<String, Object> elementMaps) {
        String messageType = (String) elementMaps.get("messageType");
        if (StringUtil.isNotEmpty(messageType)) {
            if (messageType.startsWith("pacs008001")) {
                return messageType + "(MT103)";
            } else if (messageType.startsWith("pacs009001")) {
                if (elementMaps.get("undrlygCstmrCdtTrf") != null) {
                    return messageType + "(MT202COV)";
                } else {
                    return messageType + "(MT202)";
                }
            } else if (messageType.startsWith("camt054001")) {
                if (CdtDbtCode.CRDT.value().equals(elementMaps.get("cdtDbtInd"))) {
                    return messageType + "(MT910)";
                } else {
                    return messageType + "(MT900)";
                }
            } else if (messageType.startsWith("camt053001")) {
                return messageType + "(MT950)";
            } else if (messageType.startsWith("camt029001")) {
                String orgnlMsgNmId = (String) elementMaps.get("orgnlMsgNmId");
                if (orgnlMsgNmId != null) {
                    if (orgnlMsgNmId.contains("pacs.008") || orgnlMsgNmId.contains("pacs.003") || orgnlMsgNmId.matches("MT10[0-9]{1}")) {
                        return messageType + "(MT196)";
                    } else if (orgnlMsgNmId.contains("pacs.009") || orgnlMsgNmId.contains("pacs.010") || orgnlMsgNmId.matches("MT20[0-9]{1}")) {
                        return messageType + "(MT296)";
                    }
                }
            } else if (messageType.startsWith("camt056001")) {
                String orgnlMsgNmId = (String) elementMaps.get("orgnlMsgNmId");
                if (orgnlMsgNmId != null) {
                    if (orgnlMsgNmId.contains("pacs.008") || orgnlMsgNmId.contains("pacs.003") || orgnlMsgNmId.matches("MT10[0-9]{1}")) {
                        return messageType + "(MT192)";
                    } else if (orgnlMsgNmId.contains("pacs.009") || orgnlMsgNmId.contains("pacs.010") || orgnlMsgNmId.matches("MT20[0-9]{1}")) {
                        return messageType + "(MT292)";
                    }
                }
            } else if (messageType.startsWith("camt057001")) {
                return messageType + "(MT210)";
            } else if (messageType.startsWith("pacs002001")) {
                String orgnlMsgNmId = (String) elementMaps.get("orgnlMsgNmId");
                if (orgnlMsgNmId != null) {
                    if (orgnlMsgNmId.contains("pacs.008") || orgnlMsgNmId.contains("pacs.003") || orgnlMsgNmId.matches("MT10[0-9]{1}")) {
                        return messageType + "(MT199)";
                    } else if (orgnlMsgNmId.contains("pacs.009") || orgnlMsgNmId.contains("pacs.010") || orgnlMsgNmId.matches("MT20[0-9]{1}")) {
                        return messageType + "(MT299)";
                    }
                }
            } else if (messageType.startsWith("pacs004001")) {
                if (elementMaps.get("dbtrAgt") != null && elementMaps.get("cdtrAgt") != null) {
                    return messageType + "(MT202RETN)";
                } else {
                    return messageType + "(MT103RETN)";
                }
            } else {
                return messageType;
            }
        }
        return null;
    }

    /**
     * 53域 特殊处理
     * @param targetMaps
     * @param abstractMT
     */
    protected void buildInstgRmbrsmntAgtInfo(Map<String, Object> targetMaps, AbstractMT abstractMT) {
        Tag tag = abstractMT.getSwiftMessage().getBlock4().getTagByName("53A");
        if (tag != null) {
            doubleMessageFlag = "Y";
            Map<String, Object> newMaps = new LinkedHashMap<>();
            targetMaps.put(Mx2MtConstants.NEW_LINE + getPropertyValue("instgRmbrsmntAgtInfo"), newMaps);
            Field53A field53A = (Field53A)tag.asField();
            String bicCode = field53A.getBIC();
            if (StringUtil.isNotEmpty(bicCode)) {
                newMaps.put(getPropertyValue("bicfi"), bicCode);
            }
            String account = field53A.getComponent2();
            if (StringUtil.isNotEmpty(account)) {
                newMaps.put(getPropertyValue("acctId"), account);
            }
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName("53B");
        if (tag != null) {
            doubleMessageFlag = "Y";
            Map<String, Object> newMaps = new LinkedHashMap<>();
            targetMaps.put(Mx2MtConstants.NEW_LINE + getPropertyValue("instgRmbrsmntAgtInfo"), newMaps);
            Field53B field53B = (Field53B)tag.asField();
            String account = field53B.getComponent2();
            if (StringUtil.isNotEmpty(account)) {
                newMaps.put(getPropertyValue("acctId"), account);
            }
        }
    }
}
