package com.brilliance.swift.mx2message;

import com.brilliance.swift.util.MessageUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.CdtDbtCode;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field53A;
import com.prowidesoftware.swift.model.field.Field53B;
import com.prowidesoftware.swift.model.mt.AbstractMT;
import freemarker.template.Template;
import freemarker.template.TemplateException;

import java.io.IOException;
import java.io.StringWriter;
import java.util.LinkedHashMap;
import java.util.Map;

public abstract class AbstractMx2MessageCreator implements Mx2MessageCreator {
    protected String xmlStr;

    protected String localCode;

    protected Template template;

    protected Map<String, Object> maps = new LinkedHashMap<String, Object>();

    public String getXmlStr() {
        return xmlStr;
    }

    public void setXmlStr(String xmlStr) {
        this.xmlStr = xmlStr;
    }

    public String getLocalCode() {
        return localCode;
    }

    public void setLocalCode(String localCode) {
        this.localCode = localCode;
    }

    public Template getTemplate() {
        return template;
    }

    public void setTemplate(Template template) {
        this.template = template;
    }

    protected String getPropertyValue(String key) {
        return MessageUtil.getPropertyValue(localCode, key);
    }

    protected String process() {
        try{
            StringWriter stringWriter = new StringWriter();
            maps.put("I18NHelper", new I18NHelper(localCode));
            template.process(maps, stringWriter);
            return stringWriter.toString();
        } catch (TemplateException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    protected String addSuffixWithMessageType(Map<String, Object> elementMaps) {
        String messageType = (String) elementMaps.get("messageType");
        if (StringUtil.isNotEmpty(messageType)) {
            if (messageType.startsWith("pacs008001")) {
                return messageType + "(MT103)";
            } else if (messageType.startsWith("pacs009001")) {
                if (elementMaps.get("undrlygCstmrCdtTrf") != null) {
                    return messageType + "(MT202COV)";
                } else {
                    return messageType + "(MT202)";
                }
            } else if (messageType.startsWith("camt054001")) {
                if (CdtDbtCode.CRDT.value().equals(elementMaps.get("cdtDbtInd"))) {
                    return messageType + "(MT910)";
                } else {
                    return messageType + "(MT900)";
                }
            } else if (messageType.startsWith("camt053001")) {
                return messageType + "(MT950)";
            } else {
                return messageType;
            }
        }
        return null;
    }
    /**
     * 组装party信息
     * @param name 表示party的名字
     * @param elementMaps 平铺要素MAP
     */
    protected void buildPartyInfo(String name, Map<String, Object> elementMaps) {
        String[] keys = new String[]{name+"AcctIdIban", name+"AcctId", name+"Nm", name+"OrgAnyBIC",
                name+"PstlAdrCtry", name+"PstlAdrDept", name+"PstlAdrSubDept",
                name+"PstlAdrStrtNm", name+"PstlAdrBldgNb", name+"PstlAdrBldgNm",
                name+"PstlAdrFlr", name+"PstlAdrRoom", name+"PstlAdrTwnNm",
                name+"PstlAdrPstBx", name+"PstlAdrPstCd", name+"PstlAdrCtrySubDvsn",
                name+"PstlAdrDstrctNm", name+"PstlAdrTwnLctnNm", name+"AdrLines"
        };
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                maps.put(name,"true");
                break;
            }
        }
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                maps.put(key,elementMaps.get(key));
            }
        }
        if(elementMaps.containsKey(name)){
            Object vlu = elementMaps.get(name);
            if (vlu instanceof Map) {
                Map<String, String> agentMap = (Map<String, String>) vlu;
                maps.put(name,"true");
                keys = new String[]{"acctIdIban", "acctId", "nm", "orgAnyBIC"};
                for (String key : keys) {
                    if (agentMap.get(key) != null) {
                        maps.put(name+key.substring(0, 1).toUpperCase() + key.substring(1), agentMap.get(key));
                    }
                }
            }
        }
    }
    /**
     * 组装Agent信息
     * @param name 表示Agent的名字
     * @param elementMaps 平铺要素MAP
     */
    protected void buildAgentInfo(String name, Map<String, Object> elementMaps) {
        String[] keys = new String[]{name+"AcctIdIban", name+"AcctId", name+"Bicfi",
                name+"Nm", name+"ClrSysIdType", name+"ClrSysId",
                name+"PstlAdrCtry", name+"PstlAdrDept", name+"PstlAdrSubDept",
                name+"PstlAdrStrtNm", name+"PstlAdrBldgNb", name+"PstlAdrBldgNm",
                name+"PstlAdrFlr", name+"PstlAdrRoom", name+"PstlAdrTwnNm",
                name+"PstlAdrPstBx", name+"PstlAdrPstCd", name+"PstlAdrCtrySubDvsn",
                name+"PstlAdrDstrctNm", name+"PstlAdrTwnLctnNm", name+"AdrLines"
        };
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                maps.put(name,"true");
                break;
            }
        }
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                maps.put(key,elementMaps.get(key));
            }
        }
        if (elementMaps.containsKey(name)) {
            Object vlu = elementMaps.get(name);
            if (vlu instanceof Map) {
                Map<String, String> agentMap = (Map<String, String>) vlu;
                maps.put(name,"true");
                keys = new String[]{"bicfi", "nm"};
                for (String key : keys) {
                    if (agentMap.get(key) != null) {
                        maps.put(name+key.substring(0, 1).toUpperCase() + key.substring(1), agentMap.get(key));
                    }
                }
            }
        }
    }
    /**
     * 53域 特殊处理
     * @param name
     * @param abstractMT
     */
    protected void buildInstgRmbrsmntAgtInfo(String name,AbstractMT abstractMT){
        Tag tag = abstractMT.getSwiftMessage().getBlock4().getTagByName("53A");
        if (tag != null) {
            maps.put("instgRmbrsmntAgtInfo","true");
            Field53A field53A = (Field53A)tag.asField();
            String bicCode = field53A.getBIC();
            if (StringUtil.isNotEmpty(bicCode)) {
                maps.put(name+"Bicfi", bicCode);
            }
            String account = field53A.getComponent2();
            if (StringUtil.isNotEmpty(account)) {
                maps.put(name+"AcctId", account);
            }
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName("53B");
        if (tag != null) {
            maps.put("instgRmbrsmntAgtInfo","true");
            Field53B field53B = (Field53B)tag.asField();
            String account = field53B.getComponent2();
            if (StringUtil.isNotEmpty(account)) {
                maps.put(name+"AcctId", account);
            }
        }
    }

    protected void getAppHdrMessage(Map<String,Object> elementMaps){
        if (elementMaps.get("frBic") != null) {
            maps.put("frBic",elementMaps.get("frBic"));
        }
        if (elementMaps.get("toBic") != null) {
            maps.put("toBic",elementMaps.get("toBic"));
        }
        String messageType = addSuffixWithMessageType(elementMaps);
        if (messageType != null) {
            maps.put("messageType", messageType);
        }
        if (elementMaps.get("bizSvc") != null) {
            maps.put("bizSvc", elementMaps.get("bizSvc"));
        }
        if (elementMaps.get("creDt") != null) {
            maps.put("creDt", elementMaps.get("creDt"));
        }
        if (elementMaps.get("bizMsgIdr") != null) {
            maps.put("bizMsgIdr", elementMaps.get("bizMsgIdr"));
        }
    }
}
