package com.brilliance.swift.mx2message;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2message.camt029.Mx2MessageCamt029Creator;
import com.brilliance.swift.mx2message.camt052.Mx2MessageCamt052Creator;
import com.brilliance.swift.mx2message.camt053.Mx2MessageCamt053Creator;
import com.brilliance.swift.mx2message.camt054.Mx2MessageCamt054Creator;
import com.brilliance.swift.mx2message.camt056.Mx2MessageCamt056Creator;
import com.brilliance.swift.mx2message.camt057.Mx2MessageCamt057Creator;
import com.brilliance.swift.mx2message.camt060.Mx2MessageCamt060Creator;
import com.brilliance.swift.mx2message.pacs002.Mx2MessagePacs002Creator;
import com.brilliance.swift.mx2message.pacs004.Mx2MessagePacs004Creator;
import com.brilliance.swift.mx2message.pacs008.Mx2MessagePacs008Creator;
import com.brilliance.swift.mx2message.pacs009.Mx2MessagePacs009Creator;
import com.brilliance.swift.util.XmlUtil;
import com.prowidesoftware.swift.model.MxId;
import freemarker.template.Configuration;
import freemarker.template.Template;

import java.io.IOException;

public class Mx2MessageCreatorManager {
    private final static Configuration configuration = new Configuration(Configuration.VERSION_2_3_28);

    static {
        configuration.setClassLoaderForTemplateLoading(Thread.currentThread().getContextClassLoader(), "template");
    }

    public String mx2Message(String xml, String localCode) throws SwiftException {
        xml = XmlUtil.handlingXml(xml);
        MxId mxId = XmlUtil.getMxMessageType(xml);
        if (mxId == null) {
            throw new SwiftException("找不到MX报文类型");
        }
        String messageType = (mxId.getBusinessProcess().name()
                + mxId.getFunctionality()
                + mxId.getVariant())
                //+ abstractMX.getMxId().getVersion())
                .trim();
        AbstractMx2MessageCreator creator = getCreator(messageType);
        creator.setXmlStr(xml);
        creator.setLocalCode(localCode);
        Template template = getTemplate(messageType);
        creator.setTemplate(template);
        return creator.mx2Message();
    }

    public AbstractMx2MessageCreator getCreator(String messageType) {
        if ("camt029001".equals(messageType)) {
            return new Mx2MessageCamt029Creator();
        } else if("camt054001".equals(messageType)){
            return new Mx2MessageCamt054Creator();
        } else if("camt056001".equals(messageType)){
            return new Mx2MessageCamt056Creator();
        }else if("camt057001".equals(messageType)){
            return new Mx2MessageCamt057Creator();
        }else if("camt060001".equals(messageType)){
            return new Mx2MessageCamt060Creator();
        } else if("pacs008001".equals(messageType)){
            return new Mx2MessagePacs008Creator();
        }else if("camt052001".equals(messageType)){
            return new Mx2MessageCamt052Creator();
        }else if("camt053001".equals(messageType)){
            return new Mx2MessageCamt053Creator();
        } else if ("pacs002001".equals(messageType)) {
            return new Mx2MessagePacs002Creator();
        } else if ("pacs004001".equals(messageType)) {
            return new Mx2MessagePacs004Creator();
        } else if ("pacs009001".equals(messageType)) {
            return new Mx2MessagePacs009Creator();
        } else {
            throw new SwiftException("无效的报文类型");
        }
    }

    public Template getTemplate(String messageType) {
        try {
            return configuration.getTemplate(messageType + ".ftl");
        } catch (IOException e) {
            throw new SwiftException("template error", e);
        }
    }
}
