package com.brilliance.swift.mx2message.pacs008;

import com.brilliance.swift.SwiftTransfer;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2message.AbstractMx2MessageCreator;
import com.brilliance.swift.vo.common.SettlementMethodCode;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.Map;

public class Mx2MessagePacs008Creator extends AbstractMx2MessageCreator {
    @Override
    public String mx2Message() throws SwiftException {
        Map<String, Object> elementMaps = SwiftTransfer.mx2ElementMaps(xmlStr);
        getAppHdrMessage(elementMaps);
        if (elementMaps.get("endToEndId") != null) {
            maps.put("endToEndId", elementMaps.get("endToEndId"));
        }
        if (elementMaps.get("uetr") != null) {
            maps.put("uetr", elementMaps.get("uetr"));
        }
        if (elementMaps.get("sttlmMtd") != null) {
            maps.put("sttlmMtd", elementMaps.get("sttlmMtd"));
        }
        if (elementMaps.get("sttlmAcctIdIban") != null) {
            maps.put("sttlmAcctIdIban", elementMaps.get("sttlmAcctIdIban"));
        }
        if (elementMaps.get("sttlmAcctId") != null) {
            maps.put("sttlmAcctId", elementMaps.get("sttlmAcctId"));
        }
        if(elementMaps.get("intrBkSttlmDt") != null||elementMaps.get("intrBkSttlmAmt") != null
        ||elementMaps.get("instdAmt") != null||elementMaps.get("xchgRate") != null){
            maps.put("creditTransferInfo","true");
        }
        if (elementMaps.get("intrBkSttlmDt") != null) {
            maps.put("intrBkSttlmDt", elementMaps.get("intrBkSttlmDt"));
        }
        if (elementMaps.get("intrBkSttlmAmt") != null) {
            BigDecimal amt = (BigDecimal) elementMaps.get("intrBkSttlmAmt");
            if (elementMaps.get("intrBkSttlmCcy") != null) {
                maps.put("intrBkSttlmAmt", elementMaps.get("intrBkSttlmCcy") + " " + amt.toPlainString());
            } else {
                maps.put("intrBkSttlmAmt", amt.toPlainString());
            }
        }
        if (elementMaps.get("instdAmt") != null) {
            BigDecimal amt = (BigDecimal) elementMaps.get("instdAmt");
            if (elementMaps.get("instdAmtCcy") != null) {
                maps.put("instdAmt", elementMaps.get("instdAmtCcy") + " " + amt.toPlainString());
            } else {
                maps.put("instdAmt", amt.toPlainString());
            }
        }
        if (elementMaps.get("xchgRate") != null) {
            BigDecimal xchgRate = (BigDecimal) elementMaps.get("xchgRate");
            maps.put("xchgRate", xchgRate.toPlainString());
        }
        buildPartyInfo("dbtr", elementMaps);
        buildAgentInfo("dbtrAgt", elementMaps);
        AbstractMT abstractMT = null;
        try {
            String mtMessage = SwiftTransfer.mx2Mt(xmlStr, null, null);
            abstractMT = AbstractMT.parse(mtMessage);
        } catch (IOException e) {
            throw new SwiftException(e.getMessage());
        }
        if (SettlementMethodCode.INGA.value().equals(elementMaps.get("sttlmMtd"))
                || SettlementMethodCode.INDA.value().equals(elementMaps.get("sttlmMtd"))) {
            buildInstgRmbrsmntAgtInfo("instgRmbrsmntAgt",abstractMT);
        } else {
            buildAgentInfo("instgRmbrsmntAgt", elementMaps);
        }
        buildAgentInfo( "instdRmbrsmntAgt", elementMaps);
        buildAgentInfo( "thrdRmbrsmntAgt", elementMaps);
        buildAgentInfo( "intrmyAgt1", elementMaps);
        buildAgentInfo( "cdtrAgt", elementMaps);
        buildPartyInfo( "cdtr", elementMaps);
        if (elementMaps.get("chrgBr") != null) {
            maps.put("chrgBr", elementMaps.get("chrgBr"));
        }
        if (elementMaps.get("chrgsInf") != null) {
            maps.put("chrgsInf",elementMaps.get("chrgsInf"));
        }
        Tag tag = abstractMT.getSwiftMessage().getBlock4().getTagByName("70");
        if (tag != null) {
            maps.put("remittanceInformation", tag.getValue());
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName("72");
        if (tag != null) {
            maps.put("txnRemark", tag.getValue());
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName("77B");
        if (tag != null) {
            maps.put("regulatoryReporting", tag.getValue());
        }
        return process();
    }
}
