package com.brilliance.swift.mx2message.pacs009;

import com.brilliance.swift.SwiftTransfer;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2message.AbstractMx2MessageCreator;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.vo.common.SettlementMethodCode;
import com.prowidesoftware.swift.model.SwiftTagListBlock;
import com.prowidesoftware.swift.model.Tag;
import com.prowidesoftware.swift.model.field.Field53A;
import com.prowidesoftware.swift.model.field.Field53B;
import com.prowidesoftware.swift.model.mt.AbstractMT;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class Mx2MessagePacs009Creator extends AbstractMx2MessageCreator {
    @Override
    public String mx2Message() throws SwiftException {
        Map<String, Object> elementMaps = SwiftTransfer.mx2ElementMaps(xmlStr);
        getAppHdrMessage(elementMaps);
        if (elementMaps.get("endToEndId") != null) {
            maps.put("endToEndId", elementMaps.get("endToEndId"));
        }
        if (elementMaps.get("uetr") != null) {
            maps.put("uetr", elementMaps.get("uetr"));
        }
        if (elementMaps.get("sttlmMtd") != null) {
            maps.put("sttlmMtd", elementMaps.get("sttlmMtd"));
        }
        if (elementMaps.get("sttlmAcctIdIban") != null) {
            maps.put("sttlmAcctIdIban", elementMaps.get("sttlmAcctIdIban"));
        }
        if (elementMaps.get("sttlmAcctId") != null) {
            maps.put("sttlmAcctId", elementMaps.get("sttlmAcctId"));
        }
        Map<String, Object> creditTransferMaps = new LinkedHashMap<String, Object>();
        maps.put("creditTransferInfo", creditTransferMaps);
        if (elementMaps.get("bizMsgIdr") != null) {
            creditTransferMaps.put("bizMsgIdr", elementMaps.get("bizMsgIdr"));
        }
        if (elementMaps.get("intrBkSttlmDt") != null) {
            creditTransferMaps.put("intrBkSttlmDt", elementMaps.get("intrBkSttlmDt"));
        }
        if (elementMaps.get("intrBkSttlmAmt") != null) {
            BigDecimal amt = (BigDecimal) elementMaps.get("intrBkSttlmAmt");
            if (elementMaps.get("intrBkSttlmCcy") != null) {
                creditTransferMaps.put("intrBkSttlmCcy", elementMaps.get("intrBkSttlmCcy") + " " + amt.toPlainString());
            } else {
                creditTransferMaps.put("intrBkSttlmCcy", amt.toPlainString());
            }
        }
        buildAgentInfo(maps,"dbtr", elementMaps, "dbtrAgtInfo");
        AbstractMT abstractMT = null;
        try {
            String mtMessage = SwiftTransfer.mx2Mt(xmlStr, null, null);
            if (StringUtil.isNotEmpty(mtMessage)) {
                abstractMT = AbstractMT.parse(mtMessage);
            }
        } catch (IOException e) {
            throw new SwiftException(e.getMessage());
        }
        if (SettlementMethodCode.INGA.value().equals(elementMaps.get("sttlmMtd"))
            || SettlementMethodCode.INDA.value().equals(elementMaps.get("sttlmMtd"))) {
            buildInstgRmbrsmntAgtInfo(maps, abstractMT);
        } else {
            buildAgentInfo(maps, "instgRmbrsmntAgt", elementMaps);
        }
        buildAgentInfo(maps,"instdRmbrsmntAgt", elementMaps);
        buildAgentInfo(maps,"intrmyAgt1", elementMaps);
        buildAgentInfo(maps,"cdtrAgt", elementMaps, "cdtrAgtAcctInfo");
        buildAgentInfo(maps,"cdtr", elementMaps, "cdtrAgtInfo");
        Tag tag = getTagByName(abstractMT, "72", "A");
        if (tag != null) {
            maps.put("txnRemark", tag.getValue());
        }
        //子交易信息
        if (elementMaps.get("undrlygCstmrCdtTrf") != null) {
            Map<String, Object> undrlygCstmrCdtTrfMaps = (Map<String, Object>)elementMaps.get("undrlygCstmrCdtTrf");
            Map<String, Object> undrlyTxnMaps = new LinkedHashMap<String, Object>();
            maps.put("undrlygCstmrCdtTrfInfo", undrlyTxnMaps);
            buildPartyInfo(undrlyTxnMaps, "dbtr", undrlygCstmrCdtTrfMaps);
            buildAgentInfo(undrlyTxnMaps, "dbtrAgt", undrlygCstmrCdtTrfMaps);
            buildAgentInfo(undrlyTxnMaps, "intrmyAgt1", undrlygCstmrCdtTrfMaps);
            buildAgentInfo(undrlyTxnMaps, "cdtrAgt", undrlygCstmrCdtTrfMaps);
            buildPartyInfo(undrlyTxnMaps, "cdtr", undrlygCstmrCdtTrfMaps);
            tag = getTagByName(abstractMT, "70", "B");
            if (tag != null) {
                undrlyTxnMaps.put("remittanceInformation", tag.getValue());
            }
            tag = getTagByName(abstractMT, "72", "B");
            if (tag != null) {
                undrlyTxnMaps.put("txnRemark", tag.getValue());
            }
            if (undrlygCstmrCdtTrfMaps.get("instdAmt") != null) {
                BigDecimal amt = (BigDecimal) elementMaps.get("instdAmt");
                if (elementMaps.get("instdAmtCcy") != null) {
                    undrlyTxnMaps.put("instdAmtCcy", elementMaps.get("instdAmtCcy") + " " + amt.toPlainString());
                } else {
                    undrlyTxnMaps.put("instdAmtCcy", amt.toPlainString());
                }
            }
        }
        return process();
    }

    protected Tag getTagByName(AbstractMT abstractMT, String tagName, String sequence) {
        if (abstractMT == null || abstractMT.getSwiftMessage() == null) return null;
        boolean isCov = abstractMT.getSwiftMessage().isCOV();
        if (isCov) {
            SwiftTagListBlock swiftTagListBlock = abstractMT.getSequence(sequence);
            if (swiftTagListBlock != null) {
                return swiftTagListBlock.getTagByName(tagName);
            }
        } else {
            return abstractMT.getSwiftMessage().getBlock4().getTagByName(tagName);
        }
        return null;
    }

    /**
     * 组装party信息
     * @param targetMaps targetMaps
     * @param name 表示party的名字
     * @param elementMaps 平铺要素MAP
     */
    private void buildPartyInfo(Map<String, Object> targetMaps, String name, Map<String, Object> elementMaps) {
        String[] keys = new String[]{name+"AcctIdIban", name+"AcctId", name+"Nm", name+"OrgAnyBIC",
            name+"PstlAdrCtry", name+"PstlAdrDept", name+"PstlAdrSubDept",
            name+"PstlAdrStrtNm", name+"PstlAdrBldgNb", name+"PstlAdrBldgNm",
            name+"PstlAdrFlr", name+"PstlAdrRoom", name+"PstlAdrTwnNm",
            name+"PstlAdrPstBx", name+"PstlAdrPstCd", name+"PstlAdrCtrySubDvsn",
            name+"PstlAdrDstrctNm", name+"PstlAdrTwnLctnNm", name+"AdrLines"
        };
        boolean existFlag = false;
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                existFlag = true;
                break;
            }
        }
        if (existFlag) {
            Map<String, Object> newMaps = new LinkedHashMap<String, Object>();
            targetMaps.put(name+"Info", newMaps);
            for (String key : keys) {
                if (elementMaps.get(key) != null) {
                    Object obj = elementMaps.get(key);
                    String suffixKey = key.substring(key.indexOf(name) + name.length());
                    if (obj instanceof String) {
                        newMaps.put(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1), obj);
                    } else if (obj instanceof List) {
                        List list = (List)obj;
                        newMaps.put(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1), list);
                    }
                }
            }
        }else if (elementMaps.containsKey(name)) {
            Object vlu = elementMaps.get(name);
            if (vlu instanceof Map) {
                Map<String, String> agentMap = (Map<String, String>) vlu;
                Map<String, Object> newMaps = new LinkedHashMap<String, Object>();
                targetMaps.put(name + "Info", newMaps);
                keys = new String[]{"acctIdIban", "acctId", "nm", "orgAnyBIC"};
                for (String key : keys) {
                    if (agentMap.get(key) != null) {
                        maps.put(key, agentMap.get(key));
                    }
                }
            }
        }
    }

    /**
     * 组装Agent信息
     *
     * @param targetMaps targetMaps
     * @param name 表示Agent的名字
     * @param elementMaps 平铺要素MAP
     */
    private void buildAgentInfo(Map<String, Object> targetMaps, String name, Map<String, Object> elementMaps) {
        String[] keys = new String[]{name+"AcctIdIban", name+"AcctId", name+"Bicfi",
            name+"Nm", name+"ClrSysIdType", name+"ClrSysId",
            name+"PstlAdrCtry", name+"PstlAdrDept", name+"PstlAdrSubDept",
            name+"PstlAdrStrtNm", name+"PstlAdrBldgNb", name+"PstlAdrBldgNm",
            name+"PstlAdrFlr", name+"PstlAdrRoom", name+"PstlAdrTwnNm",
            name+"PstlAdrPstBx", name+"PstlAdrPstCd", name+"PstlAdrCtrySubDvsn",
            name+"PstlAdrDstrctNm", name+"PstlAdrTwnLctnNm", name+"AdrLines"
        };
        boolean existFlag = false;
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                existFlag = true;
                break;
            }
        }
        if (existFlag) {
            Map<String, Object> newMaps = new LinkedHashMap<String, Object>();
            targetMaps.put(name + "Info", newMaps);
            for (String key : keys) {
                if (elementMaps.get(key) != null) {
                    Object obj = elementMaps.get(key);
                    String suffixKey = key.substring(key.indexOf(name) + name.length());
                    if (obj instanceof String) {
                        newMaps.put(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1), obj);
                    } else if (obj instanceof List) {
                        List jsonArray = (List)obj;
                        newMaps.put(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1), jsonArray);
                    }
                }
            }
        } else if (elementMaps.containsKey(name)) {
            Object vlu = elementMaps.get(name);
            if (vlu instanceof Map) {
                Map<String, String> agentMap = (Map<String, String>) vlu;
                Map<String, Object> newMaps = new LinkedHashMap<String, Object>();
                targetMaps.put(name + "Info", newMaps);
                keys = new String[]{"bicfi", "nm"};
                for (String key : keys) {
                    if (agentMap.get(key) != null) {
                        maps.put(key, agentMap.get(key));
                    }
                }
            }
        }
    }

    private void buildAgentInfo(Map<String, Object> targetMaps, String name, Map<String, Object> elementMaps,
                          String subMapName) {
        String[] keys = new String[]{name+"AcctIdIban", name+"AcctId", name+"Bicfi",
            name+"Nm", name+"ClrSysIdType", name+"ClrSysId",
            name+"PstlAdrCtry", name+"PstlAdrDept", name+"PstlAdrSubDept",
            name+"PstlAdrStrtNm", name+"PstlAdrBldgNb", name+"PstlAdrBldgNm",
            name+"PstlAdrFlr", name+"PstlAdrRoom", name+"PstlAdrTwnNm",
            name+"PstlAdrPstBx", name+"PstlAdrPstCd", name+"PstlAdrCtrySubDvsn",
            name+"PstlAdrDstrctNm", name+"PstlAdrTwnLctnNm", name+"AdrLines"
        };
        boolean existFlag = false;
        for (String key : keys) {
            if (elementMaps.get(key) != null) {
                existFlag = true;
                break;
            }
        }
        if (existFlag) {
            Map<String, Object> newMaps = new LinkedHashMap<String, Object>();
            targetMaps.put(subMapName, newMaps);
            for (String key : keys) {
                if (elementMaps.get(key) != null) {
                    Object obj = elementMaps.get(key);
                    String suffixKey = key.substring(key.indexOf(name) + name.length());
                    if (obj instanceof String) {
                        newMaps.put(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1), obj);
                    } else if (obj instanceof List) {
                        List jsonArray = (List)obj;
                        newMaps.put(suffixKey.substring(0,1).toLowerCase() + suffixKey.substring(1), jsonArray);
                    }
                }
            }
        }
    }

    /**
     * 53域 特殊处理
     * @param targetMaps
     * @param abstractMT
     */
    private void buildInstgRmbrsmntAgtInfo(Map<String, Object> targetMaps, AbstractMT abstractMT) {
        Tag tag = abstractMT.getSwiftMessage().getBlock4().getTagByName("53A");
        if (tag != null) {
            Map<String, Object> newMaps = new LinkedHashMap<String, Object>();
            targetMaps.put("instgRmbrsmntAgtInfo", newMaps);
            Field53A field53A = (Field53A)tag.asField();
            String bicCode = field53A.getBIC();
            if (StringUtil.isNotEmpty(bicCode)) {
                newMaps.put("bicfi", bicCode);
            }
            String account = field53A.getComponent2();
            if (StringUtil.isNotEmpty(account)) {
                newMaps.put("acctId", account);
            }
        }
        tag = abstractMT.getSwiftMessage().getBlock4().getTagByName("53B");
        if (tag != null) {
            Map<String, Object> newMaps = new LinkedHashMap<String, Object>();
            targetMaps.put("instgRmbrsmntAgtInfo", newMaps);
            Field53B field53B = (Field53B)tag.asField();
            String account = field53B.getComponent2();
            if (StringUtil.isNotEmpty(account)) {
                newMaps.put("acctId", account);
            }
        }
    }
}
