package com.brilliance.swift.util;

import com.alibaba.fastjson.JSONArray;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MapUtil {
    /**
     * 给map在指定路径的key下以先获取再赋值的方式添加value值
     * 返回操作完成后的map
     * 由调用者确保key路径每个节点都是map类型
     *
     * @param map  待操作的map
     * @param path 以”/“为分隔符的key路径
     * @param obj  待添加的value值
     * @return Map<String, Object>
     */
    @SuppressWarnings("unchecked")
    public static Map<String, Object> puts(Map<String, Object> map, String path, Object obj) {
        if (obj != null && obj != "") {
            String[] split = path.split("/");
            Map<String, Object> tmp = map;
            for (int i = 0; i < split.length - 1; i++) {
                Object vlu = tmp.get(split[i]);
                Map<String, Object> interMap = vlu == null ? new HashMap<String, Object>() : (Map<String, Object>) vlu;
                tmp.put(split[i], interMap);
                tmp = interMap;
            }
            tmp.put(split[split.length - 1], obj);
            return map;
        } else {
            return null;
        }
    }

    /**
     * 给src追加与target相同的映射关系
     * target的value类型应该为map或jsonarray
     *
     * @param src
     * @param target
     * @param key
     * @return Map<String, Object>
     */
    @SuppressWarnings("unchecked")
    private static Map<String, Object> append(Map<String, Object> src, Map<String, Object> target, String key) {
        Object vlue = src.get(key);
        if (vlue == null) {
            src.put(key, new HashMap<String, Object>(target));
        } else {
            Map<String, Object> interMap = (Map<String, Object>) vlue;
            for (Map.Entry<String, Object> entry : target.entrySet()) {
                Object vlu = entry.getValue();
                if (vlu instanceof Map) {
                    append(interMap, (Map<String, Object>) vlu, entry.getKey());
                } else if (vlu instanceof List && interMap.get(entry.getKey()) instanceof List) {
                    ((List<Object>) interMap.get(entry.getKey())).addAll((List<Object>) vlu);
                } else {
                    //应该为字符串类型
                    interMap.put(entry.getKey(), vlu);
                }
            }
        }
        return src;
    }

    @SuppressWarnings("unchecked")
    public static Map<String, Object> puts(Map<String, Object> map, String path, Map<String, Object> obj) {
        if (obj != null) {
            String[] split = path.split("/");
            Map<String, Object> tmp = map;
            for (int i = 0; i < split.length - 1; i++) {
                Object vlu = tmp.get(split[i]);
                Map<String, Object> interMap = vlu == null ? new HashMap<String, Object>() : (Map<String, Object>) vlu;
                tmp.put(split[i], interMap);
                tmp = interMap;
            }
            String lastKey = split[split.length - 1];
            append(tmp, obj, lastKey);
        }
        return map;
    }

    @SuppressWarnings("unchecked")
    public static <T> T gets(Map<String, Object> map, String path, Class<T> cls) {
        String[] split = path.split("/");
        Map<String, Object> tmp = map;
        for (int i = 0; i < split.length - 1; i++) {
            Object vlu = tmp.get(split[i]);
            if (vlu == null) {
                return null;
            } else {
                tmp = (Map<String, Object>) vlu;
            }
        }
        return cls.cast(tmp.get(split[split.length - 1]));
    }

    @SuppressWarnings("unchecked")
    public static String gets(Map<String, Object> map, String path) {
        return gets(map, path, String.class);
    }

    @SuppressWarnings("unchecked")
    public static Map<String, Object> getsMap(Map<String, Object> map, String path) {
        return gets(map, path, Map.class);
    }

    @SuppressWarnings("unchecked")
    public static Map<String, Object> getAndCreate(Map<String, Object> map, String path) {
        String[] split = path.split("/");
        Map<String, Object> tmp = map;
        for (int i = 0; i < split.length; i++) {
            Object vlu = tmp.get(split[i]);
            Map<String, Object> interMap = vlu == null ? new HashMap<String, Object>() : (Map<String, Object>) vlu;
            tmp.put(split[i], interMap);
            tmp = interMap;
        }
        return tmp;
    }

    @SuppressWarnings("unchecked")
    public static Map<String, Object> getAndCreateNotPut(Map<String, Object> map, String path) {
        Object vlu = map.get(path);
        return vlu == null ? new HashMap<String, Object>() : (Map<String, Object>) vlu;
    }

    /**
     * 创建一个新的map并在指定路径的key下以先获取再赋值的方式添加value值
     * 返回操作完成后的map
     * 由调用者确保key路径每个节点都是map类型
     *
     * @param path 以”/“为分隔符的key路径
     * @param obj  待添加的value值
     * @return Map<String, Object>
     */
    public static Map<String, Object> puts(String path, Object obj) {
        Map<String, Object> interMap = new HashMap<String, Object>();
        return puts(interMap, path, obj);
    }

    /**
     * 给map在指定路径的key下以先获取再赋值的方式添加一个JSONArray
     * 返回创建的JSONArray
     * 由调用者确保key路径每个中间节点都是map类型
     *
     * @param map  待操作的map
     * @param path 以”/“为分隔符的key路径
     * @param obj  待添加的value值
     * @return List<Object>
     */
    @SuppressWarnings("unchecked")
    public static List<Object> adds(Map<String, Object> map, String path, Object obj) {
        String[] split = path.split("/");
        Map<String, Object> tmp = map;
        for (int i = 0; i < split.length - 1; i++) {
            Object vlu = tmp.get(split[i]);
            Map<String, Object> interMap = vlu == null ? new HashMap<String, Object>() : (Map<String, Object>) vlu;
            tmp.put(split[i], interMap);
            tmp = interMap;
        }
        String lastKey = split[split.length - 1];
        Object lastValue = tmp.get(lastKey);
        JSONArray jsonArray = lastValue == null ? new JSONArray() : (JSONArray) lastValue;
        jsonArray.add(obj);
        tmp.put(lastKey, jsonArray);
        return jsonArray;
    }

    /**
     * 创建一个新的map并在指定路径的key下以先获取再赋值的方式添加一个JSONArray
     * 返回创建的JSONArray
     * 由调用者确保key路径每个中间节点都是map类型
     *
     * @param path 以”/“为分隔符的key路径
     * @param obj  待添加的value值
     * @return List<Object>
     */
    public static Map<String, Object> adds(String path, Object obj) {
        Map<String, Object> interMap = new HashMap<String, Object>();
        adds(interMap, path, obj);
        return interMap;
    }
}
